#!/usr/bin/perl -w
#
# Library to parse XML inline annotated documents to a data structure suitable for writing standoff annotation
#
# Usage:	
#	$0 <file.xml> ... > annotations.tsv
#	cat ... | $0 > annotations.tsv
# 
# Author:   Illes Solt
# Created:  Mar 12 2010


package Illes;

use strict;

## use module

use XML::Parser;
use XML::Smart::Parser; # for not so bad xml

# DO NOT USE XML::Smart::HTMLParser as it does not reliably call char_handler() leading to incorrect offsets
# use XML::Smart::HTMLParser; # for bad xml

use File::Basename; # basename()
use Data::Dumper; $Data::Dumper::Sortkeys = 1;

## sub

sub deep_copy;
# File name or file handle
sub parse($);
sub parse_bad($);
sub parse_smart($);



###################
## globals

my @tag_stack; 
my %tags;
my $offset;

######################
## parse documents XML

# 
# 
# ## open & parse docs XML file
# 
# my $file = ($ARGV[0] or '-');
# if ($file ne '-')
# {
# 	for my $file (@ARGV)
# 	{
# 		die "Can't find file \"$file\""
# 			unless -f $file;
# 	
# 		print STDERR "Processing '$file'...\n";
# 
# 		## XML::Parser
# 		# $parser->parsefile($file); 
# 
# 		## XML::Smart:HTMLParser
# 		my $data = read_file($file); 
# 		$parser->parse($data);
# 
# 		print Dumper(\%tags);
# 	}
# }
# else
# {
# 	print STDERR "Processing <STDIN>...\n";
# 	## XML::Parser
# 	# $parser->parse(\*STDIN);
# 
# 	## XML::Smart:HTMLParser
# 	my $data = read_file(\*STDIN);
# 	$parser->parse($data);
# 	print Dumper(\%tags);
# }


## handler subs

sub char_handler
{
	my ($p, $data) = @_;

	$tag_stack[$#tag_stack]->{'cdata'} .= $data;

	$offset += length($data);

# 	print $data;

}  # End of char_handler

# sub default_handler
# {
#	 my ($p, $data) = @_;
# 
#	 if ($data =~ /^<!--/)
#	 {
#	   my $line = $p->current_line;
#	   $data =~ s/\n/\n\t/g;
#	   print "$line:\t$data\n";
#	   $count++;
#	 }
# 
# }  # End of default_handler

sub start_handler
{
	my $expat = shift;
	my $element = shift; # element is the name of the tag

	my %attrs = ();
	while (@_)
	{
		my $att = shift;
		my $val = shift;

		$attrs{$att} = $val;
	}
	
	push(@tag_stack, 
		{
			'name' => $element,
			'attrs' =>  \%attrs,
			'begin_offset' => $offset,
		});
}

sub end_handler
{
	my $expat = shift;
	my $element = shift;
	
	my $start_tag = pop(@tag_stack);
	# push($tag_stack[$#tag_stack]->{'children'}) if scalar(@tag_stack);
	$start_tag->{'end_offset'} = $offset;

	# propagate cdata to ancestors
	$tag_stack[$#tag_stack]->{'cdata'} .= $start_tag->{'cdata'}
		if scalar(@tag_stack) && defined $start_tag->{'cdata'};

	push(@{$tags{$start_tag->{'name'}}}, $start_tag);
}



# sub parse_bad($)
# {
# 	my $file = shift;
# 
# 	# my $parser = new XML::Parser(ErrorContext => 2);
# 	my $parser = new XML::Smart::HTMLParser(); # bad xml
# 	init_parser($parser);
# 
# 	my $data = read_file($file); 
# 	$parser->parse($data);
# # 	print Dumper(\%tags);
# 
# 	return deep_copy(\%tags);
# }

sub parse_smart($)
{
	my $file = shift;

	my $parser = new XML::Smart::Parser();
	init_parser($parser);

	open FILE, $file;
	binmode FILE, ":utf8";
	my $data = join("", <FILE>);
	close FILE;
	$parser->parse($data);
# 	print Dumper(\%tags);

	return deep_copy(\%tags);
}

sub parse($)
{
	my $file = shift;

	
	my $parser = new XML::Parser(ErrorContext => 2);
	init_parser($parser);

	%tags = ();
	if (-f $file) 
	{
		$parser->parsefile($file); 
	} 
	else
	{
		$parser->parse($file); 
	}
	return deep_copy(\%tags);
}

sub init_parser ($)
{
	my $parser = shift;
	$parser->setHandlers(
		Char	=> \&char_handler,
		# Default => \&default_handler,
		Start   => \&start_handler,
		End	=> \&end_handler);

	@tag_stack = (); # 
	%tags = ();
	$offset = 0;
}


sub deep_copy
{
	my $this = shift;
	if (not ref $this) 
	{
		$this;
	}
	elsif (ref $this eq "ARRAY") 
	{
		[map deep_copy($_), @$this];
	}
	elsif (ref $this eq "HASH")
	{
		+{map { $_ => deep_copy($this->{$_}) } keys %$this};
	}
	else 
	{
		die "what type is $_?" ;
	}
}

# suppress warning
1;
