<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	version="1.0">
  <!--
    Description:
    Input:
		Expects an Airola format XML file. Adds an empty parse, if not already present.

Example: 
 * single file:

	$ xsltproc -stringparam t $TOKENIZER -stringparam p $PARSER add-missing-parse.xsl input.xml

 * apply to all files in current directory, indent XML elemenents, output to directory ./out/:

	 $ ls /tmp/*.xml | xargs -I'{}' bash -c 'xsltproc ... "{}" | xmllint -format - > "out/`basename {}`"'
	    
-->
  <xsl:param name="t"/>
  <xsl:param name="p"/>

  <xsl:output method="xml" indent="yes" encoding="UTF-8" omit-xml-declaration="yes"/>

  <xsl:template match="/">
  <xsl:if test="not($t)">
		  <xsl:message terminate="yes">
			  <xsl:text>Tokenizer not specfied, use: &#10; xsltproc --stringpaparam t '&lt;TOKENIZER>' ...</xsl:text>
		  </xsl:message>
  </xsl:if>
  <xsl:if test="not($p)">
		  <xsl:message terminate="yes">
			  <xsl:text>Parser not specfied, use: &#10; xsltproc --stringpaparam p '&lt;PARSER>' ...</xsl:text>
		  </xsl:message>
  </xsl:if>
    <xsl:copy>
      <xsl:apply-templates select="@*|node()"/>
    </xsl:copy>
  </xsl:template>

  <!-- catch parent -->
  <xsl:template match="sentenceanalyses">
    <xsl:copy>
	  <xsl:if test="not(tokenizations/tokenization[@tokenizer=$t])">
		  <xsl:message terminate="yes">
			  <xsl:text>Missing tokenizer: </xsl:text>
			  <xsl:value-of select="$t"/>
			  <xsl:text>|</xsl:text>
			  <xsl:value-of select="$p"/>
		  </xsl:message>
      </xsl:if>
      <!-- copy merged parses elements, if any -->
	  <xsl:if test="not(parses/parse[@tokenizer=$t and @parser=$p])">
		  <xsl:if test="not(parses)">
          <parses>
			<xsl:call-template name="add-parse"/>
	      </parses>
    	</xsl:if>
      </xsl:if>
      <!-- copy other elements -->
      <xsl:apply-templates select="@*|node()"/>
    </xsl:copy>
  </xsl:template>

  <!-- catch all -->
  <xsl:template match="parses">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()"/>
	  <xsl:if test="not(parse[@tokenizer=$t and @parser=$p])">
		<xsl:call-template name="add-parse"/>
	</xsl:if>
  </xsl:copy>
  </xsl:template>

  <!-- catch all -->
  <xsl:template match="@*|node()">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()"/>
    </xsl:copy>
</xsl:template>

<xsl:template name="add-parse">
			<parse parser="{$p}" tokenizer="{$t}"/>
</xsl:template>


</xsl:stylesheet>
