#!/usr/bin/perl -w
#
# Script to convert XML inline annotated documents to standoff annotation
#
# Usage:	$0 <file.xml>
# 
# Author:   Illes Solt
# Created:  Mar 12 2010


use strict;

package Illes;


## include

push(@INC, '..');
push(@INC, 'XML-Smart-1.6.9/lib/');
require 'XMLAnnotationParser.pl';

binmode STDIN,  ":utf8";
binmode STDOUT, ":utf8";
binmode STDERR, ":utf8";

my %sep = 
( 
	'line'	=> "\n",
	'col'	=> "\t",
	'attr'	=> '|',
);


## sub

# sub deep_copy;
sub print_annotation($$);
sub print_header();
sub normalize_whitespace($);


## open & parse docs XML file

my $file = ($ARGV[0] or '-');
if ($file ne '-')
{
	for my $file (@ARGV)
	{
		die "Can't find file \"$file\""
			unless -f $file;
	
		print STDERR "Processing '$file'...\n";
		my $annotation = parse_smart($file);
		print_annotation($annotation, $file);
# 		print Dumper($annotation);
	}
}
else
{
	die "Pipe mode unsupported.";
	print STDERR "Processing <STDIN>...\n";
	my $annotation = parse_smart(\*STDIN);
	print_annotation($annotation, 'STDIN');
# 	print Dumper($annotation);
}

sub print_annotation($$)
{
	my ($a,$f) = @_;

	print_header();

	while (my ($name, $tags) = each(%$a))
	{
		for my $tag (@$tags)
		{
			my @l = (
				$f,
				$tag->{'name'},
				$tag->{'begin_offset'},
				$tag->{'end_offset'},
				join($sep{'attr'}, map { $_.'='.$tag->{'attrs'}->{$_} } keys %{$tag->{'attrs'}}),
				$tag->{'cdata'}
			);
			print join($sep{'col'}, map { normalize_whitespace($_) } @l).$sep{'line'};
		}
	}
}

sub print_header()
{
	print join($sep{'col'}, ('#file', 'element', 'cdata_begin', 'cdata_end', 'normalized_attrs', 'normalized_content')).$sep{'line'};
}

sub normalize_whitespace($)
{
	my $s = shift;
	return '' unless defined $s;
	$s =~ s/[\n\t]/ /g;
# 	$s =~ s/^\s+//;
# 	$s =~ s/\s+$//;
	return $s;
}
