drop function if exists get_statistics(Integer);
create or replace function get_statistics(expid Integer) returns setof extendedstats as 
'select 
	calculateextendedstats(newStats(
	sum(case when a.output=TRUE and p.GroundTruth = a.output then 1 else 0 end),
	sum(case when a.output=TRUE and p.GroundTruth != a.output then 1 else 0 end),
	sum(case when a.output=FALSE and p.GroundTruth != a.output then 1 else 0 end),
	sum(case when a.output=FALSE and p.GroundTruth = a.output then 1 else 0 end)
	))
from ppiCVoutput a
inner join pair p on a.pair = p.pairLabel
where a.expid=$1
-- group by p.pairLabel, p.GroundTruth
;'
language SQL stable returns null on null input;
comment on function get_statistics(Integer)
is 'Calculates stats(tp,fp,fn,tn) for an expid(CV)';

drop function if exists cl_get_statistics(Integer);
create or replace function cl_get_statistics(expid Integer) returns setof extendedstats as 
'select 
	calculateextendedstats(newStats(
	sum(case when a.output=TRUE and p.GroundTruth = a.output then 1 else 0 end),
	sum(case when a.output=TRUE and p.GroundTruth != a.output then 1 else 0 end),
	sum(case when a.output=FALSE and p.GroundTruth != a.output then 1 else 0 end),
	sum(case when a.output=FALSE and p.GroundTruth = a.output then 1 else 0 end)
	))
from ppiCLoutput a
inner join pair p on a.pair = p.pairLabel
where a.expid=$1
-- group by p.pairLabel, p.GroundTruth
;'
language SQL stable returns null on null input;
comment on function cl_get_statistics(Integer)
is 'Calculates stats(tp,fp,fn,tn) for an expid(CL)';


drop function if exists cc_get_statistics(Integer);
create or replace function cc_get_statistics(expid Integer) returns setof extendedstats as 
'select 
	calculateextendedstats(newStats(
	sum(case when a.output=TRUE and p.GroundTruth = a.output then 1 else 0 end),
	sum(case when a.output=TRUE and p.GroundTruth != a.output then 1 else 0 end),
	sum(case when a.output=FALSE and p.GroundTruth != a.output then 1 else 0 end),
	sum(case when a.output=FALSE and p.GroundTruth = a.output then 1 else 0 end)
	))
from ppiCCoutput a
inner join pair p on a.pair = p.pairLabel
where a.expid=$1
-- group by p.pairLabel, p.GroundTruth
;'
language SQL stable returns null on null input;
comment on function cc_get_statistics(Integer)
is 'Calculates stats(tp,fp,fn,tn) for an expid(CC)';

DROP FUNCTION IF EXISTS combine_systems_and(INT,INT);
CREATE FUNCTION combine_systems_and
(
  ppicvid1 INT,
  ppicvid2 INT
)
RETURNS INT AS
$$
DECLARE
    kernel1 text;
    kernel2 text;
    corpus1 text;
    corpus2 text;
    fold1 integer;
    fold2 integer;
    combinedKernel text;
    myppicvid integer;
BEGIN
  SELECT INTO kernel1 p.kernel FROM ppicv p WHERE ppicvid=ppicvid1;
  SELECT INTO kernel2 p.kernel FROM ppicv p WHERE ppicvid=ppicvid2;

  SELECT INTO corpus1 p.corpus FROM ppicv p WHERE ppicvid=ppicvid1;
  SELECT INTO corpus2 p.corpus FROM ppicv p WHERE ppicvid=ppicvid2;
  IF corpus1 != corpus2 THEN RAISE EXCEPTION 'corpora must be equal';
	END IF;

  SELECT INTO fold1 p.fold FROM ppicv p WHERE ppicvid=ppicvid1;
  SELECT INTO fold2 p.fold FROM ppicv p WHERE ppicvid=ppicvid2;
  IF fold1 != fold2 THEN RAISE EXCEPTION 'folds must be equal';
	END IF;
  
  SELECT INTO combinedKernel (SELECT 'combined' || '(' || kernel1 || '&' || kernel2 || ')');
  
  IF EXISTS (SELECT ppicvid FROM ppicv WHERE kernel = combinedKernel and corpus = corpus1 and fold = fold1) 
    THEN RETURN (SELECT ppicvid FROM ppicv WHERE kernel = combinedKernel and corpus = corpus1 and fold = fold1 limit 1); /* already exists */
  END IF;
  
  INSERT INTO ppicv (corpus,parsertype,parser,kernel, fold, normalized,c,kernel_script) VALUES 
	(corpus1, 'mixed','mixed',combinedKernel,fold1,'t',0,'mixed(2)');
  SELECT INTO myppicvid currval('ppiCV_ppiCVid_seq');
  INSERT INTO
    ppicvoutput (expId, pair, output, prediction)
  SELECT
    myppicvid         as expId, 
    o1.pair	      as pair,
    (CASE WHEN o1.output=TRUE and o2.output=TRUE then TRUE else FALSE end) as output,
    o1.prediction+o2.prediction	as prediction
  FROM
    ppicvoutput o1
  INNER JOIN
    ppicvoutput o2
      ON o1.pair = o2.pair
  WHERE o1.expid = ppicvid1
    AND o2.expid = ppicvid2
  ;

  UPDATE ppicv set 
	tp=s.tp,
	fp=s.fp, 
	fn=s.fn, 
	tn=s.tn,
	total=s.n,
	precision_=s.p,
	recall=s.r,
	f_measure=s.f
  FROM get_statistics(myppicvid) as s
  where ppicvid=myppicvid;
  RETURN myppicvid; /* just created */
END
$$
LANGUAGE 'plpgsql'
;
comment on function combine_systems_and(INT,INT) is 
'Combines with logical AND two CV experiments given their ppicvid-s
Adds a new row in ppicv
Adds respective AND results in ppicvoutput
Fills in all preformance value in the newly added row of ppicv';

-- test
-- 
-- select combine_systems_and(exp1,exp2) from 
-- (select p1.ppicvid as exp1,p2.ppicvid as exp2 from Best_CV_SL p1, Best_CV_APG p2
-- where p1.fold=p2.fold and p1.corpus=p2.corpus and p1.corpus='LLL') as a
-- ;


DROP FUNCTION IF EXISTS combine_and(INT[],double precision,Text);
CREATE FUNCTION combine_and
(
  IN ppicvids INT[],
  posthrehold double precision,
  kernel_text Text
)
RETURNS INT AS
$$
DECLARE
    numExperiments INT;
    mycorpus text;
    myfold INT;    
    combinedKernel text;
    myppicvid integer;

    tmp text[];
BEGIN
  numExperiments := array_upper(ppicvids,1);

  IF numExperiments = ( SELECT count(*) from ppicv where ppicvid=ANY(ppicvids) group by corpus )
  THEN
     mycorpus := ( SELECT corpus from ppicv where ppicvid=ANY(ppicvids) group by corpus );
  ELSE
    RAISE EXCEPTION 'corpora must be equal';
  END IF;

  IF numExperiments = ( SELECT count(*) from ppicv where ppicvid=ANY(ppicvids) group by fold )
  THEN
     myfold := ( SELECT fold from ppicv where ppicvid=ANY(ppicvids) group by fold );
  ELSE
    RAISE EXCEPTION 'folds must be equal';
  END IF;

 
   FOR currentID IN 1..array_upper(ppicvids,1) LOOP
	tmp[currentID] := ( SELECT kernel from ppicv where ppicvid=ppicvids[currentID]);
   END LOOP;
   combinedKernel := $3 || cast($2 as Text);
   
  IF EXISTS (SELECT ppicvid FROM ppicv WHERE kernel = combinedKernel and corpus = mycorpus and fold = myfold) 
    THEN RETURN (SELECT ppicvid FROM ppicv WHERE kernel = combinedKernel and corpus = mycorpus and fold = myfold); /* already exists */
  END IF;

  INSERT INTO ppicv (corpus,parsertype,parser,kernel, fold, normalized,c,kernel_script) VALUES 
	(mycorpus, 'mixed','mixed',combinedKernel,myfold,'t',0,'mixed');
  SELECT INTO myppicvid currval('ppiCV_ppiCVid_seq');

  INSERT INTO
    ppicvoutput (expId, pair, output, prediction)
  SELECT
    myppicvid         as expId, 
    sub.pair	      as pair,
    -- (CASE WHEN positiveExperiments >= cast ( numExperiments/2 as double precision ) THEN TRUE ELSE FALSE END) as output,
    (CASE WHEN positiveExperiments >= cast ( numExperiments*$2 as double precision ) THEN TRUE ELSE FALSE END) as output,
    sumPrediction as prediction
  FROM
  (SELECT
     SUM((CASE WHEN output = TRUE THEN 1 ELSE 0 END)) AS positiveExperiments,
     sum(prediction) as sumPrediction,
     pair
     FROM
    ppiCVoutput
    WHERE expid = ANY(ppicvids)
   GROUP BY pair
   ) as sub
;

  UPDATE ppicv set 
	tp=s.tp,
	fp=s.fp, 
	fn=s.fn, 
	tn=s.tn,
	total=s.n,
	precision_=s.p,
	recall=s.r,
	f_measure=s.f
  FROM get_statistics(myppicvid) as s
  where ppicvid=myppicvid;

  RETURN myppicvid; /* just created */
  
END
$$
LANGUAGE 'plpgsql'
;
comment on function combine_and(INT[],double precision,Text) is 
'Combines with logical AND several CV experiments given their ppicvid-s
posthreshold is the ratio of min positive prediction that is needed for an ensemble pos prediction, e.g 1/2, 5/13
$3 is the name of the combination (kernel)';

DROP FUNCTION IF EXISTS combine_and(INT[]);
CREATE FUNCTION combine_and
(
  IN ppicvids INT[]
)
RETURNS INT AS
$$
DECLARE
    numExperiments INT;
    mycorpus text;
    myfold INT;    
    combinedKernel text;
    myppicvid integer;

    tmp text[];
BEGIN
  numExperiments := array_upper(ppicvids,1);

  IF numExperiments = ( SELECT count(*) from ppicv where ppicvid=ANY(ppicvids) group by corpus )
  THEN
     mycorpus := ( SELECT corpus from ppicv where ppicvid=ANY(ppicvids) group by corpus );
  ELSE
    RAISE EXCEPTION 'corpora must be equal';
  END IF;

  IF numExperiments = ( SELECT count(*) from ppicv where ppicvid=ANY(ppicvids) group by fold )
  THEN
     myfold := ( SELECT fold from ppicv where ppicvid=ANY(ppicvids) group by fold );
  ELSE
    RAISE EXCEPTION 'folds must be equal';
  END IF;

 
   FOR currentID IN 1..array_upper(ppicvids,1) LOOP
	tmp[currentID] := ( SELECT kernel from ppicv where ppicvid=ppicvids[currentID]);
   END LOOP;
   combinedKernel := 'combined(' || array_to_string(tmp,'&') || ')';
   
  IF EXISTS (SELECT ppicvid FROM ppicv WHERE kernel = combinedKernel and corpus = mycorpus and fold = myfold) 
    THEN RETURN (SELECT ppicvid FROM ppicv WHERE kernel = combinedKernel and corpus = mycorpus and fold = myfold); /* already exists */
  END IF;

  INSERT INTO ppicv (corpus,parsertype,parser,kernel, fold, normalized,c,kernel_script) VALUES 
	(mycorpus, 'mixed','mixed',combinedKernel,myfold,'t',0,'mixed');
  SELECT INTO myppicvid currval('ppiCV_ppiCVid_seq');

  INSERT INTO
    ppicvoutput (expId, pair, output, prediction)
  SELECT
    myppicvid         as expId, 
    sub.pair	      as pair,
    (CASE WHEN positiveExperiments >= cast ( numExperiments/2.0 as double precision ) THEN TRUE ELSE FALSE END) as output,
    sumPrediction as prediction
  FROM
  (SELECT
     SUM((CASE WHEN output = TRUE THEN 1 ELSE 0 END)) AS positiveExperiments,
     sum(prediction) as sumPrediction,
     pair
     FROM
    ppiCVoutput
    WHERE expid = ANY(ppicvids)
   GROUP BY pair
   ) as sub
;

  UPDATE ppicv set 
	tp=s.tp,
	fp=s.fp, 
	fn=s.fn, 
	tn=s.tn,
	total=s.n,
	precision_=s.p,
	recall=s.r,
	f_measure=s.f
  FROM get_statistics(myppicvid) as s
  where ppicvid=myppicvid;

  RETURN myppicvid; /* just created */
  
END
$$
LANGUAGE 'plpgsql'
;
comment on function combine_and(INT[]) is 
'Combines with logical AND several CV experiments given their ppicvid-s';


DROP FUNCTION IF EXISTS combine_majority(INT[]);
CREATE FUNCTION combine_majority
(
  IN ppicvids INT[]
)
RETURNS INT AS
$$
DECLARE
    numExperiments INT;
    mycorpus text;
    myfold INT;    
    combinedKernel text;
    combinedKernelScript text;
    myppicvid integer;

    tmp text[];
BEGIN
  numExperiments := array_upper(ppicvids,1);

  IF numExperiments != ( SELECT count(*) from ppicv where ppicvid=ANY(ppicvids) group by corpus )
  THEN
    RAISE EXCEPTION 'invalid ppicvids';
  END IF;

  IF 1 = ( SELECT COUNT(DISTINCT corpus) from ppicv where ppicvid=ANY(ppicvids))
  THEN
     mycorpus := ( SELECT DISTINCT corpus from ppicv where ppicvid=ANY(ppicvids));
  ELSE
    RAISE EXCEPTION 'corpora must be equal';
  END IF;

  IF 1 = ( SELECT count(distinct fold) from ppicv where ppicvid=ANY(ppicvids) )
  THEN
     myfold := ( SELECT DISTINCT fold from ppicv where ppicvid=ANY(ppicvids));
  ELSE
    RAISE EXCEPTION 'folds must be equal';
  END IF;

 
   --FOR currentID IN 1..array_upper(ppicvids,1) LOOP
--	tmp[currentID] := ( SELECT kernel || '(' || kernel_script || ')' from ppicv where ppicvid=ppicvids[currentID] ORDER BY  kernel, kernel_script);
--   END LOOP;
   tmp := ( SELECT array_agg(description) from (SELECT kernel as description from ppicv where ppicvid = ANY(ppicvids) 	ORDER BY  kernel, kernel_script) t);
   combinedKernel := 'combined_majority{' || array_to_string(tmp,'|') || '}';

   tmp := ( SELECT array_agg(description) from (SELECT kernel || '(' || kernel_script || ')' as description from ppicv where ppicvid = ANY(ppicvids) 	ORDER BY  kernel, kernel_script) t);
   combinedKernelScript := '{' || array_to_string(tmp,'|') || '}';
   
  IF EXISTS (SELECT ppicvid FROM ppicv WHERE kernel = combinedKernel and corpus = mycorpus and fold = myfold and kernel_script = combinedKernelScript) 
    THEN RETURN (SELECT ppicvid FROM ppicv WHERE kernel = combinedKernel and corpus = mycorpus and fold = myfold and kernel_script = combinedKernelScript); /* already exists */
  END IF;

  INSERT INTO ppicv (corpus,parsertype,parser,kernel, fold, normalized,c,kernel_script) VALUES 
	(mycorpus, 'mixed','mixed',combinedKernel,myfold,'t',0,combinedKernelScript);
  SELECT INTO myppicvid currval('ppiCV_ppiCVid_seq');

  INSERT INTO
    ppicvoutput (expId, pair, output, prediction)
  SELECT
    myppicvid         as expId, 
    sub.pair	      as pair,
    (positiveExperiments > numExperiments/2) as output,
    sumPrediction as prediction
  FROM
  (SELECT
     SUM((CASE WHEN output = TRUE THEN 1 ELSE 0 END)) AS positiveExperiments,
     sum(prediction) as sumPrediction,
     pair
     FROM
    ppiCVoutput
    WHERE expid = ANY(ppicvids)
   GROUP BY pair
   ) as sub
;

  UPDATE ppicv set 
	tp=s.tp,
	fp=s.fp, 
	fn=s.fn, 
	tn=s.tn,
	total=s.n,
	precision_=s.p,
	recall=s.r,
	f_measure=s.f
  FROM get_statistics(myppicvid) as s
  where ppicvid=myppicvid;

  RETURN myppicvid; /* just created */
  
END
$$
LANGUAGE 'plpgsql'
;
comment on function combine_majority(INT[]) is 
'Combines with majority vote several CV experiments given their ppicvid-s';

-- delete from ppicv where kernel_script='mixed';


DROP FUNCTION IF EXISTS combine_majority_cl(INT[]);
CREATE FUNCTION combine_majority_cl
(
  IN ppiclids INT[]
)
RETURNS INT AS
$$
DECLARE
    numExperiments INT;
    mycorpus text;
    combinedKernel text;
    combinedKernelScript text;
    myppiclid integer;

    tmp text[];
BEGIN
  numExperiments := array_upper(ppiclids,1);

  IF numExperiments != ( SELECT count(*) from ppicl where ppiclid=ANY(ppiclids) group by corpus )
  THEN
    RAISE EXCEPTION 'invalid ppiclids';
  END IF;

  IF 1 = ( SELECT COUNT(DISTINCT corpus) from ppicl where ppiclid=ANY(ppiclids))
  THEN
     mycorpus := ( SELECT DISTINCT corpus from ppicl where ppiclid=ANY(ppiclids));
  ELSE
    RAISE EXCEPTION 'corpora must be equal';
  END IF;

   --FOR currentID IN 1..array_upper(ppicvids,1) LOOP
--	tmp[currentID] := ( SELECT kernel || '(' || kernel_script || ')' from ppicv where ppicvid=ppicvids[currentID] ORDER BY  kernel, kernel_script);
--   END LOOP;
   tmp := ( SELECT array_agg(description) from (SELECT kernel as description from ppicl where ppiclid = ANY(ppiclids) 	ORDER BY  kernel, kernel_script) t);
   combinedKernel := 'combined_majority{' || array_to_string(tmp,'|') || '}';

   tmp := ( SELECT array_agg(description) from (SELECT kernel || '(' || kernel_script || ')' as description from ppicl where ppiclid = ANY(ppiclids) 	ORDER BY  kernel, kernel_script) t);
   combinedKernelScript := '{' || array_to_string(tmp,'|') || '}';
   
  IF EXISTS (SELECT ppiclid FROM ppicl WHERE kernel = combinedKernel and corpus = mycorpus and kernel_script = combinedKernelScript) 
    THEN RETURN (SELECT ppiclid FROM ppicl WHERE kernel = combinedKernel and corpus = mycorpus and kernel_script = combinedKernelScript); /* already exists */
    --THEN RAISE EXCEPTION 'already exists';
  END IF;

  INSERT INTO ppicl (corpus,parsertype,parser,kernel, normalized,c,kernel_script) VALUES 
	(mycorpus, 'mixed','mixed',combinedKernel,'t',0,combinedKernelScript);
  SELECT INTO myppiclid currval('ppiCL_ppiCLid_seq');

  INSERT INTO
    ppicloutput (expId, pair, output, prediction)
  SELECT
    myppiclid         as expId, 
    sub.pair	      as pair,
    (positiveExperiments > numExperiments/2) as output,
    sumPrediction as prediction
  FROM
  (SELECT
     SUM((CASE WHEN output = TRUE THEN 1 ELSE 0 END)) AS positiveExperiments,
     sum(prediction) as sumPrediction,
     pair
     FROM
    ppiCLoutput
    WHERE expid = ANY(ppiclids)
   GROUP BY pair
   ) as sub
;

  UPDATE ppicl set 
	tp=s.tp,
	fp=s.fp, 
	fn=s.fn, 
	tn=s.tn,
	total=s.n,
	precision_=s.p,
	recall=s.r,
	f_measure=s.f
  FROM cl_get_statistics(myppiclid) as s
  where ppiclid=myppiclid;

  RETURN myppiclid; /* just created */
  
END
$$
LANGUAGE 'plpgsql'
;
comment on function combine_majority_cl(INT[]) is 
'Combines with majority vote several CL experiments given their ppiclid-s';

DROP FUNCTION IF EXISTS combine_majority_cc(INT[]);
CREATE FUNCTION combine_majority_cc
(
  IN ppiccids INT[]
)
RETURNS INT AS
$$
DECLARE
    numExperiments INT;
    mycorpus text;
    mytest text;
    combinedKernel text;
    combinedKernelScript text;
    myppiccid integer;

    tmp text[];
BEGIN
  numExperiments := array_upper(ppiccids,1);

  IF numExperiments != ( SELECT count(*) from ppicc where ppiccid=ANY(ppiccids) group by corpus )
  THEN
    RAISE EXCEPTION 'invalid ppiccids';
  END IF;

  IF 1 = ( SELECT COUNT(DISTINCT test) from ppicc where ppiccid=ANY(ppiccids))
  THEN
     mytest := ( SELECT DISTINCT test from ppicc where ppiccid=ANY(ppiccids));
  ELSE
    RAISE EXCEPTION 'test corpora must be equal';
  END IF;
  
  IF 1 = ( SELECT COUNT(DISTINCT corpus) from ppicc where ppiccid=ANY(ppiccids))
  THEN
     mycorpus := ( SELECT DISTINCT corpus from ppicc where ppiccid=ANY(ppiccids));
  ELSE
    RAISE EXCEPTION 'corpora must be equal';
  END IF;

   --FOR currentID IN 1..array_upper(ppicvids,1) LOOP
--  tmp[currentID] := ( SELECT kernel || '(' || kernel_script || ')' from ppicv where ppicvid=ppicvids[currentID] ORDER BY  kernel, kernel_script);
--   END LOOP;
   tmp := ( SELECT array_agg(description) from (SELECT kernel as description from ppicc where ppiccid = ANY(ppiccids)   ORDER BY  kernel, kernel_script) t);
   combinedKernel := 'combined_majority{' || array_to_string(tmp,'|') || '}';

   tmp := ( SELECT array_agg(description) from (SELECT kernel || '(' || kernel_script || ')' as description from ppicc where ppiccid = ANY(ppiccids)    ORDER BY  kernel, kernel_script) t);
   combinedKernelScript := '{' || array_to_string(tmp,'|') || '}';
   
  IF EXISTS (SELECT ppiccid FROM ppicc WHERE kernel = combinedKernel and corpus = mycorpus and test = mytest) 
    --THEN RETURN (SELECT ppicvid FROM ppicv WHERE kernel = combinedKernel and corpus = mycorpus and fold = myfold); /* already exists */
    THEN RAISE EXCEPTION 'already exists';
  END IF;

  INSERT INTO ppicc (corpus,test,parsertype,parser,kernel, normalized,c,kernel_script) VALUES 
    (mycorpus, mytest, 'mixed','mixed',combinedKernel,'t',0,combinedKernelScript);
  SELECT INTO myppiccid currval('ppiCC_ppiCCid_seq');

  INSERT INTO
    ppiccoutput (expId, pair, output, prediction)
  SELECT
    myppiccid         as expId, 
    sub.pair          as pair,
    (positiveExperiments > numExperiments/2) as output,
    sumPrediction as prediction
  FROM
  (SELECT
     SUM((CASE WHEN output = TRUE THEN 1 ELSE 0 END)) AS positiveExperiments,
     sum(prediction) as sumPrediction,
     pair
     FROM
    ppiCCoutput
    WHERE expid = ANY(ppiccids)
   GROUP BY pair
   ) as sub
;

  UPDATE ppicc set 
    tp=s.tp,
    fp=s.fp, 
    fn=s.fn, 
    tn=s.tn,
    total=s.n,
    precision_=s.p,
    recall=s.r,
    f_measure=s.f
  FROM cc_get_statistics(myppiccid) as s
  where ppiccid=myppiccid;

  RETURN myppiccid; /* just created */
  
END
$$
LANGUAGE 'plpgsql'
;
comment on function combine_majority_cc(INT[]) is 
'Combines with majority vote several CC experiments given their ppiccid-s';

-- select combine_and(exp1,exp2) from 
-- (select p1.ppicvid as exp1,p2.ppicvid as exp2 from Best_CV_SL p1, Best_CV_APG p2
-- where p1.fold=p2.fold and p1.corpus=p2.corpus and p1.corpus='LLL') as a
-- ;
-- 

-- select combine_and('{77585,78115,16321,77694}');
-- 
-- select * from ppicv where kernel_script='mixed';
-- 
-- delete from ppicv where kernel LIKE 'combined(%';

-- select p.fp from get_statistics(78243) as p;

-- select avg (precision_) as p, avg(recall) as r, avg (f_measure) as f from ppicv where kernel LIKE 'combined(%';
