DROP TYPE IF EXISTS EAsentence CASCADE;
CREATE TYPE EAsentence AS 
(
    pairLabel TEXT,
    GT TEXT,
    sentence Text,
    blinded TEXT    
);



CREATE OR REPLACE FUNCTION string_to_rows(text) RETURNS SETOF TEXT AS $$ 
  DECLARE
    elems text[];      
  BEGIN
    elems := string_to_array($1, ' ');
    FOR i IN array_lower(elems, 1) .. array_upper(elems, 1) LOOP
      RETURN NEXT elems[i];
    END LOOP;
    RETURN;
  END
$$ LANGUAGE 'plpgsql';


drop function if exists select_sentences(Text,Text) cascade;
create or replace function select_sentences(hardness Text, GT Text) returns setof EAsentence as 
$$
select pairLabel,GT,s.rawtext,blinded from PosNegNeutral p 
join sentence s on p.sentenceLabel = s.label
where hardness ~ $1 and GT=$2
$$ LANGUAGE 'SQL'
;

drop view if exists blinded_sentence_view;
create or replace view blinded_sentence_view as 
select * from select_sentences('[D]','FALSE');

-- drop function if exists select_sentences_upper(Text,Text) cascade;
-- create or replace function select_sentences_upper(hardness Text, GT Text) returns setof EAsentence as 
-- $$
-- select pairLabel,GT,blinded from PosNegNeutral where hardness=$1 and GT=$2
-- $$ LANGUAGE 'SQL'
-- ;
-- 

-- test
-- select * from blinded_sentence_view;

CREATE OR REPLACE FUNCTION string_to_morerows() RETURNS SETOF TEXT AS $$ 
  DECLARE
    elems text[];
    s text;
  BEGIN
    FOR s IN SELECT 
    distinct -- comment this for all non-unique sentences
    sentence as sentence from blinded_sentence_view LOOP
        elems := string_to_array(lower(s), ' ');
	FOR j IN array_lower(elems, 1) .. array_upper(elems, 1) LOOP
	    RETURN NEXT btrim(elems[j],'.,()<>/;?!');
        END LOOP;
    END LOOP;
    RETURN;
  END
$$ LANGUAGE 'plpgsql';
comment on function string_to_morerows()
is 'Returns the tokens of the view blinded_sentence_view; tokens are trimmed; replace the view to generate new version';
-- 
-- 

drop type if exists tokensupport cascade;

create type tokensupport as (
	token text,
	support numeric
);


create or replace view token_posnegneutral as
select * from sentencetoken s
join PosNegNeutral p on s.sentence=p.sentencelabel
;

drop function if exists token_hardnessGT_support(text,text);

create or replace function token_hardnessGT_support(hardness text, GT text) returns setof tokensupport as 
$$
select tokentext,cast(count(*) as numeric) from
(select distinct sentence,position,tokentext from token_posnegneutral
where hardness ~ $1 and GT ~ $2) as t
group by tokentext
$$ language SQL 
stable returns null on null input;

drop table if exists negation_words;
create table negation_words as
select distinct token from token_hardnessGT_support('[DNE]', 'FALSE|TRUE')
where token ~ '^not?$' 
or token ~ '^(neither|nor)$'
or token ~ '^without$' 
or token ~ '^but$' 
or token ~ '^den[yi][esding]*$'
or token ~ '^never$'
or token ~ '^unlike(ly)?$';
alter table negation_words add constraint negation_words_p_key primary key (token);

drop table if exists hedge_words;
create table hedge_words as
select distinct token from token_hardnessGT_support('[DNE]', 'FALSE|TRUE')
where token ~ '^(may|might|would|could)$' 
-- or token ~ '^indicat[esding]*$' 
-- or token ~ '^suggest[esding]*$' 
-- or token ~ '^hint[esding]*$' 
or token ~ '^likely$' 
or token ~ '^possib' 
-- or token ~ '^suspect[esding]*$'
-- or token ~ '^appear[esding]*$'
or token ~ '^presum'
or token ~ '^probab'
or token ~ '^unknown$'
-- or token ~ '^undefined$'
or token ~ '^elucidat[esding]*$'
-- or token ~ '^postulat[esding]*$'
-- or token ~ '^seem[esding]*$'
or token ~ '^thought$';
alter table hedge_words add constraint hedge_words_p_key primary key (token);

-- select n.token,coalesce(a.support,0) from hedge_words n 
-- left join token_hardnessGT_support('[DNE]', 'FALSE') a on n.token=a.token order by n.token;
-- -- 


