-- This script generates for CV experiment for each corpus the table of each pairs ordered by accuracy 

DROP TYPE IF EXISTS EAtable CASCADE;
CREATE TYPE EAtable AS 
(
    acc NUMERIC,
    pairLabel TEXT,
    GT TEXT,
    n INTEGER,
    tp INTEGER,
    fp INTEGER,
    fn INTEGER,
    tn INTEGER,
    entity1 TEXT,
    entity2 TEXT,
    entityLabel1 TEXT,
    entityLabel2 TEXT,    
    be1 INTEGER,
    ee1 INTEGER,
    be2 INTEGER,
    ee2 INTEGER,
    sentenceLabel TEXT,
    sentence TEXT,
    blinded TEXT    
);


drop function if exists EA_CV_pairwise(Text);
create or replace function EA_CV_pairwise(pairLabelPattern Text) returns setof EAtable as 
$$
select case when (t.tp+t.fp+t.fn+t.tn)=0 then rr(0.0) else
            rr((t.tp + t.tn) / CAST ((t.tp + t.fn + t.fp + t.tn) AS DOUBLE PRECISION)) end AS acc,*
             from (
	select p1.pairLabel,case when pr.groundTruth then 'TRUE' else 'FALSE' end as GT, 
	(calculateextendedstats( p1.stat)).n as n,
	(calculateextendedstats( p1.stat)).tp as tp,
	(calculateextendedstats( p1.stat)).fp as fp,
	(calculateextendedstats( p1.stat)).fn as fn,
	(calculateextendedstats( p1.stat)).tn as tn,
	e1.entityText as entity1, e2.entityText as entity2,
	e1.entityLabel as entityLabel1,e2.entityLabel as entityLabel2,
	e1.beginOffset as begin_e1,e1.endOffset as end_e1,e2.beginOffset as begin_e2,e2.endOffset as end_e2,
	s.label as sentenceLabel,
	s.rawText as sentence,
	blinded_sentence(pr.pairLabel) as blinded
	from morepairs_statistics($1) as p1
	inner join pair pr on pr.pairLabel=p1.pairLabel
	inner join entity e1 on e1.entityLabel = pr.entity1
	inner join entity e2 on e2.entityLabel = pr.entity2
	inner join sentence s on s.label = e1.sentence 
	)
as t order by acc ASC;
$$
language SQL stable returns null on null input;
comment on function EA_CV_pairwise(Text)
is 'Calculates the evaluation table for a Corpus given as a pairLabel pattern';

drop function if exists EA_CV_allpairs();
create or replace function EA_CV_allpairs() returns setof EAtable as 
$$
select case when (t.tp+t.fp+t.fn+t.tn)=0 then rr(0.0) else
            rr((t.tp + t.tn) / CAST ((t.tp + t.fn + t.fp + t.tn) AS DOUBLE PRECISION)) end AS acc,*
             from (
	select p1.pairLabel,case when pr.groundTruth then 'TRUE' else 'FALSE' end as GT, 
	(calculateextendedstats( p1.stat)).n as n,
	(calculateextendedstats( p1.stat)).tp as tp,
	(calculateextendedstats( p1.stat)).fp as fp,
	(calculateextendedstats( p1.stat)).fn as fn,
	(calculateextendedstats( p1.stat)).tn as tn,
	e1.entityText as entity1, e2.entityText as entity2,
 	e1.entityLabel as entityLabel1,e2.entityLabel as entityLabel2,
	e1.beginOffset as begin_e1,e1.endOffset as end_e1,e2.beginOffset as begin_e2,e2.endOffset as end_e2,
	s.label as sentenceLabel,
	s.rawText as sentence,
	blinded_sentence(pr.pairLabel) as blinded
	from allpairs_statistics() as p1
	inner join pair pr on pr.pairLabel=p1.pairLabel
	inner join entity e1 on e1.entityLabel = pr.entity1
	inner join entity e2 on e2.entityLabel = pr.entity2
	inner join sentence s on s.label = e1.sentence 
	)
as t order by acc ASC;
$$
language SQL stable returns null on null input;
comment on function EA_CV_allpairs()
is 'Calculates the evaluation table for the entire corpus';

-- 
drop table if exists EA_CV_AIMed cascade;
create table EA_CV_AIMed as 
select * from EA_CV_pairwise('AIMed%');
-- 
drop table if exists EA_CV_BioInfer cascade;
create table EA_CV_BioInfer as 
select * from EA_CV_pairwise('BioInfer%');

drop table if exists EA_CV_HPRD50 cascade;
create table EA_CV_HPRD50 as 
select * from EA_CV_pairwise('HPRD50%');

drop table if exists EA_CV_IEPA cascade;
create table EA_CV_IEPA as 
select * from EA_CV_pairwise('IEPA%');
-- -- 
drop table if exists EA_CV_LLL cascade;
create table EA_CV_LLL as 
select * from EA_CV_pairwise('LLL%');
-- 

drop view if exists EA_CV_ALL cascade;
create or replace view EA_CV_ALL as 
select * from EA_CV_AIMed
union
select * from EA_CV_BioInfer
union
select * from EA_CV_HPRD50
union
select * from EA_CV_IEPA
union
select * from EA_CV_LLL
;