drop function if exists my_f1 (integer);
create or replace function my_f1 (CVexpID integer) returns double precision as 
'-- Calculates the f-measure for a given CV experiment
select 
(case when prec + recall>0 then (2 * prec * recall/( prec + recall )) else 0 end) as f1 from
(Select *,
(case when tp+fp>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fp as double precision))) else 0 end) as prec,
(case when tp+fn>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fn as double precision))) else 0 end) as recall from
(Select a.expid,count(*) as cnt,b.total,
	sum(case when a.output=FALSE and c.groundTruth != a.output then 1 else 0 end) as fn,
	sum(case when a.output=FALSE and c.groundTruth = a.output then 1 else 0 end) as tn,
	sum(case when a.output=TRUE and c.groundTruth != a.output then 1 else 0 end) as fp,
	sum(case when a.output=TRUE and c.groundTruth = a.output then 1 else 0 end) as tp, count(*)
from ppiCVoutput a 
join ppiCV b on a.expID=b.ppiCVid 
join pair c on a.pair = c.pairLabel where expid=$1 group by a.expid, b.total) as sub1) as sub2
;' language SQL stable returns null on null input;
comment on function my_f1 (integer) is 
'Calculates the f-measure for a given CV experiment';

drop function if exists my_f1_threshold (integer, double precision);
create or replace function my_f1_threshold (CVexpID integer, threshold double precision) returns double precision as 
'-- Calculates f1 where the decision hyperplane is given by threshold for a CV experiment
select 
(case when prec + recall>0 then (2 * prec * recall/( prec + recall )) else 0 end) as f1 from
(Select *,
(case when tp+fp>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fp as double precision))) else 0 end) as prec,
(case when tp+fn>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fn as double precision))) else 0 end) as recall from
(Select a.expid,count(*) as cnt,b.total,
	sum(case when a.prediction<$2 and c.groundTruth is TRUE then 1 else 0 end) as fn,
	sum(case when a.prediction<$2 and c.groundTruth is FALSE then 1 else 0 end) as tn,
	sum(case when a.prediction>=$2 and c.groundTruth is FALSE then 1 else 0 end) as fp,
	sum(case when a.prediction>=$2 and c.groundTruth is TRUE then 1 else 0 end) as tp, count(*)
from ppiCVoutput a 
join ppiCV b on a.expID=b.ppiCVid 
join pair c on a.pair = c.pairLabel where expid=$1 group by a.expid, b.total) as sub1) as sub2
;' language SQL stable returns null on null input;
comment on function my_f1_threshold (integer, double precision)
is 'Calculates f1 where the decision hyperplane is given by threshold for a CV experiment:
under threshold is a negative prediction, over threshold is a positive prediction
on Kims kernels the default threshold is 0.5-epsilon=0.4999999999
on APG kernel the default threshold is specified by the kernel and not attached to the experiment results, 
therefore it is not advised to use this function, but my_f1
on other kernels the default threshold is 0.0';

drop function if exists my_prec_threshold (integer, double precision);
create or replace function my_prec_threshold (CVexpID integer, threshold double precision) returns double precision as 
'-- Calculates precision where the decision hyperplane is given by threshold for a CV experiment
select 
(case when tp+fp>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fp as double precision))) else 0 end) as prec
from
(Select a.expid,count(*) as cnt,b.total,
	sum(case when a.prediction>=$2 and c.groundTruth is FALSE then 1 else 0 end) as fp,
	sum(case when a.prediction>=$2 and c.groundTruth is TRUE then 1 else 0 end) as tp, count(*)
from ppiCVoutput a 
join ppiCV b on a.expID=b.ppiCVid 
join pair c on a.pair = c.pairLabel where expid=$1 group by a.expid, b.total) as sub1
;' language SQL stable returns null on null input;
comment on function my_prec_threshold (integer, double precision)
is 'Calculates precision where the decision hyperplane is given by threshold for a CV experiment:
under threshold is a negative prediction, over threshold is a positive prediction
on Kims kernels the default threshold is 0.5-epsilon=0.4999999999
on APG kernel the default threshold is specified by the kernel and not attached to the experiment results, 
therefore it is not advised to use this function, but my_f1
on other kernels the default threshold is 0.0';



drop function if exists compare_2exps (integer,integer);
create or replace function compare_2exps (CVexpID1GT integer,CVexpID2Prediction integer) returns double precision as 
'-- Compares two CV experiment when the first is treated as the Ground Truth and the second is the Prediction
select 
(case when prec + recall>0 then (2 * prec * recall/( prec + recall )) else 0 end) as f1 from
(Select *,
(case when tp+fp>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fp as double precision))) else 0 end) as prec,
(case when tp+fn>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fn as double precision))) else 0 end) as recall from
(Select a.expid,count(*) as cnt,sum(case when a.output=FALSE and a.output != a2.output then 1 else 0 end) as fn,
	sum(case when a.output is FALSE and a.output = a2.output then 1 else 0 end) as tn,
	sum(case when a.output is TRUE and a.output != a2.output then 1 else 0 end) as fp,
	sum(case when a.output is TRUE and a.output = a2.output then 1 else 0 end) as tp, count(*)
from ppiCVoutput a 
join ppiCVoutput a2 on a.pair = a2.pair
where a.expID=$1 and a2.expID=$2 group by a.expID) as sub1) as sub2
;' language SQL stable returns null on null input;
comment on function compare_2exps (integer,integer) is
'Compares two CV experiment when the first is treated as the Ground Truth and the second is the Prediction
Calculates the similarity of two experiments';


drop function if exists compare_2exps_threshold (integer,double precision,integer,double precision);
create or replace function compare_2exps_threshold 
(CVexpID1GT integer,threshold1GT double precision,
CVexpID1Prediction integer,threshold1Prediction double precision) 
returns double precision as 
'-- Compares two CV experiments with given thresholds, first experiment is the Ground Truth, Second is the Prediction
select 
(case when prec + recall>0 then (2 * prec * recall/( prec + recall )) else 0 end) as f1 from
(Select *,
(case when tp+fp>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fp as double precision))) else 0 end) as prec,
(case when tp+fn>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fn as double precision))) else 0 end) as recall from
(Select a.expid,count(*) as cnt,
	sum(case when 
		(case when $2 is null then (not a.output) else a.prediction<$2 end) and 
		(case when $4 is null then a2.output else a2.prediction>=$4 end)
		then 1 else 0 end) as fp,
	sum(case when
		(case when $2 is null then (not a.output) else a.prediction<$2 end) and 
		(case when $4 is null then (not a2.output) else a2.prediction<$4 end)
		then 1 else 0 end) as tn,
	sum(case when 
		(case when $2 is null then a.output else a.prediction>=$2 end) and 
		(case when $4 is null then (not a2.output) else a2.prediction<$4 end)
		then 1 else 0 end) as fn,
	sum(case when 
		(case when $2 is null then a.output else a.prediction>=$2 end) and 
		(case when $4 is null then a2.output else a2.prediction>=$4 end)
		then 1 else 0 end) as tp, count(*)
from ppiCVoutput a 
join ppiCVoutput a2 on a.pair = a2.pair
where a.expID=$1 and a2.expID=$3 group by a.expID) as sub1) as sub2
;' language SQL stable;
comment on function compare_2exps_threshold (integer,double precision,integer,double precision)
is 'Compares two CV experiments with given thresholds, first experiment is the Ground Truth, Second is the Prediction
Thresholds are used to take into consideration various kernels operating with different threshold values
call with "null" thresholds to use default threshold';

drop function if exists compare_foldsf1 (integer,integer);
create or replace function compare_foldsf1 (foldID1GT integer,foldID2Prediction integer) returns double precision as 
'select cast(avg(compare_2exps(a1.ppicvid,a2.ppicvid)) as double precision)
from ppicv as a1, ppicv as a2 where a1.ppicvfoldsid=$1 and a2.ppicvfoldsid=$2 and a1.fold=a2.fold;' 
language SQL stable returns null on null input;
comment on function compare_foldsf1 (integer,integer) is 
'Compares folds of two CV experiments and calculates their average f-measure
The first experiment is treated as Ground Truth, the second as Prediction';


drop function if exists compare_foldsf1_threshold (integer,double precision,integer,double precision);
create or replace function compare_foldsf1_threshold 
(foldID1GT integer,threshold1GT double precision,
foldID2Prediction integer,threshold1Prediction double precision) returns double precision as 
'select cast(avg(compare_2exps_threshold(a1.ppicvid,$2,a2.ppicvid,$4)) as double precision)
from ppicv as a1, ppicv as a2 where a1.ppicvfoldsid=$1 and a2.ppicvfoldsid=$3 and a1.fold=a2.fold;' 
language SQL stable;
comment on function compare_foldsf1_threshold (integer,double precision,integer,double precision) is
'Compares folds of two CV experiments with given thresholds and calculates their average f-measure
The first experiment is treated as Ground Truth, the second as Prediction
Thresholds are used to take into consideration various kernels operating with different threshold values
call with "null" thresholds to use default threshold';

drop function if exists foldsf1 (integer);
create or replace function foldsf1 (foldID integer) 
returns double precision as 
'select cast(avg(my_f1(ppicvid)) as double precision) from ppicv where ppicvfoldsid=$1;' 
language SQL stable returns null on null input;
comment on function foldsf1 (integer) is
'Calculates the average f1 of a fold (of a CV experiment) using my_f1 function';

DROP TYPE IF EXISTS fold_stat CASCADE;
CREATE TYPE fold_stat AS 
(
    prec DOUBLE PRECISION,
    recall DOUBLE PRECISION,
    f1 DOUBLE PRECISION,
    auc DOUBLE PRECISION
);

drop function if exists folds_statistics(integer);
create or replace function folds_statistics(foldID integer) returns fold_stat as 
'select (foldsPRF($1)).prec,(foldsPRF($1)).recall,(foldsPRF($1)).f1,avg(auc) from ppicv where ppicvfoldsid=$1;' 
language SQL stable returns null on null input;
comment on function folds_statistics (integer) is
'Calculates the average PRF of a fold (of a CV experiment) using my_f1 function';

drop function if exists avgf1 (integer);
create or replace function avgf1 (foldID integer) returns double precision as 'select cast(avg(f_measure) as double precision) from ppicv where ppicvfoldsid=$1;' 
language SQL stable returns null on null input;
comment on function avgf1 (integer) is
'Calculates the average f1 of a fold (of a CV experiment) using the precalculated f_measure column';

drop function if exists avgfolds_statistics(integer);
create or replace function avgfolds_statistics(foldID integer) returns fold_stat as 
'select avg(precision_),avg(recall), avg(f_measure),avg(auc) from ppicv where ppicvfoldsid=$1 and ppicvfoldsid is not null;' 
language SQL stable returns null on null input;
comment on function avgfolds_statistics (integer) is
'Calculates the average f1 of a fold (of a CV experiment) using the precalculated columns';



drop function if exists foldsf1_threshold (integer,double precision);
create or replace function foldsf1_threshold (foldID integer,threshold double precision) returns double precision as 'select cast(avg(my_f1_threshold(ppicvid,$2)) as double precision) from ppicv where ppicvfoldsid=$1;' 
language SQL stable returns null on null input;
comment on function foldsf1_threshold (integer,double precision) is
'Calculates the average f1 of a fold with threshold using the my_f1_threshold';

drop function if exists foldsprec_threshold (integer,double precision);
create or replace function foldsprec_threshold (foldID integer,threshold double precision) 
returns double precision as 'select cast(avg(my_prec_threshold(ppicvid,$2)) as double precision) from ppicv where ppicvfoldsid=$1;' 
language SQL stable returns null on null input;
comment on function foldsprec_threshold (integer,double precision) is
'Calculates the average prec of a fold with threshold using the my_prec_threshold';


drop function if exists cc_my_f1 (integer);
create or replace function cc_my_f1 (CCexpID integer) returns double precision as 
'-- Calculates the f-measure for a given CC experiment
select 
(case when prec + recall>0 then (2 * prec * recall/( prec + recall )) else 0 end) as f1 from
(Select *,
(case when tp+fp>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fp as double precision))) else 0 end) as prec,
(case when tp+fn>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fn as double precision))) else 0 end) as recall from
(Select a.expid,count(*) as cnt,b.total,
	sum(case when a.output=FALSE and c.groundTruth != a.output then 1 else 0 end) as fn,
	sum(case when a.output=FALSE and c.groundTruth = a.output then 1 else 0 end) as tn,
	sum(case when a.output=TRUE and c.groundTruth != a.output then 1 else 0 end) as fp,
	sum(case when a.output=TRUE and c.groundTruth = a.output then 1 else 0 end) as tp, count(*)
from ppiCCoutput a 
join ppiCC b on a.expID=b.ppiCCid 
join pair c on a.pair = c.pairLabel where expid=$1 group by a.expid, b.total) as sub1) as sub2
;' language SQL stable returns null on null input;
comment on function cc_my_f1 (integer) is 
'Calculates the f-measure for a given CC experiment';

drop function if exists cc_my_f1_threshold (integer, double precision);
create or replace function cc_my_f1_threshold (CCexpID integer, threshold double precision) returns double precision as 
'-- Calculates f1 where the decision hyperplane is given by threshold for a CC experiment
select 
(case when prec + recall>0 then (2 * prec * recall/( prec + recall )) else 0 end) as f1 from
(Select *,
(case when tp+fp>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fp as double precision))) else 0 end) as prec,
(case when tp+fn>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fn as double precision))) else 0 end) as recall from
(Select a.expid,count(*) as cnt,b.total,
	sum(case when a.prediction<$2 and c.groundTruth is TRUE then 1 else 0 end) as fn,
	sum(case when a.prediction<$2 and c.groundTruth is FALSE then 1 else 0 end) as tn,
	sum(case when a.prediction>=$2 and c.groundTruth is FALSE then 1 else 0 end) as fp,
	sum(case when a.prediction>=$2 and c.groundTruth is TRUE then 1 else 0 end) as tp, count(*)
from ppiCCoutput a 
join ppiCC b on a.expID=b.ppiCCid 
join pair c on a.pair = c.pairLabel where expid=$1 group by a.expid, b.total) as sub1) as sub2
;' language SQL stable returns null on null input;
comment on function cc_my_f1_threshold (integer, double precision)
is 'Calculates f1 where the decision hyperplane is given by threshold for a CC experiment:
under threshold is a negative prediction, over threshold is a positive prediction
on Kims kernels the default threshold is 0.5-epsilon=0.4999999999
on APG kernel the default threshold is specified by the kernel and not attached to the experiment results, 
therefore it is not advised to use this function, but my_f1
on other kernels the default threshold is 0.0';

drop function if exists cl_my_f1 (integer);
create or replace function cl_my_f1 (CLexpID integer) returns double precision as 
'-- Calculates the f-measure for a given CL experiment
select 
(case when prec + recall>0 then (2 * prec * recall/( prec + recall )) else 0 end) as f1 from
(Select *,
(case when tp+fp>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fp as double precision))) else 0 end) as prec,
(case when tp+fn>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fn as double precision))) else 0 end) as recall from
(Select a.expid,count(*) as cnt,b.total,
	sum(case when a.output=FALSE and c.groundTruth != a.output then 1 else 0 end) as fn,
	sum(case when a.output=FALSE and c.groundTruth = a.output then 1 else 0 end) as tn,
	sum(case when a.output=TRUE and c.groundTruth != a.output then 1 else 0 end) as fp,
	sum(case when a.output=TRUE and c.groundTruth = a.output then 1 else 0 end) as tp, count(*)
from ppiCLoutput a 
join ppiCL b on a.expID=b.ppiCLid 
join pair c on a.pair = c.pairLabel where expid=$1 group by a.expid, b.total) as sub1) as sub2
;' language SQL stable returns null on null input;
comment on function cl_my_f1 (integer) is 
'Calculates the f-measure for a given CL experiment';

drop function if exists cl_my_f1_threshold (integer, double precision);
create or replace function cl_my_f1_threshold (CLexpID integer, threshold double precision) returns double precision as 
'-- Calculates f1 where the decision hyperplane is given by threshold for a CL experiment
select 
(case when prec + recall>0 then (2 * prec * recall/( prec + recall )) else 0 end) as f1 from
(Select *,
(case when tp+fp>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fp as double precision))) else 0 end) as prec,
(case when tp+fn>0 then (cast(tp as double precision)/(cast(tp as double precision)+cast(fn as double precision))) else 0 end) as recall from
(Select a.expid,count(*) as cnt,b.total,
	sum(case when a.prediction<$2 and c.groundTruth is TRUE then 1 else 0 end) as fn,
	sum(case when a.prediction<$2 and c.groundTruth is FALSE then 1 else 0 end) as tn,
	sum(case when a.prediction>=$2 and c.groundTruth is FALSE then 1 else 0 end) as fp,
	sum(case when a.prediction>=$2 and c.groundTruth is TRUE then 1 else 0 end) as tp, count(*)
from ppiCLoutput a 
join ppiCL b on a.expID=b.ppiCLid 
join pair c on a.pair = c.pairLabel where expid=$1 group by a.expid, b.total) as sub1) as sub2
;' language SQL stable returns null on null input;
comment on function cl_my_f1_threshold (integer, double precision)
is 'Calculates f1 where the decision hyperplane is given by threshold for a CL experiment:
under threshold is a negative prediction, over threshold is a positive prediction
on Kims kernels the default threshold is 0.5-epsilon=0.4999999999
on APG kernel the default threshold is specified by the kernel and not attached to the experiment results, 
therefore it is not advised to use this function, but my_f1
on other kernels the default threshold is 0.0';


-- select r(foldsf1_threshold (foldID integer,threshold double precision)

drop table if exists thresholds;
create table thresholds( th double precision );
insert into thresholds (th) values (0.05), (0.1), (0.15), (0.2), (0.25), (0.3), (0.35), (0.4), (0.45), (0.5);

-- select ppiCVfoldsid,kernel,corpus,kernel_script,tt.th as threshold,r(foldsf1_threshold(ppicvfoldsid,tt.th)) as myf1 
-- from ppiCVfolds, thresholds tt where  
-- kernel='Kim' and kernel_script NOT LIKE 'com%' and kernel_script NOT LIKE 'syn%' and kernel_script NOT LIKE 'lex%'
-- and tt.th in (0.1, 0.25)
-- limit 1;
-- 

-- 4 versions: combined, lexical, shallow, syntatic
-- select t.kernel_script, t.threshold, t.c, t.j, avg(t.myf1) as f1, count(*) as cnt from 
-- (select corpus,kernel_script,c_preset as c,j,tt.th as threshold,r(foldsf1_threshold(ppicvfoldsid,tt.th)) as myf1 
-- from ppiCVfolds, thresholds tt where  
-- kernel='Kim' and kernel_script LIKE 'lexical -w%'
-- and tt.th in (0.1, 0.15, 0.4, 0.45)
-- ) as t
-- group by t.kernel_script, t.threshold, t.c, t.j 
-- order by f1 desc
-- ;
-- -- 

-- test
-- select corpus,kernel_script as threshold,r(foldsf1_threshold(ppicvfoldsid,tt.th)) as myf1, 
-- r(foldsprec_threshold(ppicvfoldsid,tt.th)) as myprec  
-- from ppiCVfolds, thresholds tt where  
-- kernel='Kim' and (ppicvfoldsid between 11066 and 11075)
-- and tt.th in (0.25)
-- order by myf1 desc;
-- 
-- 

-- select r(compare_foldsf1_threshold(9182,0.05,10275,0.5)) from ppiCV limit 1;

-- select * from (select ppiCVfoldsid,c,kernel_script,r(avgf1(ppiCVfoldsid)) as origf1 ,r(foldsf1(ppiCVfoldsid)),r(foldsf1_threshold(ppicvfoldsid,0.05)) as myf1 
-- from ppiCVfolds where corpus='HPRD50' 
-- and kernel_script LIKE 'combined -w%' and c > 0.05 and c < 0.07
-- --and (ppicvfoldsid between 10275 and 10285) or ppicvfoldsid = 9182 or (ppicvfoldsid between 10481 and 10493)
-- limit 100) as t
-- order by t.origf1 desc;
-- 

-- SELECT ppicvid,corpus,avg(auc) AS auc, avg(precision_) AS prec, avg(recall) AS recall, avg(f_measure) AS f_measure, count(*) AS cnt, c, kernel_script,led,j,lmin,lmax
-- from ppiCV where kernel = 'Kim' and corpus LIKE 'A%' GROUP by ppicvid,corpus,kernel_script,led,j,lmin,lmax,c ORDER BY f_measure DESC;
--   