DROP TYPE IF EXISTS stats CASCADE;
CREATE TYPE stats AS 
(
    tp INTEGER,
    fp INTEGER,
    fn INTEGER,
    tn INTEGER
);


DROP FUNCTION IF EXISTS stats_add(STATS,STATS);
CREATE FUNCTION stats_add
(
    IN stat1 STATS,
    IN stat2 STATS
)
RETURNS STATS AS
$$
    SELECT $1.tp+$2.tp, $1.fp+$2.fp, $1.fn+$2.fn, $1.tn+$2.tn;
$$ 
LANGUAGE SQL 
IMMUTABLE RETURNS NULL ON NULL INPUT;


CREATE AGGREGATE stats_sum (
    sfunc = stats_add,
    basetype = stats,
    stype = stats,
    initcond = '(0,0,0,0)'
);


DROP TYPE IF EXISTS extendedStats CASCADE;
CREATE TYPE extendedStats AS
(
    P   NUMERIC,
    R   NUMERIC,
    F   NUMERIC,
    ACC NUMERIC,
    tp  INTEGER,
    fp  INTEGER,
    fn  INTEGER,
    tn  INTEGER,
    n   INTEGER
);

DROP TYPE IF EXISTS pairstats CASCADE;
CREATE TYPE pairstats AS 
(
    pairLabel TEXT,
    GT BOOLEAN,
    stat STATS
);

DROP FUNCTION IF EXISTS rr(NUMERIC);
CREATE FUNCTION rr
(
    IN x NUMERIC
)
RETURNS NUMERIC AS
$$
    SELECT round($1, 4);
$$ 
LANGUAGE SQL 
IMMUTABLE RETURNS NULL ON NULL INPUT;


DROP FUNCTION IF EXISTS rr(DOUBLE PRECISION);
CREATE FUNCTION rr
(
    IN x DOUBLE PRECISION
)
RETURNS NUMERIC AS
$$
    SELECT rr(CAST($1 AS NUMERIC));
$$ 
LANGUAGE SQL 
IMMUTABLE RETURNS NULL ON NULL INPUT;

DROP FUNCTION IF EXISTS newStats(INTEGER,INTEGER,INTEGER,INTEGER);
CREATE FUNCTION newStats
(
    IN tp INTEGER,
    IN fp INTEGER,
    IN fn INTEGER,
    IN tn INTEGER
)
RETURNS STATS AS
$$
    SELECT $1, $2, $3, $4;
$$ 
LANGUAGE SQL 
IMMUTABLE RETURNS NULL ON NULL INPUT;

DROP FUNCTION IF EXISTS calculatePrecision(STATS);
CREATE FUNCTION calculatePrecision
(
    IN stat STATS
)
RETURNS DOUBLE PRECISION AS
$$
    SELECT CASE WHEN ($1.tp + $1.fp) = 0 THEN 0.0 ELSE $1.tp / CAST ($1.tp + $1.fp AS DOUBLE PRECISION) END;
$$ 
LANGUAGE SQL 
IMMUTABLE RETURNS NULL ON NULL INPUT;


DROP FUNCTION IF EXISTS calculateRecall(STATS);
CREATE FUNCTION calculateRecall
(
    IN stat STATS
)
RETURNS DOUBLE PRECISION AS
$$
    SELECT CASE WHEN ($1.tp + $1.fn) = 0 THEN 0.0 ELSE $1.tp / CAST ($1.tp + $1.fn AS DOUBLE PRECISION) END;
$$ 
LANGUAGE SQL 
IMMUTABLE RETURNS NULL ON NULL INPUT;

DROP FUNCTION IF EXISTS calculateExtendedStats(STATS);
CREATE FUNCTION calculateExtendedStats
(
    IN stat STATS
)
RETURNS extendedStats AS
$$
    SELECT 
        rr(calculatePrecision($1)) AS p,
        rr(calculateRecall($1))    AS r,
        CASE
            WHEN (calculateRecall($1) + calculatePrecision($1)) = 0 THEN rr(0.0)
            ELSE rr((calculateRecall($1) * calculatePrecision($1) * 2) / (calculateRecall($1) + calculatePrecision($1)))
            END AS f,
        CASE 
            WHEN ($1.tp + $1.fn + $1.fp + $1.tn) = 0 THEN rr(0.0)
            ELSE rr(($1.tp + $1.tn) / CAST (($1.tp + $1.fn + $1.fp + $1.tn) AS DOUBLE PRECISION))
            END AS acc,
        $1.tp,
        $1.fp,
        $1.fn,
        $1.tn,
        ($1.tp + $1.fn + $1.fp + $1.tn) AS n
    ;
$$ 
LANGUAGE SQL 
IMMUTABLE RETURNS NULL ON NULL INPUT;

DROP FUNCTION IF EXISTS newStats(BIGINT,BIGINT,BIGINT,BIGINT);
CREATE FUNCTION newStats
(
    IN tp BIGINT,
    IN fp BIGINT,
    IN fn BIGINT,
    IN tn BIGINT
)
RETURNS STATS AS
$$
    SELECT cast($1 as INTEGER), cast($2 as INTEGER), cast($3 as INTEGER), cast($4 as INTEGER);
$$ 
LANGUAGE SQL 
IMMUTABLE RETURNS NULL ON NULL INPUT;

