CREATE OR REPLACE FUNCTION getlist(integer[]) RETURNS SETOF ppicv as $$
SELECT * FROM ppicv
WHERE ppicvid = ANY($1)
$$ LANGUAGE SQL STABLE STRICT;

-- test
-- SELECT * FROM getlist('{77542,77582,77572}');

drop function if exists getppicvids(text[],text,integer);

create or replace function getppicvids(mytables text[],mycorpus text, myfold integer) returns integer[] as
$$
declare
 rec record;
 tmp integer[];
begin
 FOR currID IN 1..array_upper(mytables,1) LOOP 
   tmp[currID] = NULL;
   for rec in EXECUTE 'select ppicvid from ' || mytables[currID] || ' where corpus=''' || mycorpus || ''' and fold=' || myfold LOOP
     tmp[currID] := rec.ppicvid;
   END LOOP;
 END LOOP;  
 return tmp;
end
$$
language 'plpgsql';

create or replace function getppicvids_new(kernels text[],mycorpus text, myfold integer) returns integer[] as
$$
declare
 rec record;
 tmp integer[];
begin
 FOR currID IN 1..array_upper(kernels,1) LOOP 
   tmp[currID] = NULL;
   for rec in EXECUTE 'select ppicvid from Best_CV where kernel=''' || kernels [currID] || ''' and corpus=''' || mycorpus || ''' and fold=' || myfold LOOP
     tmp[currID] := rec.ppicvid;
   END LOOP;
 END LOOP;  
 return tmp;
end
$$
language 'plpgsql';


-- test
-- select getppicvids(ARRAY['Best_CV_cosine','Best_CV_APG','Best_CV_cosine','Best_CV_cosine','Best_CV_cosine','Best_CV_cosine'],'LLL',0);

drop function if exists get_bestppicvids_bag(text[],text,integer);

create or replace function get_bestppicvids_bag(mykernels text[],mycorpus text, myfold integer) returns integer[] as
$$
begin
 return array(select ppicvid from Best_CV where corpus=mycorpus and fold=myfold and kernel=any(mykernels));
end
$$
language 'plpgsql';
comment on function get_bestppicvids_bag(text[],text,integer) is 
'Returns in an array the bag of matching ids, if a kernel is given more than once, only 1 id is returned
The size of input and output array therefore may be different'
;

-- test
-- select get_bestppicvids(ARRAY['cosine','APG','cosine','cosine','cosine','cosine'],'LLL',0);

drop function if exists get_bestppicvids(text[],text,integer);

create or replace function get_bestppicvids(mykernels text[],mycorpus text, myfold integer) returns integer[] as
$$
declare
 tmp integer[];
begin
 FOR i IN 1..array_upper(mykernels,1) LOOP
    tmp[i] := (select ppicvid from Best_CV where corpus=mycorpus and fold=myfold and kernel=mykernels[i]);
 END LOOP;
 return tmp;
end
$$
language 'plpgsql';
comment on function get_bestppicvids(text[],text,integer) is 
'Returns in an array the matching ids, if a kernel is given x times, its id is returned x times
The size of input and output array are the same'
;


DROP FUNCTION IF EXISTS array_to_rows( text[] );

CREATE OR REPLACE FUNCTION array_to_rows( text[])
RETURNS SETOF text AS
$BODY$
DECLARE
in_array alias FOR $1;
out_varchar varchar;
BEGIN
FOR i IN 1..array_upper(in_array,1) loop
RETURN next in_array[i];
end loop;
RETURN;
END
$BODY$
LANGUAGE 'plpgsql' STABLE;



-- select p1.ppicvid as exp1,p2.ppicvid as exp2 from Best_CV_kBSPS p1, Best_CV_cosine p2
-- where p1.fold=p2.fold and p1.corpus=p2.corpus and p1.corpus='LLL' and p1.fold='0'
-- ;
-- 


-- select corpus,count(*) from ppicv where ppicvid=ANY('{77585,78115,16321,77694,38766}') group by corpus;

