drop function if exists entity_blinding(Text,INTEGER,INTEGER,character);
create or replace function entity_blinding(rawText Text,beginOffset INTEGER,endOffset INTEGER,replacementChar character) 
returns Text AS
$$
	select 
	substring($1 from 0 for $2+1)||lpad('', ($3-$2)+1, $4)||
	substring($1 from $3+2);
$$
language SQL immutable returns null on null input;
comment on function entity_blinding(Text,INTEGER,INTEGER,character) is
'Blinds a given entity denoted by (beginOffset,endOffset) in rawText with charachter replacementChar';

drop function if exists blinded_sentence(Text);
create or replace function blinded_sentence(pairLabel Text) 
returns Text AS
$$
	select entity_blinding(entity_blinding(s.rawText,e2.beginOffset,e2.endOffset,'Y'),e1.beginOffset,e1.endOffset,'X')
from pair pr 
	inner join entity e1 on e1.entityLabel = pr.entity1
	inner join entity e2 on e2.entityLabel = pr.entity2
	inner join sentence s on s.label = e1.sentence
	where pr.pairLabel=$1;
$$
language SQL stable returns null on null input;
comment on function blinded_sentence(Text) is
'Returns the entity-blinded version of a sentence for a pair identifier: the first entity is denoted by XXXX the second by YYYY';

drop function if exists blinded_sentence_XX(Text);
create or replace function blinded_sentence_XX(pairLabel Text) 
returns Text AS
$$
	select entity_blinding(entity_blinding(s.rawText,e2.beginOffset,e2.endOffset,'X'),e1.beginOffset,e1.endOffset,'X')
from pair pr 
	inner join entity e1 on e1.entityLabel = pr.entity1
	inner join entity e2 on e2.entityLabel = pr.entity2
	inner join sentence s on s.label = e1.sentence
	where pr.pairLabel=$1;
$$
language SQL stable returns null on null input;
comment on function blinded_sentence_XX(Text) is
'Returns the entity-blinded version of a sentence for a pair identifier: both entities are denoted by XXXX';

-- test 
-- select blinded_sentence_XX('AIMed.d3.s33.p9');

CREATE OR REPLACE FUNCTION word_length(text)
RETURNS int4 AS
$$
select (case when char_length($1)=0 then 0 else array_upper(string_to_array($1, ' '), 1) end);
$$
LANGUAGE SQL IMMUTABLE RETURNS NULL ON NULL INPUT;
comment on function word_length(Text) is
'Calculates the number of words in a text, returns 0 for empty string';

drop function if exists between_entities(Text,INTEGER,INTEGER);
create or replace function between_entities(rawText Text,endEntity1 INTEGER,beginEntity2 INTEGER) 
returns Text AS
$$
	select btrim(t.substr,'(),;/- ') from
	(select
	case when $3-$2-1>0 then 
		substring($1 from $2+2 for $3-$2-1)
	     else
	      ''
	     end as substr) as t
	;
$$
language SQL immutable returns null on null input;
comment on function between_entities(Text,INTEGER,INTEGER) is
'Cuts the string between the 2 entities and trim leading and trailing non-alpha chars';

drop function if exists BetweenEntities(integer, integer, integer, integer, Text);

create or replace function BetweenEntities(be1 Integer,ee1 Integer,be2 Integer,ee2 Integer,blinded Text) returns Text as
$$
select (case when (t.char_length<=0 or start_pos<0) then '' else substring($5 from t.start_pos for t.char_length) end) as betweenentities
from
(select (case when $3>$2 then $3-$2-1 else $1-$4-1 end) as char_length,
(case when $3>$2 then $2+2 else $4+2 end) as start_pos
) as t
$$
LANGUAGE 'SQL'
STABLE RETURNS NULL ON NULL INPUT
;
comment on function BetweenEntities(integer, integer, integer, integer, Text) is 
'Return the text fragment between the two entities (given by charoffsets) from the sentence blinded.
Return empty string if there is no in-between part with the given parameters';


drop function if exists BeforeEntities(integer, integer, integer, integer, Text);

create or replace function BeforeEntities(be1 Integer,ee1 Integer,be2 Integer,ee2 Integer,blinded Text) returns Text as
$$
select (case when (end_pos<0) then '' else substring($5 from 1 for end_pos) end) as beforeentities
from
(select 
(case when $1<$3 then $1 else $3 end) as end_pos
--,(case when $3>$2 then $3-$2-1 else $1-$4-1 end) as char_length
) as t
$$
LANGUAGE 'SQL'
STABLE RETURNS NULL ON NULL INPUT
;
comment on function BeforeEntities(integer, integer, integer, integer, Text) is 
'Return the text fragment before the first entity (given by charoffsets) from the sentence blinded.
Return empty string if there is no in-between part with the given parameters';

drop function if exists AfterEntities(integer, integer, integer, integer, Text);

create or replace function AfterEntities(be1 Integer,ee1 Integer,be2 Integer,ee2 Integer,blinded Text) returns Text as
$$
select (case when (start_pos<0) then '' else substring($5 from start_pos) end) as afterentities
from
(select 
(case when $2<$4 then $4+2 else $2+2 end) as start_pos) as t
$$
LANGUAGE 'SQL'
STABLE RETURNS NULL ON NULL INPUT
;
comment on function AfterEntities(integer, integer, integer, integer, Text) is 
'Return the text fragment after the second entity (given by charoffsets) from the sentence blinded.
Return empty string if there is no in-between part with the given parameters';



-- select pr.pairLabel,e1.entityText,e1.beginOffset,e1.endOffset,
-- 	e2.entityText,e2.beginOffset,e2.endOffset,s.rawText,
-- 	entity_blinding(entity_blinding(s.rawText,e2.beginOffset,e2.endOffset,'Y'),e1.beginOffset,e1.endOffset,'X')
-- from pair pr 
-- 	inner join entity e1 on e1.entityLabel = pr.entity1
-- 	inner join entity e2 on e2.entityLabel = pr.entity2
-- 	inner join sentence s on s.label = e1.sentence;	