drop function if exists entity_blinding(Text,INTEGER,INTEGER,character);
create or replace function entity_blinding(rawText Text,beginOffset INTEGER,endOffset INTEGER,replacementChar character) 
returns Text AS
$$
	select 
	substring($1 from 0 for $2+1)||lpad('', ($3-$2)+1, $4)||
	substring($1 from $3+2);
$$
language SQL immutable returns null on null input;
comment on function entity_blinding(Text,INTEGER,INTEGER,character) is
'Blinds a given entity denoted by (beginOffset,endOffset) in rawText with charachter replacementChar';

drop function if exists blinded_sentence(Text);
create or replace function blinded_sentence(pairLabel Text) 
returns Text AS
$$
	select entity_blinding(entity_blinding(s.rawText,e2.beginOffset,e2.endOffset,'Y'),e1.beginOffset,e1.endOffset,'X')
from pair pr 
	inner join entity e1 on e1.entityLabel = pr.entity1
	inner join entity e2 on e2.entityLabel = pr.entity2
	inner join sentence s on s.label = e1.sentence
	where pr.pairLabel=$1;
$$
language SQL stable returns null on null input;
comment on function blinded_sentence(Text) is
'Returns the entity-blinded version of a sentence for a pair identifier: the first entity is denoted by XXXX the second by YYYY';

create or replace function full_blinded_sentence(sentenceLabel Text) returns Text AS 
$$
DECLARE
  maxentity int;
  entities text[];
  begins int[];
  ends int[];
  sentencetext text;
  BEGIN
    select into sentencetext rawText from sentence where label=$1;
    select into maxentity max(position) from entity where sentence=$1;
    FOR i IN 0 .. maxentity LOOP
      select into sentencetext 
      entity_blinding(sentencetext,(select beginoffset from entity where position=i and sentence=$1),(select endoffset from entity where position=i and sentence=$1),'X');
    END LOOP;
    RETURN sentencetext;
  END
$$ LANGUAGE 'plpgsql'; 
comment on function full_blinded_sentence(Text) is
'Returns the full-entity-blinded version of a sentence for a sentence identifier: all entities are blinded with X';

drop function if exists blinded_sentence_XX(Text);
create or replace function blinded_sentence_XX(pairLabel Text) 
returns Text AS
$$
	select entity_blinding(entity_blinding(s.rawText,e2.beginOffset,e2.endOffset,'X'),e1.beginOffset,e1.endOffset,'X')
from pair pr 
	inner join entity e1 on e1.entityLabel = pr.entity1
	inner join entity e2 on e2.entityLabel = pr.entity2
	inner join sentence s on s.label = e1.sentence
	where pr.pairLabel=$1;
$$
language SQL stable returns null on null input;
comment on function blinded_sentence_XX(Text) is
'Returns the entity-blinded version of a sentence for a pair identifier: both entities are denoted by XXXX';

-- test 
-- select blinded_sentence_XX('AIMed.d3.s33.p9');

CREATE OR REPLACE FUNCTION word_length(text)
RETURNS int4 AS
$$
select (case when char_length($1)=0 then 0 else array_upper(string_to_array($1, ' '), 1) end);
$$
LANGUAGE SQL IMMUTABLE RETURNS NULL ON NULL INPUT;
comment on function word_length(Text) is
'Calculates the number of words in a text, returns 0 for empty string';

drop function if exists between_entities(Text,INTEGER,INTEGER);
create or replace function between_entities(rawText Text,endEntity1 INTEGER,beginEntity2 INTEGER) 
returns Text AS
$$
	select btrim(t.substr,'(),;/- ') from
	(select
	case when $3-$2-1>0 then 
		substring($1 from $2+2 for $3-$2-1)
	     else
	      ''
	     end as substr) as t
	;
$$
language SQL immutable returns null on null input;
comment on function between_entities(Text,INTEGER,INTEGER) is
'Cuts the string between the 2 entities and trim leading and trailing non-alpha chars';


-- select pr.pairLabel,e1.entityText,e1.beginOffset,e1.endOffset,
-- 	e2.entityText,e2.beginOffset,e2.endOffset,s.rawText,
-- 	entity_blinding(entity_blinding(s.rawText,e2.beginOffset,e2.endOffset,'Y'),e1.beginOffset,e1.endOffset,'X')
-- from pair pr 
-- 	inner join entity e1 on e1.entityLabel = pr.entity1
-- 	inner join entity e2 on e2.entityLabel = pr.entity2
-- 	inner join sentence s on s.label = e1.sentence;	