--
-- corpora
--
CREATE TABLE corpus
(
  corpusName  text PRIMARY KEY
);

--INSERT INTO corpus (corpusName) VALUES ('AIMed');
--INSERT INTO corpus (corpusName) VALUES ('BioInfer');
--INSERT INTO corpus (corpusName) VALUES ('IEPA');
--INSERT INTO corpus (corpusName) VALUES ('LLL');
--INSERT INTO corpus (corpusName) VALUES ('HPRD50');

--
-- documents
--
CREATE TABLE doc
(
  docLabel    text    PRIMARY KEY,
  corpus      text    NOT NULL  REFERENCES corpus,
  position    integer NOT NULL  CHECK (position >= 0),
  origId      text    NOT NULL,
  fold        integer
);

CREATE UNIQUE INDEX doc_corpus_position_idx
  ON doc (corpus, position);


--
-- sentences
--
CREATE TABLE sentence
(
  label       text    PRIMARY KEY,
  doc         text    NOT NULL  REFERENCES doc,
  position    integer NOT NULL  CONSTRAINT nonnegative_positition CHECK (position >= 0),
  origId      text    NOT NULL,
  rawText     text    NOT NULL,
  beginOffset integer NOT NULL  CONSTRAINT nonnegative_beginOffset CHECK (beginOffset >= 0),
  endOffset   integer NOT NULL  CONSTRAINT nonnegative_endOffset CHECK (endOffset >= 0),
  CONSTRAINT less_begin_end CHECK (beginOffset <= endOffset)
);

CREATE UNIQUE INDEX sentence_corpus_doc_position_idx
  ON sentence (doc, position);
  
CREATE UNIQUE INDEX sentence_doc_offset_idx
  ON sentence (doc, beginOffset, endOffset);


--
-- enitity mentions (genes, proteins)
--
CREATE TABLE entity
(
  entityLabel text    PRIMARY KEY,  -- e.g. "LLL.d0.s3.e1" 
  sentence    text    REFERENCES sentence,
  position    integer NOT NULL  CONSTRAINT nonnegative_position     CHECK (position >= 0),
  origId      text    NOT NULL,
  entityText  text    NOT NULL,
  entityType  text,
  beginOffset integer NOT NULL  CONSTRAINT nonnegative_beginOffset  CHECK (beginOffset >= 0),
  endOffset   integer NOT NULL  CONSTRAINT nonnegative_endOffset       CHECK (endOffset >= 0),
  beginOffset2 integer CONSTRAINT nonnegative_beginOffset2  CHECK (beginOffset2 >= 0),
  endOffset2   integer CONSTRAINT nonnegative_endOffset2       CHECK (endOffset2 >= 0),
  
  CONSTRAINT unique_entity_params UNIQUE (sentence, position),
  CONSTRAINT less_begin_end       CHECK (beginOffset <= endOffset)
);

--CREATE UNIQUE INDEX entity_orig_id_idx
--  ON entity (origId);

CREATE UNIQUE INDEX entity_param_idx
  ON entity (sentence, position);


--
-- entity pairs (possible interaction)
--
CREATE TABLE pair
(
  pairLabel   text    PRIMARY KEY,  /*e.g. "LLL.d0.s3.p1" */
  entity1     text    NOT NULL  REFERENCES entity,
  entity2     text    NOT NULL  REFERENCES entity,
  groundTruth boolean NOT NULL,
  depShortestPath integer,
  syntShortestPath integer,
  border_subj integer,
  border_obj integer
  
  
  CONSTRAINT inequal_entity CHECK (entity1 <> entity2),
  UNIQUE (entity1, entity2),
  constraint subj_range check(border_subj is null or border_subj between 0 and 2),
  constraint obj_range check(border_obj is null or border_obj between 0 and 2),
  constraint subj_obj_range check(border_subj is null or border_obj is null or (border_subj+border_obj) between 0 and 2)
);

CREATE UNIQUE INDEX pair_ground_truth_idx
  ON pair (groundTruth, pairLabel);

-- Information gains of dependency labels, syntax types on pairs and sentences

alter table sentence add column dkl_synt_tree double precision;
alter table pair add column dkl_synt_shortest_path double precision;
alter table sentence add column dkl_dep_graph double precision;
alter table pair add column dkl_dep_shortest_path double precision;