create or replace function r (double precision) returns numeric as 'select round(cast(100*$1 as numeric),1);' 
language SQL immutable returns null on null input;

drop table if exists ppiCLoutput;
drop table if exists ppiCL;

CREATE TABLE ppiCL ( ppiCLid serial NOT NULL,
	corpus text REFERENCES corpus,
	parsertype text,
	parser text,
	kernel text,
	kernel_script text,
	led text,
	c double precision,
	j double precision,
	lmax integer,
	lmin integer,
	k integer,
	match_ text,
	normalized boolean,
	input_format text,
	tp integer check(tp>=0 OR tp is NULL),
	fn integer check(fn>=0 OR fn is NULL),
	tn integer check(tn>=0 OR tn is NULL),
	fp integer check(fp>=0 OR fp is NULL),
	total integer check(total>=0 OR total is NULL),
	auc double precision check(auc>=0 OR auc is NULL),
	precision_ double precision check(precision_>=0 OR precision_ is NULL),
	recall double precision check(recall>=0 OR recall is NULL),
	f_measure double precision check(f_measure>=0 OR f_measure is NULL),
	learn_sec double precision check(learn_sec>=0 OR learn_sec is NULL),
	classify_sec double precision check(classify_sec>=0 OR classify_sec is NULL),
	sv_num integer check(sv_num>0 OR sv_num is NULL),
	forced_threshold double precision,
	CONSTRAINT ppiCL_pkey PRIMARY KEY (ppiCLid),
	CONSTRAINT unique_CLexperiment UNIQUE (corpus,parsertype,parser,kernel,c,j,lmax,lmin,k,led,normalized,match_,input_format)
);

CREATE UNIQUE INDEX unique_CLexperiment_idx on ppiCL(
coalesce(corpus,'*** NULLLLLL ***'),
coalesce(parsertype,'*** NULLLLLL ***'),
coalesce(parser,'*** NULLLLLL ***'),
coalesce(kernel,'*** NULLLLLL ***'),
coalesce(kernel_script,'*** NULLLLLL ***'),
coalesce(c,-1),
coalesce(j,-1),
coalesce(lmax,-1),
coalesce(lmin,-1),
coalesce(k,-1),
coalesce(led,'*** NULLLLLL ***'),
coalesce(normalized,'0'),
coalesce(match_,'*** NULLLLLL ***'),
coalesce(input_format,'*** NULLLLLL ***'));


CREATE OR REPLACE VIEW ppiCLwithC AS 
SELECT *,
	(case when ((ppiCL.kernel <> 'SpT' or (ppiCL.kernel = 'SpT' and ppiCL.c in (0.0625, 1, 16, 64, 512))) 
		and ppiCL.kernel <> 'kBSPS') then c else null end) as c_normal,
	(case when ((ppiCL.kernel='SpT' and ppiCL.c not in (0.0625, 1, 16, 64, 512)) or ppiCL.kernel='kBSPS' ) 
		then c else null end) as c_spec
	from ppiCL;


CREATE TABLE ppiCLoutput
(
  expId       integer NOT NULL  REFERENCES ppiCL(ppiCLid) on delete cascade,
  pair        text    NOT NULL  REFERENCES pair,
  output      boolean NOT NULL,
  prediction  float,
  PRIMARY KEY (expId, pair)
);

CREATE UNIQUE INDEX ppi_cl_output_idx
  ON ppiCLoutput (expId, output, pair);
