
DROP TYPE interactionSupport CASCADE
CREATE TYPE interactionSupport AS (
    mapped1 int,
    mapped2 int,
    support int
);

CREATE TABLE predictedInteractionMap (
    experimentName text NOT NULL,
    interactionId  int NOT NULL REFERENCES interaction,
    mappingId      int NOT NULL REFERENCES mapping,
    mapped1        int,
    mapped2        int,
    support        int NOT NULL);

CREATE UNIQUE INDEX pimap_idx ON 
predictedInteractionMap ( experimentName, interactionId, mappingId, mapped1, mapped2 );

CREATE TABLE predictedInteractionSubsetMap (
    experimentName text NOT NULL,
    subsetname     text NOT NULL,
    interactionId  int  NOT NULL REFERENCES interaction,
    mappingId      int  NOT NULL REFERENCES mapping,
    mapped1        int,
    mapped2        int,
    support        int  NOT NULL CHECK(support > 0));

CREATE UNIQUE INDEX pismap_idx ON 
predictedInteractionSubsetMap ( experimentName, subsetName, interactionId, mappingId, mapped1, mapped2 );


-- No self interactions
CREATE OR REPLACE FUNCTION predictedInteractions(
    experimentName text,
    interactionId  int,
    mappingId      int) 
RETURNS SETOF interactionSupport AS
$$
SELECT
 LEAST(   gene1, gene2) AS mapped1,
 GREATEST(gene1, gene2) AS mapped2,
 CAST(COUNT(*) AS int)  AS support
FROM pairmap pm
-- genes are in RegulonDB
JOIN interactingMap i1 ON gene1 = i1.mapped AND i1.interactionId = $2
JOIN interactingMap i2 ON gene2 = i2.mapped AND i2.interactionId = $2
-- method output is ??
JOIN ppisplitoutput o ON o.pair = pm.pair AND o.output = True
JOIN ppisplit s ON s.ppisplitid = o.expid AND s.name = $1
---------------
-- AND o.expid = 158 -- TODO remove !!!!!!!!!
-----------------
WHERE pm.mappingId = $3
AND gene1 <> gene2
GROUP BY mapped1, mapped2
$$
LANGUAGE SQL 
STABLE
RETURNS NULL ON NULL INPUT
;

CREATE OR REPLACE FUNCTION predictedInteractionsSubset(
    experimentName text,
    interactionId  int,
    mappingId      int,
    subsetName     text) 
RETURNS SETOF interactionSupport AS
$$
SELECT
 LEAST(   gene1, gene2) AS mapped1,
 GREATEST(gene1, gene2) AS mapped2,
 CAST(COUNT(*) AS int)  AS support
FROM pairmap pm
-- genes are in RegulonDB
JOIN interactingMap i1 ON gene1 = i1.mapped AND i1.interactionId = $2
JOIN interactingMap i2 ON gene2 = i2.mapped AND i2.interactionId = $2
-- method output is ??
JOIN ppisplitoutput o ON o.pair = pm.pair AND o.output = True
JOIN ppisplit s ON s.ppisplitid = o.expid AND s.name = $1
JOIN pair_subset ps ON ps.name = $4 AND ps.pair = o.pair
WHERE pm.mappingId = $3
AND gene1 <> gene2
GROUP BY mapped1, mapped2
$$
LANGUAGE SQL 
STABLE
RETURNS NULL ON NULL INPUT
;


CREATE OR REPLACE FUNCTION predictedInteractionStats(
    experimentName text,
    interactionId  int,
    mappingId      int) 
RETURNS stats AS
$$
SELECT 
 CAST(SUM(CASE WHEN pm.mapped1 IS NOT NULL AND im.mapped1 IS NOT NULL THEN 1 ELSE 0 END) AS int) AS tp,
 CAST(SUM(CASE WHEN pm.mapped1 IS NOT NULL AND im.mapped1 IS     NULL THEN 1 ELSE 0 END) AS int) AS fp,
 CAST(SUM(CASE WHEN pm.mapped1 IS     NULL AND im.mapped1 IS NOT NULL THEN 1 ELSE 0 END) AS int) AS fn,
 CAST(SUM(CASE WHEN pm.mapped1 IS     NULL AND im.mapped1 IS     NULL THEN 1 ELSE 0 END) AS int) AS tn
FROM 
    -- predictedInteractions($1, $2, $3)
   (SELECT mapped1, mapped2 FROM predictedInteractionMap WHERE experimentName = $1 AND interactionId = $2 AND mappingId = $3)
pm 
FULL JOIN (SELECT mapped1, mapped2 FROM interactionMap WHERE interactionId = $2) im 
  ON im.mapped1 = pm.mapped1 AND im.mapped2 = pm.mapped2 
;
$$
LANGUAGE SQL 
STABLE
RETURNS NULL ON NULL INPUT
;

CREATE OR REPLACE FUNCTION predictedInteractionStatsAboveSupport(
    experimentName text,
    interactionId  int,
    mappingId      int,
    minSupport     int)
RETURNS stats AS
$$
SELECT 
 CAST(SUM(CASE WHEN pm.mapped1 IS NOT NULL AND im.mapped1 IS NOT NULL THEN 1 ELSE 0 END) AS int) AS tp,
 CAST(SUM(CASE WHEN pm.mapped1 IS NOT NULL AND im.mapped1 IS     NULL THEN 1 ELSE 0 END) AS int) AS fp,
 CAST(SUM(CASE WHEN pm.mapped1 IS     NULL AND im.mapped1 IS NOT NULL THEN 1 ELSE 0 END) AS int) AS fn,
 CAST(SUM(CASE WHEN pm.mapped1 IS     NULL AND im.mapped1 IS     NULL THEN 1 ELSE 0 END) AS int) AS tn
FROM 
 -- (SELECT mapped1, mapped2 FROM predictedInteractions($1, $2, $3) WHERE support >= $4) pm 
 (SELECT mapped1, mapped2 FROM predictedInteractionMap WHERE experimentName = $1 AND interactionId = $2 AND mappingId = $3 AND support >= $4) pm
FULL JOIN (SELECT mapped1, mapped2 FROM interactionMap WHERE interactionId = $2) im 
  ON im.mapped1 = pm.mapped1 AND im.mapped2 = pm.mapped2 
;
$$
LANGUAGE SQL 
STABLE
RETURNS NULL ON NULL INPUT
;

CREATE OR REPLACE FUNCTION predictedInteractionSubsetStatsAboveSupport(
    experimentName text,
    interactionId  int,
    mappingId      int,
    minSupport     int,
    subsetName     text
    )
RETURNS stats AS
$$
SELECT 
 CAST(SUM(CASE WHEN pm.mapped1 IS NOT NULL AND im.mapped1 IS NOT NULL THEN 1 ELSE 0 END) AS int) AS tp,
 CAST(SUM(CASE WHEN pm.mapped1 IS NOT NULL AND im.mapped1 IS     NULL THEN 1 ELSE 0 END) AS int) AS fp,
 CAST(SUM(CASE WHEN pm.mapped1 IS     NULL AND im.mapped1 IS NOT NULL THEN 1 ELSE 0 END) AS int) AS fn,
 CAST(SUM(CASE WHEN pm.mapped1 IS     NULL AND im.mapped1 IS     NULL THEN 1 ELSE 0 END) AS int) AS tn
FROM 
 -- (SELECT mapped1, mapped2 FROM predictedInteractions($1, $2, $3) WHERE support >= $4) pm 
 (SELECT mapped1, mapped2 
 FROM predictedInteractionSubsetMap
 WHERE experimentName = $1 AND interactionId = $2 AND mappingId = $3 AND support >= $4 AND subsetname = $5) pm
FULL JOIN (SELECT mapped1, mapped2 FROM interactionMap WHERE interactionId = $2) im 
  ON im.mapped1 = pm.mapped1 AND im.mapped2 = pm.mapped2 
;
$$
LANGUAGE SQL 
STABLE
RETURNS NULL ON NULL INPUT
;

CREATE OR REPLACE FUNCTION cachePredictedInteractions(
    experimentName text,
    interactionId  int,
    mappingId      int)
RETURNS void AS
$$
INSERT 
INTO predictedInteractionMap
    SELECT
        $1, $2, $3, 
        mapped1, mapped2, support 
    FROM predictedInteractions($1, $2, $3) 
;
$$
LANGUAGE SQL 
VOLATILE
;

CREATE OR REPLACE FUNCTION cachePredictedInteractionsSubset(
    experimentName text,
    interactionId  int,
    mappingId      int,
    subsetName     text)
RETURNS void AS
$$
INSERT 
INTO predictedInteractionSubsetMap
    SELECT
        $1, $4, $2, $3, 
        mapped1, mapped2, support 
    FROM predictedInteractionsSubset($1, $2, $3, $4) 
;
$$
LANGUAGE SQL 
VOLATILE
;


\set experimentName '\'GeneReg2PMC [piped9107]\''
select 1 AS minsupport, 
          * from calculateExtendedStats(predictedInteractionStatsAboveSupport(:experimentName, 2, 1, 1)) UNION ALL
select 2, * from calculateExtendedStats(predictedInteractionStatsAboveSupport(:experimentName, 2, 1, 2)) UNION ALL
select 3, * from calculateExtendedStats(predictedInteractionStatsAboveSupport(:experimentName, 2, 1, 3)) UNION ALL
select 4, * from calculateExtendedStats(predictedInteractionStatsAboveSupport(:experimentName, 2, 1, 4)) UNION ALL
select 5, * from calculateExtendedStats(predictedInteractionStatsAboveSupport(:experimentName, 2, 1, 5)) UNION ALL
select 6, * from calculateExtendedStats(predictedInteractionStatsAboveSupport(:experimentName, 2, 1, 6)) UNION ALL
select 7, * from calculateExtendedStats(predictedInteractionStatsAboveSupport(:experimentName, 2, 1, 7)) UNION ALL
select 8, * from calculateExtendedStats(predictedInteractionStatsAboveSupport(:experimentName, 2, 1, 8)) UNION ALL
select 9, * from calculateExtendedStats(predictedInteractionStatsAboveSupport(:experimentName, 2, 1, 9)) UNION ALL
select 10,* from calculateExtendedStats(predictedInteractionStatsAboveSupport(:experimentName, 2, 1,10))
;

--\set experimentName '\'PMID co-occurrence\''
-- jSRE GeneReg
--\set experimentName '\'piped9387\''
-- jSRE GeneReg+BioNLP'09
\set experimentName '\'piped9753\''
\set subsetName '\'RegulonDB\''
select cachePredictedInteractionsSubset(:experimentName, 2, 1, :subsetName);
select 1 AS minsupport, 
          * from calculateExtendedStats(predictedInteractionSubsetStatsAboveSupport(:experimentName, 2, 1, 1, :subsetName)) UNION ALL
select 2, * from calculateExtendedStats(predictedInteractionSubsetStatsAboveSupport(:experimentName, 2, 1, 2, :subsetName)) UNION ALL
select 3, * from calculateExtendedStats(predictedInteractionSubsetStatsAboveSupport(:experimentName, 2, 1, 3, :subsetName)) UNION ALL
select 4, * from calculateExtendedStats(predictedInteractionSubsetStatsAboveSupport(:experimentName, 2, 1, 4, :subsetName)) UNION ALL
select 5, * from calculateExtendedStats(predictedInteractionSubsetStatsAboveSupport(:experimentName, 2, 1, 5, :subsetName)) UNION ALL
select 6, * from calculateExtendedStats(predictedInteractionSubsetStatsAboveSupport(:experimentName, 2, 1, 6, :subsetName)) UNION ALL
select 7, * from calculateExtendedStats(predictedInteractionSubsetStatsAboveSupport(:experimentName, 2, 1, 7, :subsetName)) UNION ALL
select 8, * from calculateExtendedStats(predictedInteractionSubsetStatsAboveSupport(:experimentName, 2, 1, 8, :subsetName)) UNION ALL
select 9, * from calculateExtendedStats(predictedInteractionSubsetStatsAboveSupport(:experimentName, 2, 1, 9, :subsetName)) UNION ALL
select 10,* from calculateExtendedStats(predictedInteractionSubsetStatsAboveSupport(:experimentName, 2, 1,10, :subsetName))
;


-- populate co-occurrence
INSERT INTO ppisplit (name) values ('PMC co-occurrence') RETURNING ppisplitid;
INSERT INTO ppisplitoutput SELECT 312, pairlabel, True, 1.0 FROM pair WHERE pairlabel LIKE 'PMC%' AND pairlabel BETWEEN 'PMB' AND 'PMD';


-- <sentence id="PMC.dPMC1285198.s1" origId="1" text="Histamine dependence of pentagastrin-stimulated gastric acid secretion in rats.
--
SELECT
 LEAST(   gene1, gene2) AS mapped1,
 GREATEST(gene1, gene2) AS mapped2,
 CAST(COUNT(*) AS int)  AS support
FROM pairmap pm
-- genes are in RegulonDB
JOIN interactingMap i1 ON gene1 = i1.mapped AND i1.interactionId = 2
JOIN interactingMap i2 ON gene2 = i2.mapped AND i2.interactionId = 2
-- method output is ??
JOIN ppisplitoutput o ON o.pair = pm.pair AND o.output = True
JOIN ppisplit s ON s.ppisplitid = o.expid AND s.name = 'piped9753'
---------------
--AND o.expid = 158 -- TODO remove !!!!!!!!!
-----------------
WHERE pm.mappingId = 1
GROUP BY mapped1, mapped2

--DROP TABLE pair_subset;
CREATE TABLE pair_subset(name text NOT NULL, pair text NOT NULL REFERENCES pair);
CREATE UNIQUE INDEX pair_subset_idx ON pair_subset (name, pair);
-- INSERT INTO pair_subset 
--     SELECT 'PMC', p.pairlabel
--     FROM doc pmc 
--     JOIN doc pmid ON pmid.corpus = 'PMID' AND pmid.doclabel = 'PMID.dPMID' || SUBSTRING(pmc.doclabel from 9) AND pmc.corpus = 'PMC'
--     JOIN sentence s ON s.doc = pmid.doclabel
--     JOIN entity e ON e.sentence = s.label
--     JOIN pair p ON p.entity2 = e.entitylabel;
INSERT INTO pair_subset 
    SELECT 'RegulonDB', p.pairlabel
    FROM regulondb_pmid rdb
    JOIN doc pmid ON pmid.corpus = 'PMID' AND pmid.doclabel = 'PMID.dPMID' || CAST(rdb.pmid AS text)
    JOIN sentence s ON s.doc = pmid.doclabel
    JOIN entity e ON e.sentence = s.label
    JOIN pair p ON p.entity2 = e.entitylabel;
    
SELECT 
    d.doclabel AS doc,
    s.label AS sentence,
    p.pair AS pair,
    e1.entitylabel AS entity1,
    e2.entitylabel AS entity2,
    e1.origId AS gene1,
    e2.origId AS gene2,
    o.output AS interaction,
    o.prediction AS prediction,
    FROM doc d ON d.corpus = 'PMID'
    JOIN sentence s ON s.doc = pmid.doclabel
    JOIN entity e1 ON e.sentence = s.label
    JOIN pair p ON p.entity1 = e1.entitylabel
    JOIN entity e2 ON p.entity2 = e2.entitylabel
    JOIN ppisplitoutput o ON o.pair = p.pairlabel AND o.output = True
    JOIN ppisplit s ON s.ppisplitid = o.expid AND s.name = 'piped9387'
    
