#install.packages("gplots")
library(gplots)

#setwd("/home/philippe/workspace/ppi-benchmark")


# fancy hclust plot
drawHeatmap  <- function(
	data,
	groupLabels,
    plotFile = "heatmap.pdf",
    margin=20) {

	groupNames<- as.character(unique(groupLabels))
	numColorsForGroups<-length(groupNames)
	colForGroups<-rainbow(numColorsForGroups, s=0.5) # topo.colors(n)
	colorPlot<-as.character(groupLabels)
	for(i in 1:numColorsForGroups) {
		colorPlot[colorPlot==groupNames[i]] <- colForGroups[i]
	}
	pdf(plotFile,width=7, height=7)
	par(xpd=T, mar=par()$mar+c(0,0,0.1,0.1))
	heatmap.2( as.matrix(data),ColSideColors=colorPlot,key=FALSE,densadj = 0.5,trace="none", margin=c(margin, margin),dendrogram = "column")
	legend(x="topright",col=colForGroups,legend=groupNames,fill=colForGroups, bty="n", title="Groups", cex=.75)
	dev.off()
}


# read 
data <- read.csv("kernel_distances.csv",header=TRUE, as.is=TRUE)

kernels <-  union(as.vector(unique(data[,"kernel1"])),as.vector(unique(data[,"kernel2"])))

distances <- as.data.frame(matrix (data=0, nrow=length(kernels), ncol=length(kernels)))
colnames(distances) <- kernels
rownames(distances) <- kernels



for (i in 1:nrow(data)){
  distances[data[i,"kernel1"],data[i,"kernel2"]] <- data[i,"mismatches"]
  distances[data[i,"kernel2"],data[i,"kernel1"]] <- data[i,"mismatches"]
}



pdf("kernel_cluster.heatmap.pdf", width=5, height=5)
heatmap.2(as.matrix(distances), #RowSideColors=rpcol, ColSideColors=spcol,
#          distfun=function(data){dist(data, method = "euclidian")},
#          hclustfun=function(data){hclust(dist(data), method = "complete")},
          scale="none",
          legend=2,trace="none", density.info="none"#,density.info="none"
          )
dev.off()

pdf("kernel_cluster.tree.pdf", width=4, height=5)
plclust(hclust(as.dist(distances)),hmin=2000)
dev.off()


# Load kernel group labels from file

kernelgroups <- read.csv("kernel_groups.csv",header=TRUE, as.is=TRUE)
groups <- as.data.frame(matrix (data=NA, nrow=length(kernels), ncol=1))
rownames(groups) <- kernels

for (i in 1:nrow(kernelgroups)){
  groups[kernelgroups[i,"kernel"],1] <- kernelgroups[i,"group"]
}

# produce extra plot

drawHeatmap(distances,groups[,1],margin=12,plotFile="kernel_cluster.heatmap.extra.pdf")
