drop view if exists sl_settings;
drop view if exists st_settings;
drop view if exists sst_settings;
drop view if exists pt_settings;
drop view if exists spt_settings;
drop view if exists kbsps_settings;
drop view if exists cosine_settings;
drop view if exists edit_settings;
drop view if exists apg_settings;


create or replace view sl_settings as 
select avg((t.avgs).prec) as avg_prec,stddev((t.avgs).prec)/sqrt(cast(count(*) as numeric)) as prec_stderr,
       avg((t.avgs).recall) as avg_recall,stddev((t.avgs).recall)/sqrt(cast(count(*) as numeric)) as recall_stderr,avg((t.avgs).f1) as avg_f1,
       (kernel||'('||kernel_script||')') as setting,count(*) as cnt
from
(select avgfolds_statistics(ppicvfoldsid) as avgs,kernel,kernel_script from ppicvfolds where kernel='SL' and kernel_script like '%(window)')
as t
group by t.kernel,kernel_script
order by avg_f1 desc;


create or replace view st_settings as 
select avg((t.avgs).prec) as avg_prec,stddev((t.avgs).prec)/sqrt(cast(count(*) as numeric)) as prec_stderr,
       avg((t.avgs).recall) as avg_recall,stddev((t.avgs).recall)/sqrt(cast(count(*) as numeric)) as recall_stderr,avg((t.avgs).f1) as avg_f1,
       (kernel||'(c='||c_preset|| ', j=' || j ||')') as setting,count(*) as cnt
from
(select avgfolds_statistics(ppicvfoldsid) as avgs,c_preset,j,kernel from ppicvfolds where kernel='ST' and kernel_script='lambda:0.4 mu:0.4')
as t
group by t.c_preset,t.j,t.kernel
order by avg_f1 desc;

create or replace view sst_settings as 
select avg((t.avgs).prec) as avg_prec,stddev((t.avgs).prec)/sqrt(cast(count(*) as numeric)) as prec_stderr,
       avg((t.avgs).recall) as avg_recall,stddev((t.avgs).recall)/sqrt(cast(count(*) as numeric)) as recall_stderr,avg((t.avgs).f1) as avg_f1,
       (kernel||'(c='||c_preset|| ', j=' || j ||')') as setting,count(*) as cnt
from
(select avgfolds_statistics(ppicvfoldsid) as avgs,c_preset,j,kernel from ppicvfolds where kernel='SST' and kernel_script='lambda:0.4 mu:0.4')
as t
group by t.c_preset,t.j,t.kernel
order by avg_f1 desc;

create or replace view pt_settings as 
select avg((t.avgs).prec) as avg_prec,stddev((t.avgs).prec)/sqrt(cast(count(*) as numeric)) as prec_stderr,
       avg((t.avgs).recall) as avg_recall,stddev((t.avgs).recall)/sqrt(cast(count(*) as numeric)) as recall_stderr,avg((t.avgs).f1) as avg_f1,
       (kernel||'(c='||c_preset|| ', j=' || j ||')') as setting,count(*) as cnt
from
(select avgfolds_statistics(ppicvfoldsid) as avgs,c_preset,j,kernel from ppicvfolds where kernel='PT' and kernel_script='lambda:0.4 mu:0.4')
as t
group by t.c_preset,t.j,t.kernel
order by avg_f1 desc;

create or replace view spt_settings as 
select avg((t.avgs).prec) as avg_prec,stddev((t.avgs).prec)/sqrt(cast(count(*) as numeric)) as prec_stderr,
       avg((t.avgs).recall) as avg_recall,stddev((t.avgs).recall)/sqrt(cast(count(*) as numeric)) as recall_stderr,avg((t.avgs).f1) as avg_f1,
       (kernel||'(script='||kernel_script|| ', c='|| c_preset || ', j=' || j ||')') as setting,count(*) as cnt
from
(select avgfolds_statistics(ppicvfoldsid) as avgs,c_preset,j,kernel_script,kernel from ppicvfolds where kernel='SpT' and (kernel_script = 'u-spectrum(2)' or kernel_script = 'u-spectrum(2)') 
and c_preset is null or c_preset in (0.0625,1,16,64,512))
as t
group by t.c_preset,t.j,t.kernel,kernel_script
order by avg_f1 desc;


create or replace view kBSPS_settings as 
select avg((t.avgs).prec) as avg_prec,stddev((t.avgs).prec)/sqrt(cast(count(*) as numeric)) as prec_stderr,
       avg((t.avgs).recall) as avg_recall,stddev((t.avgs).recall)/sqrt(cast(count(*) as numeric)) as recall_stderr,avg((t.avgs).f1) as avg_f1,
       (kernel_script||'(led='||led|| ', j=' || j||')') as setting,count(*) as cnt
from
(select avgfolds_statistics(ppicvfoldsid) as avgs,led,j,lmax,lmin,k,kernel_script from ppicvfolds 
where kernel='kBSPS' and kernel_script like '%match=t)')
as t
group by t.kernel_script,t.j,t.led,t.k,t.lmax,t.lmin
order by avg_f1 desc;

create or replace view cosine_settings as 
select avg((t.avgs).prec) as avg_prec,stddev((t.avgs).prec)/sqrt(cast(count(*) as numeric)) as prec_stderr,
       avg((t.avgs).recall) as avg_recall,stddev((t.avgs).recall)/sqrt(cast(count(*) as numeric)) as recall_stderr,avg((t.avgs).f1) as avg_f1,
       (kernel||'(c='||c_preset||', split normalized)') as setting,count(*) as cnt
from
(select avgfolds_statistics(ppicvfoldsid) as avgs,c_preset,kernel from ppicvfolds where kernel='cosine')
as t
group by t.c_preset,kernel
order by avg_f1 desc;

create or replace view edit_settings as 
select avg((t.avgs).prec) as avg_prec,stddev((t.avgs).prec)/sqrt(cast(count(*) as numeric)) as prec_stderr,
       avg((t.avgs).recall) as avg_recall,stddev((t.avgs).recall)/sqrt(cast(count(*) as numeric)) as recall_stderr,avg((t.avgs).f1) as avg_f1,
       (kernel||'(c='||c_preset||', split normalized)') as setting,count(*) as cnt
from
(select avgfolds_statistics(ppicvfoldsid) as avgs,c_preset,kernel from ppicvfolds where kernel='edit')
as t
group by t.c_preset,kernel
order by avg_f1 desc;

create or replace view apg_settings as 
select avg((t.avgs).prec) as avg_prec,stddev((t.avgs).prec)/sqrt(cast(count(*) as numeric)) as prec_stderr,
       avg((t.avgs).recall) as avg_recall,stddev((t.avgs).recall)/sqrt(cast(count(*) as numeric)) as recall_stderr,avg((t.avgs).f1) as avg_f1,
       (kernel||'(c='||c_preset||', split normalized)') as setting,count(*) as cnt
from
(select avgfolds_statistics(ppicvfoldsid) as avgs,c_preset,kernel from ppicvfolds where kernel='APG')
as t
group by t.c_preset,kernel
order by avg_f1 desc;

