#!/usr/bin/python
#nice python run.py 1>run.out 2>run.err
import os
import glob
import subprocess
import re
import time
import math
import sys
import shutil
import locale
import random
locale.setlocale(locale.LC_ALL, '')

sys.path.append('..')
from jsreconfig import *
from addids import *
from convert import *
from distributed import *
from pipe import *
from PerformanceMeasures import computeAUC
from Fscore import *

def readAUC(source):
    predictions = []
    correct = []
    for line in source:
        line = line.strip().split()
        predictions.append(float(line[2]))
        correct.append(float(line[1]))
    auc = computeAUC(predictions, correct)
    return auc



base  = basepath +os.sep +'split' +os.sep
splits  = 'corpus'
trained = 'trained'
predict = 'predict'
result  = base + 'output.sql'

#corpora= os.listdir(base+splits) #Contains all five corpora
trainFiles = [os.path.abspath(x) for x in os.environ["TRAIN_FILES"].split()]
testFiles  = [os.path.abspath(x) for x in os.environ["TEST_FILES" ].split()]

java = os.environ["JAVA"]
nprocs = int(os.environ["NUM_PROCS"])

projname="piped" + str(random.randint(9000, 9999)) # TODO give unique name 
model = base +splits +os.sep +projname +os.sep +"train.model"

#Train a Model
print "Step1-Training"
cmd =[]
regex = re.compile('\D+\d+')
if len(trainFiles) > 0:
    #print corpus
	
	train = base +splits +os.sep +projname +os.sep +"train.pipe"
	os.makedirs(os.path.dirname(train))
	os.mkfifo(train)
	producer = concat(trainFiles, train)

	cmd.append(java + " -classpath '" +jsre_classpath  +"' org.itc.irst.tcc.sre.Train -m 2048 -k SL" +" " +train +" " +model)
	os.chdir(jsre)
	runDistributed(1, cmd) 

	retcode = producer.wait()
	if retcode != 0:
		raise RuntimeException
	os.remove(train)



print "Step 2 predict!"
cmd= []
for test in testFiles:
	print test
	prediction = (base +splits +os.sep +projname +os.sep + os.path.basename(test) + ".predict")
	cmd.append(java + " -classpath '" +jsre_classpath +"' org.itc.irst.tcc.sre.Predict " +test   +" "  +model +" " + prediction )

os.chdir(jsre)
runDistributed(nprocs, cmd)


#print "Step 3 evaluate!"
threshold = 0
resultFile = open(result,'wb')

resultFile.write("-- DATE:  " + time.strftime("%a, %d %b %Y %H:%M:%S +0000", time.gmtime()) + "\n")
resultFile.write("-- TRAIN: " + " ".join(trainFiles) + "\n")
resultFile.write("-- TEST:  " + " ".join(testFiles)  + "\n")

for test in testFiles:
    print test
    prediction = (base +splits +os.sep +projname +os.sep + os.path.basename(test) + ".predict")
    resultFile.write("INSERT INTO ppiSplit (name, corpus, parsertype, parser, kernel, kernel_script) VALUES" +"('" +projname +"','" + test + "','POS tagger','"  +"?" +"','SL'," "'jsre; n=3 (n-gram), w=2 (window)');\n")
    convert(prediction)

    f = open(prediction)
    auc = readAUC(f)
    f.close()

    f = open(prediction)
    F, prec, rec = readResults(f,0)
    f.close()

    print auc, F, prec, rec

    addids(prediction, test) # Replace sequence number with original pair IDs from


    f = open(prediction)
    for line in f:
        line = line.split()
        resultFile.write("INSERT INTO ppiSplitOutput (expId, pair, output, prediction) VALUES (currval('ppiSplit_ppiSplitid_seq'), '" + line[0] + "', '" + ('1' if float(line[2]) >= threshold else '0') + "', " + line[2] +");\n")
    f.close()
    resultFile.write("UPDATE ppiSplit SET auc = " +str(auc) +" , precision_ = " +str(prec) +" , recall = " +str(rec) +", f_measure = " +str(F) +" WHERE ppiSplitId = currval('ppiSplit_ppiSplitid_seq');\n")


resultFile.close


