#!/bin/bash

#./CC_syntree_core.sh '${expType}' '${corpus}' '${j}' '${c}' '${lambda}' '${mu}'  '${inDir}' '${learner}' '${kernel}' '${tester}'

expType=$1; shift 1
corpus=$1; shift 1 
j=$1; shift 1
c=$1; shift 1
lambda=$1; shift 1
mu=$1; shift 1
inDir=$1; shift 1
learner=$1; shift 1
kernel=$1; shift 1
tester=$1; shift 1

trains='IEPA HPRD50 LLL AIMed BioInfer'

. kernelparams.sh

set -e

outDir=./${kerneltext}/${expType}/${corpus}/out-${expType}-${corpus}-j${j}-c${c}-L${lambda}-M${mu}
tmpDir=./${kerneltext}/${expType}/${corpus}/tmp-${expType}-${corpus}-j${j}-c${c}-L${lambda}-M${mu}

set +e
	rm -fR ${outDir}
	mkdir -p ${outDir}
	rm -fR ${tmpDir}
	mkdir -p ${tmpDir}
set -e

logFile=${outDir}/${expType}-${corpus}-j${j}-c${c}-L${lambda}-M${mu}.log
echo "started: " | tee ${logFile}
date -R | tee -a ${logFile}

learnCorpus=${tmpDir}/learn.txt
cp ${inDir}/${corpus}.txt.id ${learnCorpus}
model=${outDir}/model.txt

traindata=${outDir}/${expType}-${corpus}-q${q}-j${j}-c${c}
${learner} -v -1 -t 5 -F ${kernel} -D ppi${expType} -C T -c ${c} -j ${j} -L ${lambda} -M ${mu} ${learnCorpus} ${model} >> ${traindata}
# remove train set from tests
tests=${trains/$corpus/}	

for test in ${tests}
do
	evalFile=${outDir}/${expType}-${corpus}-j${j}-c${c}-L${lambda}-M${mu}.sql
	echo "------------- extType=${expType}, corpus=${corpus}, test=${test}, parser=${combo}, kernel=${kerneltext}(${kernel}), c=${c}, j=${j}, lambda=${lambda}, mu=${mu}" | tee -a ${logFile}

	testCorpus=${tmpDir}/test.txt
	cp ${inDir}/${test}.txt.id ${testCorpus}
	
	echo -n "insert into ppi${expType} (corpus, test, parsertype, parser, kernel, c, j, kernel_script) values (" >> ${evalFile}
	echo "'${corpus}', '${test}', '${parsertype}', '${combo}', '${kerneltext}', ${c}, ${j}, 'lambda:${lambda} mu:${mu}');" >> ${evalFile}
	cat ${traindata} >> ${evalFile}
	#learn
	${tester} -v -1 -D ppi${expType} ${testCorpus} ${model} >> ${evalFile}
	sed -i 's/nan/0.0/g' ${evalFile}
done
set +e
	rm -fr ${tmpDir}
	rm -fr ${traindata}
set -e

echo "finished: " | tee -a ${logFile}
date -R | tee -a ${logFile}
