#!/bin/bash

#./CV_syntree_core.sh '${expType}' '${corpus}' '${j}' '${c}' '${lambda}' '${mu}'  '${inDir}' '${learner}' '${kernel}' '${tester}' 

expType=$1; shift 1
corpus=$1; shift 1 
j=$1; shift 1
c=$1; shift 1
lambda=$1; shift 1
mu=$1; shift 1
inDir=$1; shift 1
learner=$1; shift 1
kernel=$1; shift 1
tester=$1; shift 1

folds='0 1 2 3 4 5 6 7 8 9'
useFolds='0 1 2 3 4 5 6 7 8 9'

. kernelparams.sh

set -e

outDir=./${kerneltext}/${expType}/${corpus}/out-${expType}-${corpus}-j${j}-c${c}-L${lambda}-M${mu}
tmpDir=./${kerneltext}/${expType}/${corpus}/tmp-${expType}-${corpus}-j${j}-c${c}-L${lambda}-M${mu}

set +e
	rm -fR ${outDir}
	mkdir -p ${outDir}
set -e

logFile=${outDir}/${expType}-${corpus}-j${j}-c${c}-L${lambda}-M${mu}.log
echo "started: " | tee ${logFile}
date -R | tee -a ${logFile}

evalFile=${outDir}/${expType}-${corpus}-j${j}-c${c}-L${lambda}-M${mu}.sql

for fold in ${useFolds};
do
	echo "------------- extType=${expType}, corpus=${corpus}, parser=${combo}, kernel=${kerneltext}(${kernel}), c=${c}, j=${j}, lambda=${lambda}, mu=${mu}, fold=${fold}" | tee -a ${logFile}
	set +e
	rm -fR ${tmpDir}
	mkdir -p ${tmpDir}
	set -e
	learnCorpus=${tmpDir}/learn.txt
	testCorpus=${tmpDir}/test.txt

	# delete the current fold from folds
	learnFolds=${folds/$fold/}
	for learnFold in ${learnFolds};
	do
		cat ${inDir}/${learnFold}.txt.id >> ${learnCorpus}
	done
	cp ${inDir}/${fold}.txt.id ${testCorpus}
	model=${outDir}/${fold}-model.txt

	echo -n "insert into ppi${expType} (corpus, parsertype, parser, kernel, c, j, fold, kernel_script) values (" >> ${evalFile}
	echo "'${corpus}', '${parsertype}', '${combo}', '${kerneltext}', ${c}, ${j}, ${fold}, 'lambda:${lambda} mu:${mu}');" >> ${evalFile}
	#learn
	${learner} -v -1 -t 5 -F ${kernel} -D ppi${expType} -C T -c ${c} -j ${j} -L ${lambda} -M ${mu} ${learnCorpus} ${model} >> ${evalFile}
	#eval
	${tester} -v -1 -D ppi${expType} ${testCorpus} ${model} >> ${evalFile}
	sed -i 's/nan/0.0/g' ${evalFile}
done
set +e
	rm -fr ${tmpDir}
set -e

echo "finished: " | tee -a ${logFile}
date -R | tee -a ${logFile}

