#!/bin/bash

#./CC_SpT_core.sh '${expType}' '${corpus}' '${q}' '${c}' '${j}' '${u}' '${inDir}' '${learner}' '${tester}'

expType=$1; shift 1
corpus=$1; shift 1 
q=$1; shift 1
c=$1; shift 1
j=$1; shift 1
u=$1; shift 1
inDir=$1; shift 1
learner=$1; shift 1
tester=$1; shift 1

trains='AIMed BioInfer IEPA HPRD50 LLL'

. kernelparams.sh

set -e

outDir=./${expType}/${corpus}/out-${expType}-${corpus}-q${q}-j${j}-c${c}
tmpDir=./${expType}/${corpus}/tmp-${expType}-${corpus}-q${q}-j${j}-c${c}

set +e
	rm -fR ${outDir}
	mkdir -p ${outDir}
	rm -fR ${tmpDir}
	mkdir -p ${tmpDir}
set -e

logFile=${outDir}/${expType}-${corpus}-q${q}-j${j}-c${c}.log
echo "started: " | tee ${logFile}
date -R | tee -a ${logFile}

learnCorpus=${tmpDir}/learn.txt
cp ${inDir}/${corpus}.txt.id ${learnCorpus}
model=${outDir}/model.txt
traindata=${outDir}/${expType}-${corpus}-q${q}-j${j}-c${c}
${learner} -v -1 -t ${kernel} -D ppi${expType} -u "${u},q=${q}" -c ${c} -j ${j} ${learnCorpus} ${model} >> ${traindata}

# remove train set from tests
tests=${trains/$corpus/}	

for test in ${tests}
do
	echo "------------- corpus=${corpus}, test=${test}, combo=${combo}, c=${c}, j=${j}, q=${q}, u=${u}" | tee -a ${logFile}
	testCorpus=${tmpDir}/test.txt
	cp ${inDir}/${test}.txt.id ${testCorpus}

	evalFile=${outDir}/${expType}-${corpus}-${test}-q${q}-j${j}-c${c}.sql
	echo -n "insert into ppi${expType} (corpus, test, parsertype, parser, kernel, c, j, kernel_script) values (" >> ${evalFile}
	echo "'${corpus}', '${test}', '${parsertype}', '${combo}', '${kerneltext}', ${c}, ${j}, '${kernelscript}');" >> ${evalFile}
	cat ${traindata} >> ${evalFile}

	${tester} -v -1 -D ppi${expType} ${testCorpus} ${model} >> ${evalFile}

	sed -i 's/nan/0.0/g' ${evalFile}
done
set +e
	rm -fr ${tmpDir}
	rm -fr ${traindata}
set -e

echo "finished: " | tee -a ${logFile}
date -R | tee -a ${logFile}

