#!/bin/bash

# ./CV_kBSPS_core.sh '${expType}' '${corpus}' '${j}' '${n}' '${t}' '${L}' '${E}' '${D}' '${l}' '${e}' '${d}' '${inDir}' '${learner}' '${tester}' '${kParam}'

expType=$1; shift 1
corpus=$1; shift 1
j=$1; shift 1
n=$1; shift 1
t=$1; shift 1
L=$1; shift 1
E=$1; shift 1
D=$1; shift 1
l=$1; shift 1
e=$1; shift 1
d=$1; shift 1
inDir=$1; shift 1
learner=$1; shift 1
tester=$1; shift 1
kParam=$1; shift 1

folds='0 1 2 3 4 5 6 7 8 9'
useFolds='0 1 2 3 4 5 6 7 8 9'

. kernelparams.sh

set -e

outDir=./${expType}/${corpus}/out-${expType}-${corpus}-${kParam}-${j}-LED-${L}${E}${D}${l}${e}${d}
tmpDir=./${expType}/${corpus}/tmp-${expType}-${corpus}-${kParam}-${j}-LED-${L}${E}${D}${l}${e}${d}

LEDled="L=${L},E=${E},D=${D},l=${l},e=${e},d=${d}"
lmin=${kParam:0:1}
lmax=${kParam:3:1}
k=${kParam:6:1}
kernelscript="k-band(${kParam:0:4} k${k} match=${t})"

set +e
rm -fr ${outDir}
mkdir -p ${outDir}
set -e
evalFile=${outDir}/${expType}-${corpus}-${kParam}-${j}-LED-${L}${E}${D}${l}${e}${d}.sql

logFile=${outDir}/${expType}-${corpus}-${kParam}-${j}-LED-${L}${E}${D}${l}${e}${d}.log
echo "started: " | tee ${logFile}
date -R | tee -a ${logFile}

for fold in ${useFolds};
do
	echo "------------- expType=${expType}, corpus=${corpus}, kernel=${kerneltext}, kParam = ${kParam}, j=${j}, n=${n}, t=${t}, ${LEDled}, fold=${fold}" | tee -a ${logFile}
	set +e
	rm -fr ${tmpDir}
	mkdir -p ${tmpDir}
	set -e
	learnCorpus=${tmpDir}/learn.txt
	testCorpus=${tmpDir}/test.txt

	# delete the current fold from folds
	learnFolds=${folds/$fold/}

	for learnFold in ${learnFolds};
	do
		cat ${inDir}/${learnFold}.txt >> ${learnCorpus}
	done
	cp ${inDir}/${fold}.txt ${testCorpus}
	model=${outDir}/${fold}-model.txt

	echo -n "insert into ppi${expType} (corpus, parsertype, parser, kernel, j, lmax, lmin, k, match_, fold, normalized, input_format, led, c, kernel_script) values (" >> ${evalFile}
	echo "'${corpus}', '${parsertype}', 'Charniak-Lease+Stanford converter', '${kerneltext}', ${j}, ${lmax}, ${lmin}, ${k}, '${t}', ${fold}, '${n}', '${inputFormat}', '${LEDled}', 0, '${kernelscript}');" >> ${evalFile}
	#learn
	${learner} -v -1 -D ppi${expType} -t ${kernel} -u "f=${inputFormat},N=${n},a=${lmax},i=${lmin},k=${k},t=${t},${LEDled}" -j ${j} -m 10000 ${learnCorpus} ${model} >> ${evalFile}
	#eval
	${tester} -v -1 -D ppi${expType} ${testCorpus} ${model} >> ${evalFile}
	sed -i 's/nan/0.0/g' ${evalFile}
done
set +e
	rm -fr ${tmpDir}
set -e

echo "finished: " | tee -a ${logFile}
date -R | tee -a ${logFile}