/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package client;

import auc.AUCCalculator;
import corpus.Text;
import corpus.ppi.CorpusFactory;
import deeper.Interaction;
import deeper.InteractionType;
import edu.uci.ics.jung.graph.Edge;
import graph.Keys;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import machinelearning.features.DepFeatureBuilder;
import machinelearning.features.FeatureBuilder;
import machinelearning.kernel.Kernel;
import machinelearning.kernel.SyntacticKernel;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import preprocessing.DefaultPreprocessor;
import util.PPIMapping;
import util.PropertyManager;

/**
 *
 * @author tfayruzo
 */
public class OtherStuff {

    private static Log _log = LogFactory.getLog(OtherStuff.class);
    private PropertyManager pm;

    //needed one time to convert AIMed folds to 'normal' representation
    private void convertAIMedFolds() {
        PPIMapping m = new PPIMapping("AIMed");
        File foldsDir = new File(pm.getParam("splits"));
        File[] testFiles = foldsDir.listFiles(new FileFilter() {

            public boolean accept(File pathname) {
                return pathname.getName().contains("test");
            }
        });
        int i = 0;
        for (File file : testFiles) {
            try {
                PrintWriter writer = new PrintWriter(pm.getParam("splits") + File.separator + "test" + i);
                try {
                    BufferedReader reader = new BufferedReader(new FileReader(file));
                    String s;
                    while ((s = reader.readLine()) != null) {
                        if (!s.equals("")) {
                            List<String> list = m.invMap(s);
                            for (String id : list) {
                                writer.println(id.substring(0, id.length() - 1));
                            }
                        }
                    }
                } catch (FileNotFoundException e) {
                    e.printStackTrace();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                writer.close();
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
            i++;
        }
    }

    private void numberOfInteractions() throws Exception {
        CorpusFactory cFactory = new CorpusFactory();
        cFactory.suppressDirection = true;
        Text corpus = cFactory.getCorpusFromPPIXML(pm.getParam("data"), new DefaultPreprocessor());
        corpus.parse(null, "serialized", pm.getParam("parse"));
        Map<String, Integer> map = new HashMap<String, Integer>();
        for (Interaction i : corpus.interactions()) {
            String key = i.status.toString();
            Integer val = map.get(key);
            if (val == null) {
                val = new Integer(0);
            }
            map.put(key, val + 1);
        }
        for (String key : map.keySet()) {
            System.out.println(key + "\t" + map.get(key));
        }
    }

    private void refreshIds() throws Exception {
        CorpusFactory cFactory = new CorpusFactory();
        Text corpus = cFactory.getCorpusFromPPIXML(pm.getParam("data"), new DefaultPreprocessor());
        corpus.parse(null, "serialized", pm.getParam("parse"));
        Collection<Interaction> full = corpus.interactions();
        Map<String, List<Interaction>> map = new HashMap<String, List<Interaction>>();
        //combine interactions per document in the map
        for (Interaction i : full) {
            String[] split = i.id.split("\\.");
            String key = split[0] + "." + split[1] + ".";
            List<Interaction> list = map.get(key);
            if (list == null) {
                list = new ArrayList<Interaction>();
                map.put(key, list);
            }
            list.add(i);
        }
        File[] splits = new File(pm.getParam("splits")).listFiles();
        PrintWriter writer = null;
        try {
            writer = new PrintWriter(pm.getParam("ids"));
        } catch (FileNotFoundException e1) {
            e1.printStackTrace();
        }
        for (File file : splits) {
            try {
                BufferedReader reader = new BufferedReader(new FileReader(file));
                String s;
                while ((s = reader.readLine()) != null) {
                    if (!s.equals("")) {
                        List<Interaction> list = map.get(s);
                        for (Interaction i : list) {
                            writer.println(i.id);
                        }
                    }
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        writer.close();
    }

    public void splitBuilder() {
        int numFolds = 10;
        int numDocs = 1100;
        for (int i = 0; i < numFolds; i++) {
            int num = i;
            try {
                PrintWriter writer = new PrintWriter(pm.getParam("splits") + File.separator + "test" + Integer.toString(i + 1));
                while (num < numDocs) {
                    writer.println("BioInfer.d" + num);
                    num += 10;
                }
                writer.close();
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
        }
    }

    /*public void lengthAnalysis() {
        CorpusFactory cFactory = new CorpusFactory();
        Text corpus = cFactory.getCorpusFromPPIXML(pm.getParam("data"), new DefaultPreprocessor());
        corpus.parse(null, "serialized", pm.getParam("parse"));
        Collection<Interaction> full = corpus.interactions();
        Map<String, String> pred = new HashMap<String, String>();
        try {
            BufferedReader reader = new BufferedReader(new FileReader(pm.getParam("prediction")));
            String s;
            while ((s = reader.readLine()) != null) {
                String[] split = s.split("\\s+");
                pred.put(split[0], split[1]);
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        Kernel<Interaction> kernel = new SyntacticKernel();
        PrintWriter writer = null;
        Map<Interaction, List<Double>> result = new HashMap<Interaction, List<Double>>();
        try {
            writer = new PrintWriter(pm.getParam("analysis"));
            for (Interaction i1 : full) {
                i1.predClass = pred.get(i1.id);
                // 'values' collects different length subtree matches
                List<Double> values = new ArrayList<Double>();
                values.add(0.0);
                for (Interaction i2 : full) {
                    writer.print(i1.id + "\t" + i2.id + "\t");
                    double kern = kernel.compute(i1, i2);
                    writer.print(kern);
                    String dist = ((SyntacticKernel) kernel).getDistribution();
                    writer.println(dist);
                    String[] split = dist.split("\t");
                    values.set(0, values.get(0) + kern);
                    for (String part : split) {
                        if (part.equals("")) {
                            continue;
                        }
                        String[] split2 = part.split("\\s");
                        int length = Integer.parseInt(split2[0]);
                        int amount = Integer.parseInt(split2[1]);
                        for (int i = values.size(); i < length + 1; i++) {
                            values.add(0.0);
                        }
                        values.set(length, values.get(length) + amount);
                    }
                }
                result.put(i1, values);
            }
            writer.close();
            writer = new PrintWriter(pm.getParam("distribution"));
            NumberFormat nf = NumberFormat.getNumberInstance();
            nf.setMaximumFractionDigits(3);
            List<Interaction> keys = new ArrayList<Interaction>(result.keySet());
            Collections.sort(keys, new InteractionComparator());
            double[][] avg = new double[5][15];
            double[] len = new double[5];
            double[] kern = new double[5];
            double size = (double) keys.size();
            for (Interaction key : keys) {
                String type = key.getType();
                int i = 0;
                if (type.equals("TP")) {
                    i = 0;
                } else if (type.equals("TN")) {
                    i = 1;
                } else if (type.equals("FP")) {
                    i = 2;
                } else if (type.equals("FN")) {
                    i = 3;
                } else if (type.equals("WD")) {
                    i = 4;
                }
                writer.print(type + " " + key.id + "\t");
                List<Double> values = result.get(key);
                avg[i][0]++;
                len[i] += key.depPath == null ? 0 : key.depPath.length;
                double divisor = values.get(0);
                kern[i] += divisor;
                for (int j = 1; j < values.size(); j++) {
                    avg[i][j] += values.get(j) / size;
                    writer.print(nf.format(values.get(j) / divisor) + "\t");
                }
                writer.println();
            }
            for (int i = 0; i < 5; i++) {
                switch (i) {
                    case 0:
                        writer.print("TP\t");
                        break;
                    case 1:
                        writer.print("TN\t");
                        break;
                    case 2:
                        writer.print("FP\t");
                        break;
                    case 3:
                        writer.print("FN\t");
                        break;
                    case 4:
                        writer.print("WD\t");
                        break;
                }
                writer.print(avg[i][0] + "\t");
                writer.print(nf.format(kern[i] / avg[i][0] / size) + "\t");
                writer.print(nf.format(len[i] / avg[i][0]) + "\t");
                for (int j = 1; j < 15; j++) {
                    writer.print(nf.format(avg[i][j] / avg[i][0]) + "\t");
                }
                writer.println();
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } finally {
            writer.close();
        }
    }*/

    public void featureVectorBuilder() throws Exception {
        CorpusFactory cFactory = new CorpusFactory();
        Text corpus = cFactory.getCorpusFromPPIXML(pm.getParam("data"), new DefaultPreprocessor());
        corpus.parse(null, "serialized", pm.getParam("parse"));
        Collection<Interaction> full = corpus.interactions();
        Map<String, List<Interaction>> map = new HashMap<String, List<Interaction>>();
        for (Interaction i : full) {
            String[] split = i.id.split("\\.");
            String key = split[0] + "." + split[1] + ".";
            List<Interaction> list = map.get(key);
            if (list == null) {
                list = new ArrayList<Interaction>();
                map.put(key, list);
            }
            list.add(i);
        }

        File foldsDir = new File(pm.getParam("splits"));
        File[] testFiles = foldsDir.listFiles();
        int numFolds = testFiles.length;
        ArrayList<String>[] folds = new ArrayList[numFolds];
        int i = 0;
        for (File testFile : testFiles) {
            ArrayList<String> fold = new ArrayList<String>();
            try {
                BufferedReader reader = new BufferedReader(new FileReader(testFile));
                String s;
                while ((s = reader.readLine()) != null) {
                    fold.add(s.concat("."));
                }
                reader.close();
                folds[i] = fold;
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
            i++;
        }
        try {
            FeatureBuilder fb = new DepFeatureBuilder();
            for (int k = 0; k < numFolds; k++) {
                PrintWriter writer = new PrintWriter(pm.getParam("trainCVFeatures") + k);
                PrintWriter writer2 = new PrintWriter(pm.getParam("trainCVFeatClassValues") + k);
                List<Interaction> test = new ArrayList<Interaction>(), train = new ArrayList<Interaction>();
                for (String doc : folds[k]) {
                    if (map.get(doc) == null) {
                        _log.error("Document " + doc + " not found");
                    } else {
                        test.addAll(map.get(doc));
                    }
                }
                for (int j = 0; j < numFolds; j++) {
                    if (k != j) {
                        for (String doc : folds[j]) {
                            if (map.get(doc) == null) {
                                _log.error("Document " + doc + " not found");
                            } else {
                                train.addAll(map.get(doc));
                            }
                        }
                    }
                }
                i = 1;
                for (Interaction i1 : train) {
                    double[] v = fb.getFeatureVector(i1).toDoubleArray();
                    writer.print(i1.status.ordinal());
                    writer2.println(i1.status.ordinal());
                    writer.print("\t");
                    writer.print("0:" + Integer.toString(i));
                    for (int j = 0, size = v.length; j < size; j++) {
                        writer.print("\t");
                        writer.print(j + 1);
                        writer.print(":");
                        writer.print(v[j]);
                    }
                    writer.print("\n");
                    i++;
                }
                writer.close();
                writer2.close();
                writer = new PrintWriter(pm.getParam("testCVFeatures") + k);
                writer2 = new PrintWriter(pm.getParam("testCVFeatClassValues") + k);
                i = 1;
                for (Interaction i1 : test) {
                    double[] v = fb.getFeatureVector(i1).toDoubleArray();
                    writer.print(i1.status.ordinal());
                    writer2.println(i1.status.ordinal());
                    writer.print("\t");
                    writer.print("0:" + Integer.toString(i));
                    writer.print("\t");
                    for (int j = 0, size = v.length; j < size; j++) {
                        writer.print("\t");
                        writer.print(j + 1);
                        writer.print(":");
                        writer.print(v[j]);
                    }
                    writer.print("\n");
                    i++;
                }
                writer.close();
                writer2.close();
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }

    }

    public void contentAnalysis(boolean directed) throws Exception {
        CorpusFactory cFactory = new CorpusFactory();
        Text corpus = cFactory.getCorpusFromPPIXML(pm.getParam("data"), new DefaultPreprocessor());
        corpus.parse(null, "serialized", pm.getParam("parse"));
        Collection<Interaction> full = corpus.interactions();
        int numClasses = directed ? 5 : 3;
        Map<String, Integer>[] comps = new HashMap[numClasses];
        double[] amount = new double[numClasses];
        double[] length = new double[numClasses];	//for counting average length
        //for counting median length
        List<Integer>[] length2 = new ArrayList[numClasses];
        try {
            PrintWriter writer = new PrintWriter(pm.getParam("contentStat"));
            for (Interaction i : full) {
                if (i.depPath == null) {
                    continue;
                }
                int index = i.status.ordinal();
                if (!directed && index > 2) {
                    index = 0;
                }
                Map<String, Integer> map = comps[index];
                amount[index]++;
                length[index] += i.depPath.numEdges();
                if (length2[index] == null) {
                    length2[index] = new ArrayList<Integer>();
                }
                length2[index].add(i.depPath.numEdges());
                if (map == null) {
                    map = new HashMap<String, Integer>();
                    comps[index] = map;
                }
                for (Iterator it = i.depPath.getEdges().iterator(); it.hasNext();) {
                    Edge e = (Edge) it.next();
                    String key = (String) e.getUserDatum(Keys.LABEL);
                    Integer count = map.get(key);
                    if (count == null) {
                        count = new Integer(0);
                    }
                    map.put(key, count + 1);
                }
            }
            NumberFormat nf = NumberFormat.getInstance();
            nf.setMaximumFractionDigits(3);
            for (int i = 0; i < comps.length; i++) {
                if (comps[i] == null) {
                    continue;
                }
                Collections.sort(length2[i]);
                writer.println(InteractionType.values()[i].toString() + "\t Avg len: " + length[i] / amount[i] + "Med len: " + length2[i].get(length2[i].size() / 2));
                for (String key : comps[i].keySet()) {
                    writer.println(key + "\t" + nf.format(comps[i].get(key) / amount[i]));
                }
            }
            writer.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    public void nestedProteinsInfo() {
        CorpusFactory cFactory = new CorpusFactory();
        Text lll = cFactory.getCorpusFromPPIXML("resource/bioinfer.xml", new DefaultPreprocessor());
    }

    public void calculateROC() {
        List<String> list = readList(pm.getParam("prediction"));
        List<String> result = new ArrayList<String>();
        for (String s : list) {
            String[] split = s.split("\t");
            //temporary hack to make '1' positve label and '0' - negative
            split[1] = Double.toString(1 - Double.parseDouble(split[1]));
            result.add(split[1] + "\t" + split[3]);
        }
        Collections.sort(result, new ProbablitiesComparator());
        try {
            PrintWriter writer = new PrintWriter(pm.getParam("rocSource"));
            for (String s : result) {
                writer.append(s + "\n");
            }
            writer.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        AUCCalculator auc = new AUCCalculator();
        auc.main(new String[]{pm.getParam("rocSource"), "list"});
    }

    class InteractionComparator implements Comparator<Interaction> {

        public int compare(Interaction o1, Interaction o2) {
            return o1.getType().compareTo(o2.getType());
        }
    }

    class ProbablitiesComparator implements Comparator<String> {

        public int compare(String o1, String o2) {
            double p1 = Double.parseDouble(o1.split("\t")[0]);
            double p2 = Double.parseDouble(o2.split("\t")[0]);
            if (p1 == p2) {
                return 0;
            }
            if (p1 < p2) {
                return 1;
            }
            return -1;
        }
    }

    private List<String> readList(String path) {
        BufferedReader reader;
        List<String> list = new ArrayList<String>();
        try {
            reader = new BufferedReader(new FileReader(path));
            String s;
            while ((s = reader.readLine()) != null) {
                list.add(s);
            }
            reader.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return list;
    }
}
