package client;

import controllers.ParseSerializer;
import jargs.gnu.CmdLineParser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


import preprocessing.DefaultPreprocessor;
import util.PropertyManager;
import util.Params;
import util.Params.ExperimentType;
import util.Params.OutputType;
import weka.core.FindWithCapabilities;
import corpus.Text;
import corpus.ppi.CorpusFactory;

import machinelearning.SVM;
import machinelearning.evaluation.EvaluationProcedure;
import machinelearning.kernel.Kernel;
import deeper.Interaction;
import deeper.InteractionType;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Locale;
import java.util.Set;
import java.util.TreeMap;
import machinelearning.evaluation.PerformanceMeasure;
/** TODO 
 * type of LearningFormat
 * @author user
 *
 */

public class Runners {

    private static Log _log = LogFactory.getLog(Runners.class);
    private PropertyManager pm;
    
    public class MyStringCompare implements Comparator<String> {
    
    	Matcher m = Pattern.compile("([0-9]+)(\\.txt)?$").matcher("");
    	@Override
    	public int compare(String o1, String o2) {
			m.reset( o1 );
			if ( ! m.find())
				throw new IllegalArgumentException("Unrecognized file name: " + o1);
			int i1 = Integer.parseInt(m.group( 1 ));
			m.reset( o2 );
			if ( ! m.find())
				throw new IllegalArgumentException("Unrecognized file name: " + o2);
			int i2 = Integer.parseInt(m.group( 1 ));
			
			return (i1 - i2);
		}
    }
    
    public Runners(String props) {
        pm = new PropertyManager();
        pm.readParam(props);
    }

    public void setProps(String props) {
        pm = new PropertyManager();
        pm.readParam(props);
    }

    public static class SvnExcludeFilter implements FilenameFilter {
        public boolean accept(File dir, String name) {
            return !(name.endsWith(".svn"));
        }
    }
    
    static public boolean deleteDirectory(String path)
    {
    	return deleteDirectory(new File(path));    	
    }
    
    public BufferedReader checkReadFile(String param, int k) throws FileNotFoundException 
    {
    	if ( param != null ) {
    		if ( k== -1 ) 
    			return (new BufferedReader(new FileReader(param)));
    		else
    			return (new BufferedReader(new FileReader(param + k)));
    	}
        else
        	throw new RuntimeException("No '"+ param + "' is given propsfile");
    }
    
    public BufferedReader checkReadFile(String param) throws FileNotFoundException 
    {
    	return checkReadFile( param, -1);    	
    }

    
    public PrintWriter checkWriteFile(String param, int k) throws FileNotFoundException 
    {
    	if ( param != null ) {
    		if ( k == -1)
    			return (new PrintWriter( param ));
    		else
    			return (new PrintWriter( param + k));
    	}
        else
        	throw new RuntimeException("No '"+ param + "' is given propsfile");
    }
    
    public PrintWriter checkWriteFile(String param) throws FileNotFoundException 
    {
    	return checkWriteFile(param, -1 );
    }

    private Set<Integer> preparePosLabels()
    {
    	Set<Integer> posLabels = new HashSet<Integer>();
        posLabels.add(InteractionType.REALSTRAIGHT.ordinal());
        posLabels.add(InteractionType.REALINVERSE.ordinal());
        posLabels.add(InteractionType.REAL.ordinal());
        return posLabels;
    }
    
    public void writeResultDB(String outFileFold, List<String> realLabels)
    {
    	writeResultDB(outFileFold, realLabels, null );
    }
    
    public void writeResultDB(String outFileFold, List<String> realLabels, List<String> ids)
    {	Set<Integer> posLabels = preparePosLabels();
    	if (Params.outputType.equals(OutputType.DB)){
    	EvaluationProcedure eval = new EvaluationProcedure(outFileFold);
    	PerformanceMeasure performance = eval.basicPerformanceMeasures(realLabels, posLabels, ids);
    	Params.sqlFile.println("update " + Params.dbName + 
    			" set tp = " + performance.truePositives + 
    			", fn = " + performance.falseNegatives +
    			", tn = " + performance.trueNegatives +
    			", fp = " + performance.falsePositives +
    			", total = " + performance.getTotal() +
    			", auc = " + (float)performance.getAUC() +
    			", precision_ = " + (float)performance.getPrecision() +
    			", recall = " + (float)performance.getRecall() +
    			", f_measure = " + (float)performance.getFMeasure() +
    			" where " +  Params.dbName + "id = " + Params.currID() + ";");
    	}
    }

    
    public void writeResultsText(String outFile, List<String> realLabels) throws FileNotFoundException
    {
    	Set<Integer> posLabels = preparePosLabels();
    	EvaluationProcedure eval = new EvaluationProcedure(outFile);
        double step = Params.thresholdStep;
	    PrintWriter results = checkWriteFile( Params.results);
	    results.write("Th\tRecall\tprecision\tfMeasure" + System.getProperty("line.separator"));
	    final int steps = Math.round((float)(1.0/step));
        final double min = -0.0001; 
        final double max = 1.0+min;
        for ( int i = 0; i<=steps; i++) {
        	final double th = i/(double)steps*(max-min)+min;
            PerformanceMeasure performance = eval.getPrediction4threshold(realLabels, posLabels, th);
            results.write((float)th + "\t" + (float)performance.getRecall() + "\t" + (float)performance.getPrecision() + "\t" + (float)performance.getFMeasure() + System.getProperty("line.separator"));
        }
	    results.close();
    }
    
    static public boolean deleteDirectory(File path) {
        if( path.exists() ) {
          File[] files = path.listFiles();
          for(int i=0; i<files.length; i++) {
             if(files[i].isDirectory()) {
               deleteDirectory(files[i]);
             }
             else {
               files[i].delete();
             }
          }
        }
        return( path.delete() );
      }
    
    public void SVMCV() {
        File foldsDir = new File(Params.splits);
        File[] foldsFiles = foldsDir.listFiles(new SvnExcludeFilter());
        int numFolds = foldsFiles.length;
        String[] trainFolds = new String[numFolds];
        String[] testFolds = new String[numFolds];
        String[] idFolds = new String[numFolds];
        String[] labelFolds = new String[numFolds];
        for (int i = 0; i < numFolds; i++) {
            trainFolds[i] = Params.trainCVKernel + i;
            testFolds[i] = Params.testCVKernel + i;
            labelFolds[i] = Params.testLabels + i;
            idFolds[i] = Params.ids + i;
        }
        Arrays.sort(trainFolds, new MyStringCompare());
        Arrays.sort(testFolds, new MyStringCompare());
        Arrays.sort(labelFolds, new MyStringCompare());
        Arrays.sort(idFolds, new MyStringCompare());
//        String userDir = System.getProperty("user.dir");
        SVM svm = new SVM( Params.modelFile, Params.SVMtrainexe, Params.SVMtestexe);
        List<String> realLabels = new ArrayList<String>();
        List<String> ids = new ArrayList<String>();
        PrintWriter writer = null;
        try {
            writer = new PrintWriter(Params.outFile);
        } catch (FileNotFoundException ex) {
            Logger.getLogger(Runners.class.getName()).log(Level.SEVERE, null, ex);
        }
        for (int k = 0; k < numFolds; k++) {
        	Params.fold = k;
            String outFileFold = Params.outFile + ".fold"+k;
            svm.trainOnPrecomp(trainFolds[k]);
            svm.testOnPrecomp(testFolds[k], outFileFold);
            try {
                BufferedReader predictionReader = new BufferedReader(new FileReader(outFileFold));
                BufferedReader labelReader = new BufferedReader(new FileReader(labelFolds[k]));
                BufferedReader idReader = new BufferedReader(new FileReader(idFolds[k]));
                Params.addLog(2, "train: " + trainFolds[k] + ", test: " + testFolds[k]+ 
                		", test: " + testFolds[k] + ", labelFolds: " + labelFolds[k] 
                		+ ", ids: " + idFolds[k]);
                String s = "";
                while ((s = predictionReader.readLine()) != null) {
                    writer.write(s + System.getProperty("line.separator"));
                }
                while ((s = labelReader.readLine()) != null) {
                    realLabels.add(s);
                }
                while ((s = idReader.readLine()) != null) {
                    ids.add(s);
                }
                labelReader.close();
                idReader.close();
            
                predictionReader.close();
                if ( Params.errorAnalysis ) 
					writeResultDB(outFileFold, realLabels, ids );
				else
					writeResultDB(outFileFold, realLabels);
                // delete temporary model and prediction files
                if ( Params.verbose < 0 )
                {                	
	                if ( new File(outFileFold).delete() ) 
	                	Params.addLog(-1, "prediction file deleted " + outFileFold);
	        		else
	        			Params.addLog(-1, "prediction file not deleted " + outFileFold);
                }
                svm.removeSVMTempFile();
                ids.clear();
                realLabels.clear();
            } catch (IOException e) {
                Logger.getLogger(Runners.class.getName()).log(Level.SEVERE, null, e);
            }
        }
        writer.close();
        if (!(Params.outputType.equals(OutputType.DB)))
        {
        	try {
				writeResultsText(Params.outFile, realLabels);
			} catch (FileNotFoundException e) {
				Logger.getLogger(Runners.class.getName()).log(Level.SEVERE, null, e);
			}
        }
     if ( Params.verbose < 0 )
     {
    	 if ( new File(Params.outFile).delete() )
    	 	Params.addLog(-1, "prediction file deleted " + Params.outFile);
    	 else
			Params.addLog(-1, "prediction file not deleted " + Params.outFile);
     }
    }

    /*
     * Method builds kernels for a cross-validation setup based on 'folds'
     * parameter
     *
     * parallel - optional parameter, for parallel computation
     */
    @SuppressWarnings("unchecked")
	public void buildCVkernel(boolean suppressDirection) throws Exception {
        CorpusFactory cFactory = new CorpusFactory();
        cFactory.suppressDirection = suppressDirection;
        Text corpus = cFactory.getCorpusFromPPIXML(Params.data, new DefaultPreprocessor());
        File parseDir = new File(Params.parse);
        parseDir.mkdirs();
        corpus.parse(null, "serialized", Params.parse);
        Collection<Interaction> full = corpus.interactions();
        Map<String, List<Interaction>> map = new TreeMap<String, List<Interaction>>();
        //combine interactions per document in the map
        for (Interaction i : full) {
            String[] split = i.id.split("\\.");
            String key = split[0] + "." + split[1] + ".";
            List<Interaction> list = map.get(key);
            if (list == null) {
                list = new ArrayList<Interaction>();
                map.put(key, list);
            }
            list.add(i);
        }
        //read folds
        File foldsDir = new File( Params.splits );
        
        File[] testFiles = foldsDir.listFiles( new SvnExcludeFilter());
        String[] testFileNames= new String[testFiles.length];
        for ( int i = 0; i< testFiles.length; i++)
        	testFileNames[i] = testFiles[i].getAbsolutePath();
        Arrays.sort(testFileNames, new MyStringCompare());
        testFiles = null;
        int numFolds = testFileNames.length;
        ArrayList<String>[] folds = new ArrayList[numFolds];
        int i = 0;
        for ( String testFileName : testFileNames ) {
        	System.err.println( testFileName );
            ArrayList<String> fold = new ArrayList<String>();
            try {
                BufferedReader reader = new BufferedReader(new FileReader(testFileName));
                String s;
                while ((s = reader.readLine()) != null) {
                    fold.add(s.concat("."));
                }
                reader.close();                
                folds[i] = fold;
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
            i++;
        }
        //write kernels
        Kernel<Interaction> kern = null;
        try {
            kern = (Kernel<Interaction>) Class.forName(Params.kernelClass).newInstance();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        PrintWriter labelWriter = null, idWriter = null;
        for (int k = 0; k < numFolds; k++) {
//            if (parallel.size() > 0) {
//                if (!parallel.contains(k)) {
//                    continue;
//              }
//          }
            //build interaction sets for one fold
            List<Interaction> test = new ArrayList<Interaction>(), train = new ArrayList<Interaction>();
            for (String doc : folds[k]) {
                if (map.get(doc) == null) {
                    _log.error("Document " + doc + " not found");
                } else {
                    test.addAll(map.get(doc));
                }
            }
            for (int j = 0; j < numFolds; j++) {
                if (k != j) {
                    for (String doc : folds[j]) {
                        if (map.get(doc) == null) {
                            _log.error("Document " + doc + " not found");
                        } else {
                            train.addAll(map.get(doc));
                        }
                    }
                }
            }
            KernelBuilder.buildKernel(train, train, Params.trainCVKernel + k, kern);
            KernelBuilder.buildKernel(test, train, Params.testCVKernel + k, kern);
            try {
                //in parallel with kernel folds building, write labels and id's file
                labelWriter = new PrintWriter(Params.testLabels + k);
                idWriter = new PrintWriter(Params.ids + k);
                for (Interaction inst : test) {
                    labelWriter.write(inst.status.ordinal() + System.getProperty("line.separator"));
                    idWriter.write(inst.id + System.getProperty("line.separator"));
                    Params.addLog( 5, String.valueOf("Instance: " + inst.id + " label: " + inst.status.ordinal()));
                }
                labelWriter.close();
                idWriter.close();
            } catch (FileNotFoundException ex) {
                Logger.getLogger(Runners.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    
	public void buildCrossDatasetKernel(boolean doTrain, boolean doTest, boolean CC) throws Exception {
        String[] trainSets = Params.trainSet.split(";");
        String[] testSets = Params.testSet.split(";");
        PropertyManager _pm = new PropertyManager();
        CorpusFactory cFactory = new CorpusFactory();
        cFactory.suppressDirection = true;
        // CLtest is used for CL experiments (all test sets together)
        Collection<Interaction> CLtest = null;
        Collection<Interaction> train = new ArrayList<Interaction>();
        List<Collection<Interaction>> tests = new ArrayList(new ArrayList<Interaction>());
        for (String prop : trainSets) {
            _pm.readParam(Params.baseDir + File.separator + prop);
            Params.addLog( 1, "propFile:" + prop);
            Text corpus = cFactory.getCorpusFromPPIXML( 
            		Params.corpDir + File.separator + _pm.getParam("data"), 
            		new DefaultPreprocessor());
            corpus.parse(null, "serialized", Params.baseDir + File.separator + _pm.getParam("parse"));
            train.addAll(corpus.interactions());
            Params.addLog( 1, "TestData: " + train.size());
        }
        for (String prop : testSets) {
        	Collection<Interaction> test = new ArrayList<Interaction>();
            _pm.readParam(Params.baseDir + File.separator + prop);
            Params.addLog( 1, "propFile: " + prop);
            Text corpus = cFactory.getCorpusFromPPIXML(
            		Params.corpDir + File.separator + _pm.getParam("data"), 
            		new DefaultPreprocessor());
            corpus.parse(null, "serialized", 
            		Params.baseDir + File.separator + _pm.getParam("parse"));
            test.addAll(corpus.interactions());
            Params.addLog( 1, "TestData: " + test.size());
            if ( CC )
            	tests.add(test);
            else
            	CLtest = test;
        }
        Kernel<Interaction> kern = null;
        try {
            kern = (Kernel<Interaction>) Class.forName(Params.kernelClass).newInstance();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        if (doTrain) {
            KernelBuilder.buildKernel(train, train, Params.trainKernel, kern);
        }
        if (doTest) {
            try {
            	// if tests.size() == 0 then do CL, otherwise CC 
            	for ( int k=0; k< Math.max(1,tests.size()); k++)
            	{
            		PrintWriter labelWriter = checkWriteFile(Params.testClassValues, k);            	
	            	PrintWriter idWriter = checkWriteFile(Params.ids, k);
	            	Collection<Interaction> test; 
	            	if (CC)
	            		test = tests.get(k);
	            	else
	            		test = CLtest;
	            	KernelBuilder.buildKernel(test, train, Params.testKernel + k, kern);                    
	            	Params.addLog(2, "Test size: " + test.size());
	                for (Interaction inst : test) {
	                    labelWriter.write(inst.status.ordinal() + System.getProperty("line.separator"));
	                    idWriter.write(inst.id + System.getProperty("line.separator"));
	                    Params.addLog( 2, String.valueOf("Instance: " + inst.id + " label: " + inst.status.ordinal()));
	                }
	                labelWriter.close();
	                idWriter.close();
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }

        }
    }

    public void SVMCrossDataset( boolean CC) throws Exception {
        CorpusFactory cFactory = new CorpusFactory();
        cFactory.suppressDirection = true;
        String[] testSets = Params.testSet.split(";");
        List<String> testCorpus = new ArrayList<String>(); 
        List<Collection<Interaction>> tests = new ArrayList(new ArrayList<Interaction>());
        for (String prop : testSets) {
        	Collection<Interaction> test = new ArrayList<Interaction>();
        	PropertyManager _pm = new PropertyManager();
            _pm.readParam(Params.baseDir + File.separator + prop);
            // SOMETHING/<testcorpus>.xml
    		if ( CC ) 
    			{
    			Pattern corpusPattern = Pattern.compile("\\w*\\/(\\w+)\\.xml");
    			Matcher m = corpusPattern.matcher(_pm.getParam("data"));
    			if ( m.find()) 
    				testCorpus.add(m.group(1));
    			else
    				System.err.println("Warning: no pattern found for corpusname in: "
    						+ _pm.getParam("data"));
    			}
            Params.addLog( -1, "propFile: " + prop); 
            Text corpus = cFactory.getCorpusFromPPIXML(Params.corpDir + File.separator + _pm.getParam("data"), new DefaultPreprocessor());
            corpus.parse(null, "serialized", Params.baseDir + File.separator + _pm.getParam("parse"));
            
            test.addAll(corpus.interactions());
            Params.addLog( 1, "TestData: " + test.size());
            if ( CC )
            	tests.add( test );
        }
        // corpus name normalization
/*        for ( int k=0; k<testCorpus.size(); k++ ) {
        	String corpus=testCorpus.get(k);
        	if ( corpus.equals("aimed") )
        		{ testCorpus.set(k, "AIMed") ; }
        	else if ( corpus.equals("bioinfer") )
        		{ testCorpus.set(k, "BioInfer") ; }
        	else if ( corpus.equals("hprd50") )
    			{ testCorpus.set(k, "HPRD50") ; }
        	else if ( corpus.equals("iepa") )
				{ testCorpus.set(k, "IEPA") ; }
        	else if ( corpus.equals("lll") )
				{ testCorpus.set(k, "LLL") ; }
        	else 
        	{ System.err.println("Warning: not normalized corpusname: " + corpus );}
    	}*/
        List<String> realLabels = new ArrayList<String>();
        List<String> ids = new ArrayList<String>();
//        String modelFile = Params.modelFile;
        SVM svm = new SVM(Params.modelFile, Params.SVMtrainexe, Params.SVMtestexe);
        try {
        	// tests.size == 0 -> do CL, otherwise CC
        	if ( tests.size() != testCorpus.size())
        	{ 
        		System.err.println("Size of tests and testCorpus are different: " + tests.size() + ", " + testCorpus.size() );
        	}
        	for ( int k=0; k< Math.max(1,tests.size()); k++)
        	{
	        	BufferedReader labelReader = checkReadFile( Params.testClassValues, k);
	            BufferedReader idReader = checkReadFile(Params.ids, k); 
	            String s = "";
	            while ((s = labelReader.readLine()) != null) {
	                realLabels.add(s);
	            }
	            Params.addLog(3, "realLabels: " + realLabels.size());
	            while ((s = idReader.readLine()) != null) {
	                ids.add(s);
	            }
	            labelReader.close();
	            idReader.close();
	            if ( CC ) 
	            	{
	            		Params.oldTestCorpus = Params.testCorpus;
	            		Params.testCorpus = testCorpus.get(k);
	            		Params.addLog(1, "oldTestCorpus: " + Params.oldTestCorpus 
	            				+ ", testCorpus: " + Params.testCorpus);
	            	}
	            if ( k == 0 )//training is needed only once 
	            	svm.trainOnPrecomp(Params.trainKernel);
				svm.testOnPrecomp(Params.testKernel + k, Params.outFile);
				try {
					if (Params.outputType.equals(OutputType.DB)) {
						if ( Params.errorAnalysis ) 
							writeResultDB(Params.outFile, realLabels, ids );
						else
							writeResultDB(Params.outFile, realLabels);
						
					}
					else
						writeResultsText(Params.outFile, realLabels);
				} catch (FileNotFoundException e) {
					Logger.getLogger(Runners.class.getName()).log(Level.SEVERE,
							null, e);
				}
				realLabels.clear();
				ids.clear();
        	}
        	// delete temporary model and prediction files
        	if ( Params.verbose < 0 ) 
        	{
        		if ( new File(Params.outFile).delete() ) 
        			Params.addLog(-2, "prediction file deleted " + Params.outFile);
        		else
        			Params.addLog(-2, "prediction file not deleted " + Params.outFile);
        		svm.removeSVMTempFile();
        	}
        	
        	
        } catch (IOException e) {
            Logger.getLogger(Runners.class.getName()).log(Level.SEVERE, null, e);
        }
    }

	
    public static String initParam( PropertyManager pm, String value, String defaultValue )
    {
    	if (pm.getParam( value ) != null ) 
    		return pm.getParam( value );
    	else
    		return defaultValue;
    }
    
    public static int initParam( PropertyManager pm, String propertyName, int defaultvalue )
    {
    	if (pm.getParam( propertyName ) != null ) 
    		return Integer.parseInt(pm.getParam( propertyName ));
    	else
    		return defaultvalue;
    }

    public static double initParam( PropertyManager pm, String propertyName, double defaultvalue )
    {
    	if (pm.getParam( propertyName ) != null ) 
    		return Double.parseDouble(pm.getParam( propertyName ));
    	else
    		return defaultvalue;
    }

    public static boolean initParam( PropertyManager pm, String propertyName, boolean defaultvalue )
    {
    	if (pm.getParam( propertyName ) != null ) 
    		return Boolean.parseBoolean(pm.getParam( propertyName ));
    	else
    		return defaultvalue;
    }

    
    public static void readParams( PropertyManager pm ) 
    {
    	String outputType = pm.getParam("outputType");
        if (outputType != null) {
			try {
				Params.outputType = OutputType.valueOf(outputType);
			} catch (IllegalArgumentException e) {
				throw new RuntimeException("Unknown ExperimentType: "
						+ outputType);
			}
		} else
			System.err.println("Warning: assuming default OutputType:"
					+ Params.outputType);
        if (pm.getParam("verbose") != null )
    		Params.verbose = Integer.parseInt(pm.getParam("verbose"));
        if (Params.outputType.equals(OutputType.DB))
        {
        	Params.dbName = initParam( pm, "dbName", Params.dbName);
        	Params.corpus = initParam( pm, "corpus", Params.corpus);
        	Params.kerneltype = initParam( pm, "kerneltype", Params.kerneltype);
            Params.parsertype = "dependency";
            if ( Params.kerneltype.equals("lexical") )
            	Params.parsertype += "+lexical";
            if ( Params.kerneltype.equals("shallow") )
            	Params.parsertype += "+shallow";
            if ( Params.kerneltype.equals("combined") )
            	Params.parsertype += "+lexical+shallow";
        	Params.options = initParam( pm, "options", Params.options);
        	if ( !Params.options.equals("")) 
        		Params.kerneltype += " " + Params.options;
        	//Params.c = initParam(pm, "c", Params.c);
        }
        String expTypeParam = pm.getParam("experimentType");
        if ( expTypeParam != null ) 
        	{
        		try {
        			Params.expType = ExperimentType.valueOf( expTypeParam );
        		} catch (IllegalArgumentException e) {
        			throw new RuntimeException("Unknown ExperimentType: " + expTypeParam );
        		}
        	}
        else
        	System.err.println("Warning: assuming default ExperimentType:" + Params.expType);
        
        Params.kernelClass = initParam( pm, "kernelClass", Params.kernelClass);
        Params.results = Params.baseDir + File.separator + initParam( pm, "results", Params.results);
        Params.SVMtrainexe = Params.baseDir + File.separator + initParam( pm, "SVMtrainexe", Params.SVMtrainexe);
        Params.SVMtestexe = Params.baseDir + File.separator + initParam( pm, "SVMtestexe", Params.SVMtestexe);
        Params.modelFile = Params.expbaseDir + File.separator + initParam( pm, "modelFile", Params.modelFile);
        Params.outFile = Params.expbaseDir + File.separator + initParam( pm, "outFile", Params.outFile);
        Params.splits = Params.corpDir + File.separator + initParam( pm, "splits", Params.splits);
        Params.data = Params.corpDir + File.separator + initParam( pm, "data", Params.data);
        Params.parse = Params.baseDir + File.separator + initParam( pm, "parse", Params.parse);
        Params.PCFGfile = Params.baseDir + File.separator + initParam( pm, "PCFG", Params.PCFGfile );
        Params.trainCVKernel = Params.LFDir + File.separator + initParam( pm, "trainCVKernel", Params.trainCVKernel);
        Params.testCVKernel = Params.LFDir + File.separator + initParam( pm, "testCVKernel", Params.testCVKernel);
        Params.testLabels = Params.LFDir + File.separator + initParam( pm, "testLabels", Params.testLabels);
        Params.ids = Params.LFDir + File.separator + initParam( pm, "ids", Params.ids);
        Params.thresholdStep = initParam( pm, "thresholdStep", Params.thresholdStep);
        Params.rocSource = initParam( pm, "rocSource", Params.rocSource);

        Params.testSet = initParam( pm, "testSet", Params.testSet);
        Params.trainSet = initParam( pm, "trainSet", Params.trainSet);
        Params.trainKernel = Params.LFDir + File.separator + initParam( pm, "trainKernel", Params.trainKernel);
        Params.testKernel = Params.LFDir + File.separator + initParam( pm, "testKernel", Params.testKernel);
        Params.trainClassValues = Params.LFDir + File.separator + initParam( pm, "trainClassValues", Params.trainClassValues);
        Params.testClassValues = Params.LFDir + File.separator + initParam( pm, "testClassValues", Params.testClassValues);
        Params.suppressDirection = initParam( pm, "suppressDirection", Params.suppressDirection);
        
        Params.errorAnalysis = initParam( pm, "errorAnalysis", Params.errorAnalysis);
        
        String baseName = new File(Params.propsFileName).getName().replaceAll("\\.[^.]+$", "");
        
        Params.expIdentifier = Params.corpus + "-" + Params.expType.toString() + "-" + 
        	baseName + "-" + "C-" + Params.c 
        	+ "J-" + Params.j + Params.options.replaceAll("\\s+", "");
        Params.outFile += "." + Params.expIdentifier;
    }
    
    /*
     * args:
     * 	0 - experiment type: CV|crossDataset|CC|CL
     * 	1 - out and err redirection: system
     * 	2 - propFile: resource/props/[CorpusName][comp|lex|pos|syn].props
     *  3 - doKernels : 0|1|2|3 (no, train ,test, both. for CV 0 is not, >0 is do)
     *  4 - runSVM : true|false
     *  5 - calculateROC : true|false
     *  6 - parallel : 1 2 3
     */
    
    private static void printUsage() {
        System.err.println(
        "Usage: Runners [{-n,--name} exp|'parsing'] [{-l,--log} file] [{-v,--verbose}]\n" +
        "               [{-p,--props} file] [{-m,--mode} 0..3]\n" +
        "               [{-b,--build}] [{-t,--train}]\n" +
        "				[{--basedir} dir] [{--lfdir} dir] [{--corpdir} dir] [{--expdir} dir]\n" +
        "               [{-c,--cvalue} float] [{-j,--jvalue} float]\n" +
        "               [{-o,--svm-options} 'svm_options']");
    } 
    
   
    public static void main(String[] args) throws Exception {
    	Locale.setDefault(Locale.ENGLISH);
    	
    	// define options
    	CmdLineParser parser = new CmdLineParser();
        CmdLineParser.Option expNameOpt = parser.addStringOption('n',"name"); //former arg[0]
        CmdLineParser.Option logNameOpt = parser.addStringOption('l', "log"); //former arg[1]
        CmdLineParser.Option propsNameOpt = parser.addStringOption('p', "props"); //former arg[2]
        CmdLineParser.Option runModeOpt = parser.addIntegerOption('m', "mode"); //former arg[3]
        
        CmdLineParser.Option buildOpt = parser.addBooleanOption('b', "build"); //former arg[4]
        CmdLineParser.Option trainOpt = parser.addBooleanOption('t', "train"); //former arg[4]
        
        // this version do not support former arg[5] that is depreciated
        CmdLineParser.Option verboseOpt = parser.addBooleanOption('v', "verbose");
        
        CmdLineParser.Option baseDirOpt = parser.addStringOption("basedir");
        CmdLineParser.Option lfDirOpt = parser.addStringOption("lfdir");
        CmdLineParser.Option corpDirOpt = parser.addStringOption("corpdir");
        CmdLineParser.Option expDirOpt = parser.addStringOption("expdir");
        
        CmdLineParser.Option cValueOpt = parser.addDoubleOption('c', "cvalue");
        CmdLineParser.Option jValueOpt = parser.addDoubleOption('j', "jvalue");
        CmdLineParser.Option svmOptionsOpt = parser.addStringOption('o',"svm-options");
        
        try {
            parser.parse(args);
        }
        catch ( CmdLineParser.OptionException e ) {
            System.err.println(e.getMessage());
            printUsage();
            System.exit(2);
        } 
        
        // check availability and validity of command line options
        
       
        final String expName = (String)parser.getOptionValue(expNameOpt);
        if ( expName == null ) 
        	throw new IllegalArgumentException("Experiment name has to be given!") ;
        final String logName = (String)parser.getOptionValue(logNameOpt,"default");
        final String propsName = (String)parser.getOptionValue(propsNameOpt);
        if ( propsName == null ) 
        	throw new IllegalArgumentException("Props file has to be given!") ;
        final Integer runMode = (Integer)parser.getOptionValue(runModeOpt);
        if ( !expName.equals("parsing") && runMode == null ) 
        	throw new IllegalArgumentException("Running mode has to be given!") ;
        final Boolean build = (Boolean)parser.getOptionValue(buildOpt, Boolean.FALSE);
        
        Params.baseDir = (String)parser.getOptionValue(baseDirOpt, "./");
        File checkDir = new File(Params.baseDir);
        if ( !(checkDir.exists()) )
        	throw new FileNotFoundException("Directory (baseDir) '" + Params.baseDir + "' does not exists!");
        
        Params.corpDir = (String)parser.getOptionValue(corpDirOpt, "./");
        checkDir = new File(Params.corpDir);
        if ( !(checkDir.exists()) )
        	throw new FileNotFoundException("Directory (corpDir) '" + Params.corpDir + "' does not exists!");
        
        Params.LFDir = (String)parser.getOptionValue(lfDirOpt, "./");
        checkDir = new File(Params.LFDir); 
        if ( !(Params.LFDir.equals("./")) && !(checkDir.exists()) )
        	checkDir.mkdirs();
        
        Params.expbaseDir = (String)parser.getOptionValue(expDirOpt, "./");
        checkDir = new File(Params.expbaseDir); 
        if ( !(Params.expbaseDir.equals("./")) && !(checkDir.exists()) )
        	checkDir.mkdirs();
        
        Params.verbose = (Integer)parser.getOptionValue(verboseOpt, 0);
        
        final Boolean train;
        if ( build )
        	train = (Boolean)parser.getOptionValue(trainOpt, Boolean.FALSE);
        else
        	train = (Boolean)parser.getOptionValue(trainOpt, Boolean.TRUE);
        if ( build && train )
        	throw new IllegalArgumentException("buildOnly and train cannot be together specified!") ;
        final Double cValue = (Double)parser.getOptionValue(cValueOpt, 1.0);
        final Double jValue = (Double)parser.getOptionValue(jValueOpt, 0.0);
        final String svmOptions = (String)parser.getOptionValue(svmOptionsOpt);
        
        
    	//prepare error and log files
        PrintStream mylog = null;
        PrintStream err = null;
    	String logDir = Params.baseDir + File.separator + "log";
    	File lg = new File(logDir);
    	if (!(lg.exists()))
    		lg.mkdirs();
        try {
        	// read property file
        	Params.propsFileName = propsName;
        	Runners r = new Runners(Params.propsFileName); // this comes with full path
            // overwrite properties if specified as options 
            if ( svmOptions != null ) 
            	Params.options = svmOptions;
        	if ( cValue != null )
        		Params.c = cValue;
        	if ( jValue != null ) { 
        		Params.j = jValue;
        		Params.options += " -nu " + Params.j;
        	}
            // read properties
            readParams( r.pm );
            String logFileName = logDir + File.separator + logName+"." + Params.expIdentifier+".log";
            String errFileName = logDir + File.separator + logName+"." + Params.expIdentifier+".err";
        	mylog = new PrintStream(new File( logFileName));
            err = new PrintStream(new File( errFileName));
        	
            System.setErr(err);
            if ( Params.outputType.equals(OutputType.DB))
            	{           	
            	Params.logFile = new PrintStream(new FileOutputStream(logFileName));
            	if ( Params.logFile != null )
            	System.err.println("logFile named: " + logFileName + " opened.");
            	}
            else
            	System.setOut(mylog);
        	Params.addLog( -2, "Working directory: " + Params.baseDir);
            Params.addLog( -2, Params.printall());
            Params.addLog( 3, String.valueOf(Params.options));
            Params.addLog( 3, String.valueOf("Task: " + expName ));
            if(expName.equals("parsing")){
                ParseSerializer.main(new String[]{ 
                		Params.data, 
                		Params.parse,
                		Params.PCFGfile});
                return;
            }
            
            //int doKernels = runMode;
            //boolean runSVM = train;
            //boolean calculateROC = Boolean.parseBoolean(args[5]);
            
            // delete temporary files from directory if training is performed
            if ( !(Params.modelFile.equals("")) )
            {
            	Params.expDir = new File( Params.modelFile).getParent().toString(); 
            	File f = new File(Params.expDir);
            	if ( runMode == 1|| runMode == 3 ) {
	            	deleteDirectory(f);
	            }
	            // create root directory for results
	            f.mkdirs();
	            Params.addLog(-2, "expDir: " + Params.expDir);
				
            }
            
            if ( train || !build ) 
            	try {
				Params.createSQLFile();
			} catch (IOException e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}

            if(Params.expType.equals(ExperimentType.CV)){
//                Set<Integer> parallel = new HashSet<Integer>();
//                for(int i = 6; i<args.length; i++){
//                    parallel.add(new Integer(args[i]));
//                }
                if( runMode>0 ){
                    //r.buildCVkernel(Params.suppressDirection, parallel);
                    r.buildCVkernel( Params.suppressDirection );
                }
                if( train || (!build) ){
                    r.SVMCV();
                }
//                if(calculateROC){
//                    r.calculateROC();
//                }
            }else if(Params.expType.equals(ExperimentType.CL)||Params.expType.equals(ExperimentType.CC)){
            	Params.fold = 11;
                switch(runMode){
                    case 1: r.buildCrossDatasetKernel(true, false, Params.expType.equals(ExperimentType.CC)); break;
                    case 2: r.buildCrossDatasetKernel(false, true, Params.expType.equals(ExperimentType.CC)); break;
                    case 3: r.buildCrossDatasetKernel(true, true, Params.expType.equals(ExperimentType.CC)); break;
                }
                if( train || (!build) ){
                    r.SVMCrossDataset(Params.expType.equals(ExperimentType.CC));
                }
//                if(calculateROC){
//                    r.calculateROC();
//                }
            }
            Params.addLog(-2, "finished");
            System.err.println("finished");
            mylog.close();
            err.close();
            Params.closeFiles();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (NumberFormatException e) {
			throw new IllegalArgumentException(e);
	    } catch (IllegalArgumentException e) { 
            System.err.println(e.getMessage());
            e.printStackTrace();
            printUsage();
            System.exit(2);
	    }
    }
}
 
/*    public void calculateROC() {
        BufferedReader reader;
        List<String> list = new ArrayList<String>();
        try {
            reader = new BufferedReader(new FileReader(Params.outFile));
            String s;
            while ((s = reader.readLine()) != null) {
                list.add(s);
            }
            reader.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        List<String> result = new ArrayList<String>();
        for (String s : list) {
            String[] split = s.split("\t");
            //temporary hack to make '1' positve label and '0' - negative
            //split[1] = Double.toString(1 - Double.parseDouble(split[1]));
            result.add(split[1] + "\t" + split[3]);
        }
        Collections.sort(result, new ProbablitiesComparator());
        try {
            PrintWriter writer = new PrintWriter(Params.rocSource);
            for (String s : result) {
                writer.append(s + "\n");
            }
            writer.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        AUCCalculator.main(new String[]{Params.rocSource, "list"});
    }

    
     class ProbablitiesComparator implements Comparator<String> {

        public int compare(String o1, String o2) {
            double p1 = Double.parseDouble(o1.split("\t")[0]);
            double p2 = Double.parseDouble(o2.split("\t")[0]);
            if (p1 == p2) {
                return 0;
            }
            if (p1 < p2) {
                return 1;
            }
            return -1;
        }
    }

}
*/