package controllers;

import parser.Parser;
import parser.StanfordParser;
import preprocessing.DefaultPreprocessor;
import corpus.Text;
import corpus.ppi.CorpusFactory;

public class CorpusParser {
	/**
	 * 
	 * @param params
	 * Format:
	 * 	-xml xml_file_location
	 *  -serialized serialized_dir_location - optional arg
	 *  -output output_file
	 *  -pcfg location of englishPCFG.ser.gz file
	 * @throws Exception 
	 */
	public void run(String...params) throws Exception{
		String sourceXML = "", serialDir = "", out = "", PCFGfile = "";
		for(int i = 0; i< params.length; i++){
			if(params[i].equals("-xml")){
				sourceXML = params[++i];
			}
			if(params[i].equals("-serialized")){
				serialDir = params[++i];
			}
			if(params[i].equals("-output")){
				out = params[++i];
			}
			if(params[i].equals("-pcfg")){
				PCFGfile = params[++i];
			}
		}
		CorpusFactory cFactory = new CorpusFactory();
		Text corpus = cFactory.getCorpusFromPPIXML(sourceXML, new DefaultPreprocessor());
		if(!serialDir.equals("")){
			corpus.parse(null,"serialized",serialDir);
		}else{
			Parser stanfordParser = new StanfordParser();
			stanfordParser.init("-maxLength","70", "-pcfg", PCFGfile );
			corpus.parse(stanfordParser);
		}		
		/*CorpusPrinter printer = new CorpusPrinter();
		String txt = printer.printInARFF(corpus);
		FileWriter writer = null;
		try {
			writer = new FileWriter(out);
			writer.write(txt);
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (writer != null) {
				try {
					writer.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}*/
	}
}
