/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package machinelearning;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import machinelearning.wekawrapper.Dataset;
import util.AdvancedStopwatch;
import util.Params;
import util.Params.ExperimentType;
import util.Params.OutputType;

/**
 * 
 * @author tfayruzo
 */
public class SVM {

	private String modelFileBaseName;
	private String modelFileCurr;
	// executables
	private String SVMtrain;
	private String SVMtest;
	private String trainOptions;
	private String testOptions;
	private static boolean isLinux = System.getProperty("os.name").matches(
			".*(?i)Linux.*");

	public SVM(String modelFile, String SVMtrain, String SVMtest) {

		this.SVMtrain = SVMtrain;
		this.SVMtest = SVMtest;
		this.modelFileBaseName = modelFile;
		if (isLinux) {
			this.SVMtrain = SVMtrain.replaceAll("(?i)\\.exe", "");
			this.SVMtest = SVMtest.replaceAll("(?i)\\.exe", "");
		}
		if (Params.outputType.equals(OutputType.DB)) {
			if (Params.options.equals(""))
				trainOptions = "-q -t 4 -b 1 -c " + Params.c + " ";
			else
				trainOptions = "-q -t 4 -b 1 -c " + Params.c + " " + Params.options + " ";
			testOptions = "-b 1 ";
		}
		else {
			trainOptions = " -t 4 -b 1 ";
			testOptions = " -b 1 ";
		}
	}

	public void train(Dataset trainSet) {
		long uid = System.currentTimeMillis();
		String trainFile = System.getProperty("java.io.tmpdir") + "train" + uid;
		modelFileCurr = modelFileBaseName + uid;
		trainSet.writeToFile(trainFile);
		String command = SVMtrain + " " + trainOptions + " " + trainFile + " " + modelFileCurr;
		System.err.println("Modelfile named: " + modelFileCurr);
		if (Params.outputType.equals(OutputType.DB))
			execWithTimer(command, "train");
		else
			exec(command);
	}

	public void test(Dataset testSet, String outFile) {
		String testFile = System.getProperty("java.io.tmpdir") + "test"
				+ System.currentTimeMillis();
		testSet.writeToFile(testFile);
		String command = SVMtest + " " + testOptions + " " + testFile + " " + modelFileCurr + " " + outFile;
		if (Params.outputType.equals(OutputType.DB))
			execWithTimer(command, "test");
		else
			exec(command);
	}

	public void trainOnPrecomp(String trainKernelFile) {
		long uid = System.currentTimeMillis();
		modelFileCurr = modelFileBaseName + uid;
		String command = SVMtrain + " " + trainOptions + " " + trainKernelFile + " " + modelFileCurr;
		System.err.println("Modelfile named: " + modelFileCurr);
		if (Params.outputType.equals(OutputType.DB))
			execWithTimer(command, "train");
		else
			exec(command);
	}

	public void testOnPrecomp(String testKernelFile, String outFile) {
		String command = SVMtest + " " + testOptions + " " + testKernelFile + " " + modelFileCurr + " "
				+ outFile;
		if (Params.outputType.equals(OutputType.DB))
			execWithTimer(command, "test");
		else
			exec(command);
		
	}

	public void removeSVMTempFile()
	{
		if ( new File(modelFileCurr).delete() )
			Params.addLog(-1, "modelfile deleted " + modelFileCurr);
		else
			Params.addLog(-1, "modelfile not deleted " + modelFileCurr);
	}
	
	private void execWithTimer(String command, String mode) {
			AdvancedStopwatch sw = new AdvancedStopwatch();
			sw.start();
			exec(command);
			sw.stop();
			Params.addLog(1, "done.");
			Params.addLog(5, sw.toString());
			if (Params.outputType.equals(OutputType.DB)) {
				if (mode.equals("train")){
					Params.CCTrainTime = "insert into " + Params.dbName + 
							" (corpus, parsertype, parser, kernel, c, j, fold, kernel_script) values ('" +
							Params.corpus + "', '" + Params.parsertype +"', '" +
							Params.parser + "', 'Kim', " +
							Double.toString(Params.c) + ", " +
							Double.toString(Params.j) + ", " +
							Integer.toString(Params.fold) + ", '" +  
							Params.kerneltype + "');\n" +
//							"update " + Params.dbName + " set kernel_script ='" +
//							Params.kerneltype + "' where " + Params.dbName + "id = " + Params.currID() + ";\n" + 
							"update " + Params.dbName + " set learn_sec = " 
							+ sw.toStringUser() + " where "  + Params.dbName + "id = " + Params.currID() + ";\n" +
							"update " + Params.dbName + " set sv_num = " + 
							fetchSVnum(modelFileCurr) + " where "  + Params.dbName + "id = " + Params.currID() + ";";
					
					
				}
				else
				{
					if ( Params.expType.equals(ExperimentType.CC))
					{
						if ( Params.oldTestCorpus.length() == 0 ) //first test
						{
							Params.CCTrainTime = Params.CCTrainTime.replace("corpus, ", "corpus, test, ");
							Params.CCTrainTime = Params.CCTrainTime.replace( Params.corpus+ "', '", Params.corpus+ "', '" + Params.testCorpus+ "', '");
						}
						else
							Params.CCTrainTime = Params.CCTrainTime.replace( Params.oldTestCorpus, Params.testCorpus);
					}
					Params.sqlFile.println(Params.CCTrainTime);
					Params.sqlFile.println("update " + Params.dbName + " set classify_sec = " 
							+ sw.toStringUser() + " where " + Params.dbName + "id = " + Params.currID() + ";");
				}
			}
	}
	
	private static Integer fetchSVnum(String modelFileCurr2) {
		try {
			InputStreamReader isr = new InputStreamReader(new FileInputStream(modelFileCurr2));
			BufferedReader br = new BufferedReader(isr);
			// total_sv #num
			Pattern svpattern = Pattern.compile("^total_sv\\s+(\\d+)");
			String line = null;
			while ((line = br.readLine()) != null) {
				Matcher m = svpattern.matcher(line);
				if (m.find()) { 
					isr.close(); 
					return Integer.parseInt(m.group(1)); 
				}
			}
			isr.close();		
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		return null;
	}
		
	private void exec(String command) {
		try {
			Params.addLog(1, command);
			Process p = Runtime.getRuntime().exec(command);
			StreamGobbler errorGobbler = new StreamGobbler(p.getErrorStream(),
					"SVM ERROR");
			errorGobbler.start();
			StreamGobbler outputGobbler = new StreamGobbler(p.getInputStream(),"SVM OUTPUT");
			outputGobbler.start();
			int exitcode = p.waitFor();
			if ( exitcode != 0 )
				throw new RuntimeException("Command '" + command +"' returned with exitcode" + exitcode);
		} catch (IOException e) {
			throw new RuntimeException("Command '" + command +"' failed", e);
		} catch (InterruptedException e) {
			throw new RuntimeException("Command '" + command +"' interrupted", e);
		}
	}

	class StreamGobbler extends Thread {

		InputStream is;
		String type;

		StreamGobbler(InputStream is, String type) {
			this.is = is;
			this.type = type;
		}

		public void run() {
			try {
				if (Params.outputType.equals(OutputType.Fayruzov)) {
					InputStreamReader isr = new InputStreamReader(is);
					BufferedReader br = new BufferedReader(isr);
					String line = null;
					while ((line = br.readLine()) != null) {
						Params.addLog(-2, type + ">" + line);
					}
					isr.close();		
				}
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
		}
	}
}
