package machinelearning.evaluation;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;


import java.io.BufferedReader;
import java.io.FileReader;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import deeper.Interaction;
import util.Params;
import weka.classifiers.Classifier;
import weka.core.Instance;
import machinelearning.SVM;
import machinelearning.wekawrapper.Dataset;

/**
 * Class contains two evaluation strategies - train-test and folds cv.
 * 
 * @author tfayruzo
 *
 */
public class EvaluationProcedure {

    //file with classification results
    public String classifierOutput;
    private static List<String> SVMoutput;

    public EvaluationProcedure(String classifierOutput) {
        this.classifierOutput = classifierOutput;
    }

    /* 
     * Evaluation for the precomputed kernel
     */
    /*public void trainTestEvaluation(Dataset train, Dataset test, SVM svm, Set<Integer> posLabels) {
        svm.train(train);
        List<String> testLabels = svm.test(test);
        writeClassifierOutput(testLabels);
    }*/

    public PerformanceMeasure getPrediction4threshold(List<String> realLabels, Set<Integer> posLabels, double threshold) {
        readClassifierOutput(); 
        return getPrediction4threshold_inner(realLabels, posLabels, threshold);
    }

    void readClassifierOutput()
    {
    	SVMoutput = new ArrayList<String>();
    	try {
	        BufferedReader reader = new BufferedReader(new FileReader(classifierOutput));
	        String s = "";
	        while ((s = reader.readLine()) != null) {
	            SVMoutput.add(s);
	        }
	        reader.close();
    	} catch (IOException ex) {
        Logger.getLogger(EvaluationProcedure.class.getName()).log(Level.SEVERE, null, ex);
    	}
    }
    
    public PerformanceMeasure basicPerformanceMeasures(List<String> realLabels, Set<Integer> posLabels, List<String> ids )
    {
    	readClassifierOutput();
    	boolean EA = true;
    	PerformanceMeasure result = new PerformanceMeasure();
    	Iterator<String> it = realLabels.iterator();
    	Iterator<String> oit = SVMoutput.iterator();
    	Iterator<String> id = null;
    	if (( ids != null )&&(!ids.isEmpty())) {
    		id = ids.iterator();  
    	}
    	else EA = false;
    	int predLabel;
    	boolean firstneg=false;		    	
    	float probPos,probNeg;
    	while ( oit.hasNext() )
    	{
    		String str = oit.next();
    		if (str.startsWith("labels")) 
    			{
    			String[] mylabels = str.split("\\s+");
    				if ( Integer.parseInt(mylabels[1]) == 0)//first prediction value belongs to negative class
    					firstneg = true;
    				if ( Integer.parseInt(mylabels[1]) == 1)//first prediction value belongs to positive class
    					firstneg = false;
    			continue;
    			}
    		else
    		{
    			String[] values = str.split("\\s+");
    			int realLabel = Integer.parseInt(it.next());
    			
    			if (values.length>=3)
    			{
    				predLabel = Integer.parseInt(values[0]);
    				if ( firstneg ) {
    					probNeg = Float.parseFloat(values[1]);
    					probPos = Float.parseFloat(values[2]);
    				}
    				else {
    					probPos = Float.parseFloat(values[1]);
    					probNeg = Float.parseFloat(values[2]);
    				}
    			}
    			else
    				throw new RuntimeException("Prediction row too short");
    			if ( EA ) 
    				{
    				Float prediction = probPos;
    				Params.sqlFile.println("insert into " + 
    	    				Params.dbName+"output (expId, pair, output, prediction) values " + 
    	    				"(" + Params.currID() + ", " + 
    	    				"'" + id.next() + "', " + 
    	    				"'" + predLabel + "', " + 
    	    				String.format("%.4f", new Float(prediction)) +");");
    				}
    			// set label prediction
    			if ( realLabel == 1)
    				result.confPos.add(probPos);
    			else
    				result.confNeg.add(probPos);
    			if ( predLabel == realLabel ) {
    				if ( realLabel == 1)
    					result.truePositives++;
    				else
    					result.trueNegatives++;
    			}
    			else {
    				if ( realLabel == 1)
    					result.falseNegatives++;
					else
						result.falsePositives++;
				}
    		}
    	}
    	return result;
    }
    
    public PerformanceMeasure getPrediction4threshold_inner(List<String> realLabels, Set<Integer> posLabels, double threshold) {
        PerformanceMeasure result = new PerformanceMeasure();
        Set<Integer> posIndices = new HashSet<Integer>();
        int negLabel = 0;
        int[] labels = new int[]{};
        Iterator<String> it = realLabels.iterator();
    	Iterator<String> oit = SVMoutput.iterator();
        while ( oit.hasNext()) {
        	String str = oit.next();
            if (str.startsWith("labels")) {
                String[] labelsStr = str.split("\\s+");
                labels = new int[labelsStr.length];
                for (int i = 1; i < labelsStr.length; i++) {
                    labels[i] = Integer.parseInt(labelsStr[i]);
                    if (posLabels.contains(labels[i])) {
                        posIndices.add(i);
                    } else {
                        negLabel = labels[i];
                    }
                }
            } else {
                String[] values = str.split("\\s+");
                int realLabel = Integer.parseInt(it.next());
                int predLabel = 0;
                double classProb = 0;
                double sum = 0;
                for (Integer i : posIndices) {
                    double prob = Double.parseDouble(values[i]);
                    if (prob > classProb) {
                        classProb = prob;
                        predLabel = labels[i];
                    }
                    if (labels[i] != negLabel) {
                        sum += prob;
                    }
                }
                if (sum < threshold) {
                    predLabel = negLabel;
                }
                if (realLabel == negLabel) {
                    if (predLabel == negLabel) {
                        result.trueNegatives++;
                    } else {
                        result.falsePositives++;
                    }
                } else {
                    if (predLabel == negLabel) {
                        result.falseNegatives++;
                    } else if (realLabel == predLabel) {
                        result.truePositives++;
                    } else if (realLabel != negLabel) {
                        result.falsePositives++;
                    }
                }
            }
        }
        return result;
    }

    private void writeClassifierOutput(List<String> output) {
        try {
            PrintWriter writer = new PrintWriter(classifierOutput);
            for (String str : output) {
                writer.write(str + System.getProperty("line.separator"));
            }
        } catch (FileNotFoundException ex) {
            Logger.getLogger(EvaluationProcedure.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void pairedTtest(){

    }

    /**
     * Peculiarity of this method is that it could
     * evaluate several positive classes against one negative
     * confusions between different positive classes are considered as false positives.
     * @param train
     * @param test
     * @param classifier
     * @param negativeClassValue - index of negative class
     * @param positiveClassValues - indices of positive classes
     * @return
     */
    public List<double[]> trainTestEvaluation(Dataset train, Dataset test, Classifier classifier, Set<Integer> positiveClassValues) {
        List<double[]> result = new ArrayList<double[]>();
        try {
            classifier.buildClassifier(train.getDataset());
            //classifying test instances
            for (Enumeration<Instance> e = test.getDataset().enumerateInstances(); e.hasMoreElements();) {
                Instance inst = e.nextElement();
                double[] dist = classifier.distributionForInstance(inst);
                result.add(dist);
            }
        } catch (Exception ex) {
            Logger.getLogger(EvaluationProcedure.class.getName()).log(Level.SEVERE, null, ex);
        }
        return result;
    }

    /*
    public PerformanceMeasure getPrediction4threshold(List<double[]> classifierOutput, List<Integer> realLabels, Set<Integer> posLabels, double threshold) {
        PerformanceMeasure result = new PerformanceMeasure();
        for (double[] dist : classifierOutput) {
            double sum = 0;
            int maxInd = 0;
            double maxElem = 0;
            //looking for the most probable positive class (although negative class could have the highest value)
            //and computing the summary probability of positive classification
            for (int i : posLabels) {
                sum += dist[i];
                if (maxElem < dist[i]) {
                    maxElem = dist[i];
                    maxInd = i;
                }
            }
            int pred = maxInd;
            if (sum < threshold) {
                pred = negativeClassValue;
            }
            prediction.add(Double.toString(sum) + "\t" + Integer.toString(pred));
            double real = inst.classValue();
            if (real == negativeClassValue) {
                if (real == pred) {
                    result.trueNegatives++;
                } else {
                    result.falsePositives++;
                }
            } else {
                if (real == pred) {
                    result.truePositives++;
                } else if (pred == negativeClassValue) {
                    result.falseNegatives++;
                } else {
                    result.falsePositives++;
                }
            }
        }
        return result;
    }*/

    /**
     * Simple stratified cross-validation evaluation
     * @param data
     * @param classifier
     * @param numFolds
     * @param negativeClassValue
     * @param positiveClassValues
     * @return
     */
    /*public PerformanceMeasure crossValidation(Dataset data, Classifier classifier, int numFolds, int negativeClassValue, int... positiveClassValues) {
        PerformanceMeasure[] foldPerformance = new PerformanceMeasure[numFolds];
        Dataset cvData = new Dataset(data);
        cvData.stratify(numFolds);
        for (int i = 0; i < numFolds; i++) {
            Dataset test = cvData.getTestCVFold(numFolds, i);
            Dataset train = cvData.getTrainCVFold(numFolds, i);
            foldPerformance[i] = trainTestEvaluation(train, test, classifier, negativeClassValue, positiveClassValues);
        }
        int tp = 0, tn = 0, fp = 0, fn = 0;
        for (int i = 0; i < numFolds; i++) {
            tp += foldPerformance[i].truePositives;
            tn += foldPerformance[i].trueNegatives;
            fp += foldPerformance[i].falsePositives;
            fn += foldPerformance[i].falseNegatives;
        }
        return new PerformanceMeasure(tp, tn, fp, fn);
    }*/

    /**
     * Cross-validation on predefined folds
     * @param classifier
     * @param folds
     * @param negativeClassValue
     * @param positiveClassValues
     * @return
     */
    public void crossValidation(SVM svm, String[] trainFolds, String[] testFolds, Set<Integer> posLabels) {
        PerformanceMeasure[] foldPerformance = new PerformanceMeasure[trainFolds.length];
        for (int i = 0; i < trainFolds.length; i++) {
            String baseForm = classifierOutput+".fold"+i;
            svm.trainOnPrecomp(trainFolds[i]);
            svm.testOnPrecomp(testFolds[i], baseForm);
            //trainTestEvaluation(trainFolds[i], testFolds[i], svm, posLabels);
        }
    }

    /**
     *
     * @param trainFile
     * @param testFile
     * @param classifier
     * @param negativeClassValue
     * @param positiveClassValues
     * @return
     */
    /*public String SVMEvaluationForLLL(Dataset train, Dataset test, SVM classifier, int negativeClassValue, int... positiveClassValues) {
        Map<String, EvaluationEntry> result = new HashMap<String, EvaluationEntry>();
        List<Interaction> lInter = test.getInteractions();
        String outFile = "resource/svm/result.svm";
        //running the classifier
        ((SVM) classifier).buildClassifier(train.getPrecomputedKernel());
        ((SVM) classifier).testOnPrecomputedKernel(test.getPrecomputedKernel(), outFile);
        ClassificationResult cr = new ClassificationResult();
        //Map<Interaction, Integer> classification = cr.getClassificationFile4SVM(outFile, test, negativeClassValue, positiveClassValues);
        prediction.addAll(cr.getClassification4SVM(outFile, test.getClassValues(), negativeClassValue, positiveClassValues));

        //for(Interaction i : classification.keySet()){
        for (int k = 0, size = lInter.size(); k < size; k++) {
            Interaction i = lInter.get(k);
            String id = i.origin.id();
            int predClass = Integer.parseInt(prediction.get(k).split("\t")[1]);

            EvaluationEntry ee = result.get(id);
            if (ee == null) {
                ee = new EvaluationEntry();
                ee.sentenceId = id;
            }
            if (predClass == negativeClassValue) {
                //do nothing
            } else if (predClass == InteractionType.REALINVERSE.ordinal()) {
                ee.agents.add(i.p2.getName());
                ee.targets.add(i.p1.getName());
                ee.interactions.add(new String[]{i.p2.getName(), i.p1.getName()});
            } else {
                ee.agents.add(i.p1.getName());
                ee.targets.add(i.p2.getName());
                ee.interactions.add(new String[]{i.p1.getName(), i.p2.getName()});
            }
            result.put(id, ee);
        }
        StringBuffer sb = new StringBuffer("% Participant name: Timur Fayruzov\n" +
                "% Participant institution: UGent\n" +
                "% Participant email address: timur.fayruzov@ugent.be\n" +
                "% Format checked: YES\n" +
                "% Basic data: YES\n" +
                "% Coreference distinction: WITH COREFERENCE and WITHOUT COREFERENCE\n");
        for (String key : result.keySet()) {
            sb.append(result.get(key).toString());
        }
        return sb.toString();
    }

    public void writePrediction(PrintWriter writer) {
        for (String p : prediction) {
            writer.println(p);
        }
    }

    public List<String> getPrediction() {
        return prediction;
    }

    public void clearPrediction() {
        prediction.clear();
    }*/
    
//    public static void main(String[] args) {
//    	EvaluationProcedure eval = new EvaluationProcedure("SVMpredictionfilename");
//    	PerformanceMeasure performance = eval.basicPerformanceMeasures(realLabels, posLabels, ids);
//    	
//    	PerformanceMeasure pm = new PerformanceMeasure(1,1,1,1);
//    	System.out.println(pm.getAUC());
//    }    
        
}

