/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package machinelearning.evaluation;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import util.Params;

/**
 *
 * @author tfayruzo,dtikk
 */
public class PerformanceMeasure {

    public int truePositives;

    public int falsePositives;

    public int trueNegatives;

    public int falseNegatives;

    public ArrayList<Float> confPos = new ArrayList<Float>();
    public ArrayList<Float> confNeg = new ArrayList<Float>();
    
    
    private double AUC;
       
    public PerformanceMeasure(int tp, int tn, int fp, int fn) {
        this.truePositives = tp;
        this.trueNegatives = tn;
        this.falsePositives = fp;
        this.falseNegatives = fn;
    }

    public PerformanceMeasure(){
        this(0,0,0,0);
    }


    public int getNumofPos() {
    	return truePositives+falseNegatives;
    }
   
    public int getNumofNeg() {
    	return trueNegatives + falsePositives;
    }

    public double getTPRate() {
        return (double)((this.truePositives + this.falseNegatives))==0?0.0:((double)this.truePositives)/(double)((this.truePositives + this.falseNegatives));
    }
    
    public double getTNRate() {
        return ((double)(this.trueNegatives + this.falsePositives))==0?0.0:((double)this.trueNegatives)/((double)(this.trueNegatives + this.falsePositives));
    }

    public double getFNRate() {
        return ((double)(this.truePositives + this.falseNegatives))==0?0.0:((double)this.falseNegatives)/((double)(this.truePositives + this.falseNegatives));
    }

    public double getFPRate() {
        return ((double)(this.falsePositives + this.trueNegatives))==0.0?0.0:((double)this.falsePositives)/((double)(this.falsePositives + this.trueNegatives));
    }

    public double getRecall() {
        return ((double)(this.truePositives + this.falseNegatives))==0?0.0:((double)this.truePositives)/((double)(this.truePositives + this.falseNegatives));
    }

    public double getPrecision() {
        return (double)(this.truePositives + this.falsePositives)==0?0.0:((double)this.truePositives)/((double)(this.truePositives + this.falsePositives));
    }

    public double getFMeasure() {
        return getFMeasure(1);
    }

    public double getFMeasure(int beta) {
        double f = ((beta * beta + 1) * this.getPrecision() * this.getRecall())
                / (beta * beta * this.getPrecision() + this.getRecall());
        if (Double.isNaN(f))
            return 0;
        else
            return f;
    }

    @Override
    public String toString() {
        return "[TP=" + this.truePositives + ", FP=" + this.falsePositives + ", TN=" + this.trueNegatives + ", FN="
                + this.falseNegatives + "]";
    }

    public int getTotal() {
        return falseNegatives + falsePositives + trueNegatives + truePositives;
    }
    
    
    private double computeAUC() {
    	Params.addLog(3 , "Compute AUC!" + 
    			Arrays.toString(confPos.toArray()) + " " +
    			Arrays.toString(confNeg.toArray()));
      checkConsistency();
      if (getNumofNeg() == 0) {
    	  AUC = getTPRate();
    	  System.err.println("TP rate is returned for AUC");
      	  return AUC;
      }
      if (getNumofPos() == 0 ){
    	  AUC = getTNRate();
    	  System.err.println("TN rate is returned for AUC");
    	  return AUC;
	  }
      double numerator = 0.0;
      // numerator
      if ( confPos.size() == 0 )
    	  System.err.println("No positive prediction (AUC)!");
      if ( confNeg.size() == 0 )
    	  System.err.println("No negative prediction (AUC)!");
      for ( int i = 0; i<confPos.size(); i++) {
    	for ( int j = 0; j<confNeg.size(); j++) {
          numerator += computeSingleAUC((double)confPos.get(i)-confNeg.get(j));
        }
      }
      // divide by denominator
      AUC = numerator / ((double)(getNumofPos()*getNumofNeg()));
      return AUC;
    }
    
    private double computeSingleAUC(double r) {
      if (r > 0.0) {
        return 1;
      }
      else if (r < 0.0) {
        return 0;
      }
      else {
        return 0.5;
      }
    }
    
    public double getAUC()
    {
    	return (Math.abs(AUC)<0.001)?computeAUC():AUC;
    }

    public void checkConsistency() throws IllegalStateException{
    	if ( getNumofPos()!=confPos.size() || getNumofNeg()!=confNeg.size() )
    		throw new IllegalStateException("Invalid number of positive or negative predictions");
    }
    

}
