package machinelearning.features;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import treedata.types.PennPOS;
import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import deeper.Interaction;
import edu.uci.ics.jung.graph.Vertex;
import graph.Keys;
import java.util.Iterator;

/**
 * Feature builder that takes into account POS in DEPENDENCY path.
 * @author tfayruzo
 *
 */
public class ParseFeatureBuilder implements FeatureBuilder {
	
	private List<Attribute> structure;
	private Instances data;
		
	public ParseFeatureBuilder(boolean verbs){
		structure = new ArrayList<Attribute>();
		for(PennPOS pos : verbs?PennPOS.verbs():PennPOS.values())
			structure.add(new Attribute(pos.name()));
		FastVector  v = new FastVector(); 
		for(Attribute a : structure)
			v.addElement(a);
		data = new Instances("",v,1);
	}

	/**
	 * Returns a binary feature vector, that reflects which POS
	 * are occur in shortest dependency path  
	 * @param i
	 * @return
	 */
	public Instance getFeatureVector(Interaction i) {
		Instance inst = new Instance(structure.size());
		Map<String, Integer> posMap = new HashMap<String, Integer>();
		for(Iterator it = i.depPath.getVertices().iterator(); it.hasNext();){
            Vertex v = (Vertex) it.next();
			String key = (String)v.getUserDatum(Keys.LABEL);
			Integer count = posMap.get(key);
			if(count == null)
				count = new Integer(0);
			posMap.put(key, count+1);
		}
		for(Enumeration<Attribute> e = data.enumerateAttributes(); e.hasMoreElements();){
			Attribute a = e.nextElement();
			Integer count = posMap.get(a.name());
			if(count != null)
				inst.setValue(a, new Double(count));
			else
				inst.setValue(a, new Double(0));
		}
		return inst;
	}

	public Instances getInstanceStructure() {		
		return data;
	}

}
