package preprocessing;

import corpus.Sentence;
import java.text.CharacterIterator;
import java.text.StringCharacterIterator;
import java.util.Collection;
import java.util.Collections;
import java.util.ListIterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import deeper.Protein;

/**
 * Implementation of preprocessor, intended to simplify
 * parsing task.
 * @author tfayruzo
 *
 */
public class DefaultPreprocessor implements Preprocessor {
	private static Log _log = LogFactory.getLog(DefaultPreprocessor.class);

	public void preprocess(Sentence s) {
		String sentence = s.text;
		Collections.sort(s.getProteins());
		int offset = 0;
		//substituting protein names with unique ID's
		int c = 0;
		for(ListIterator<Protein> i = s.getProteins().listIterator(); i.hasNext();){
			Protein prot = i.next();
            if(prot.equalsTo != null){ //do not substitute nested proteins
                continue;
            }
			String id = "ccoterm"+c;
			int length = prot.getEndOffset(prot.getEndOffset().length-1) - prot.getStartOffset(0) + 1;
			prot.setStartOffset_mod(prot.getStartOffset(0) + offset);
            prot.setEndOffset_mod(prot.getStartOffset_mod() + id.length()-1);
			prot.setName_mod(id);
			sentence = sentence.substring(0,prot.getStartOffset_mod()) + id + sentence.substring(prot.getEndOffset(prot.getEndOffset().length-1)+offset+1);
			offset += id.length() - length;
 			c++;
		}
		sentence = htmlReplacements(sentence, s.getProteins());
		sentence = referenceReplacement(sentence, s.getProteins());
        s.modText = sentence;
	}
	
	private void readjustProteinPositions(int[] oldInterval, int[] newInterval, Collection<Protein> proteins){
		int offset = newInterval[1]-newInterval[0] - oldInterval[1] + oldInterval[0];
		for(Protein p : proteins){
			if(p.getStartOffset_mod()>oldInterval[1]){
				p.setStartOffset_mod(p.getStartOffset_mod()+ offset);
                p.setEndOffset_mod(p.getEndOffset_mod()+ offset);
			}
		}
	}
	
	private String htmlReplacements(String sentence, Collection<Protein> interactors){
		Pattern p = Pattern.compile("(&quot)|(&amp)|(&gt)|(&lt)");
		Matcher m = p.matcher(sentence);
		while(m.find()){
			String group = m.group();
			if(group.equals("&quot"))
				sentence = sentence.replace(group, "'");
			else if(group.equals("&amp"))
				sentence = sentence.replace(group, "&");
			else if(group.equals("&gt"))
				sentence = sentence.replace(group, ">");
			else if(group.equals("&lt"))
				sentence = sentence.replace(group, "<");
			readjustProteinPositions(new int[]{m.start(), m.end()}, new int[]{m.start(),m.start()+1},interactors);
			m = p.matcher(sentence);
		}
		return sentence;
	}
	
	private String findBoundary(int[] interval, String sentence){
		String begin = sentence.substring(0,interval[0]);
		String end = sentence.substring(interval[1]);
		int openBr1 = begin.lastIndexOf("(");
		int openBr2 = begin.lastIndexOf("[");
		char openBr, closeBr;
		if(openBr1<0 && openBr2<0){
			return sentence;
		}else if(openBr1>openBr2){
			openBr = '(';
			closeBr = ')';
		}else{
			openBr = '[';
			closeBr = ']';
		}
		CharacterIterator it = new StringCharacterIterator(begin);
		int counter = 0;
		int startIndex = 0;
		for(char c = it.last(); c != CharacterIterator.DONE; c = it.previous()){
			if(c == closeBr)
				counter--;
			if(c == openBr)
				counter++;			
			if(counter==1){
				startIndex = it.getIndex();
				break;
			}
		}
		it = new StringCharacterIterator(end);
		counter = 1;
		int endIndex = 0;
		for(char c = it.first(); c != CharacterIterator.DONE; c = it.next()){
			if(c == openBr)
				counter++;
			if(c == closeBr)
				counter--;
			if(counter==0){
				endIndex = it.getIndex();
				break;
			}
		}
		endIndex = begin.length() + interval[1] - interval[0] + 1 + endIndex;
		return sentence.substring(startIndex, endIndex);
	}
	
	public String referenceReplacement(String sentence, Collection<Protein> interactors){
		Pattern p = Pattern.compile("\\[[\\d,\\-\\s]+\\]");
		Matcher m = p.matcher(sentence);
		while(m.find()){
			String group = m.group();
			sentence = sentence.replace(group, "");
			readjustProteinPositions(new int[]{m.start(), m.end()}, new int[]{m.start(),m.start()},interactors);
		}
		p = Pattern.compile("(([A-Z]\\.\\s*)+[A-Z][a-z'-]+)|([A-Z][a-z'-]+,\\s+([A-Z]\\.\\s*))");
		m = p.matcher(sentence);
		while(m.find()){
			String surroundingGroup = findBoundary(new int[]{m.start(),m.end()}, sentence);
			int start = sentence.indexOf(surroundingGroup);
			int[] oldBound = new int[]{start, start + surroundingGroup.length()};
			int[] newBound = new int[]{start, start};			
			sentence = sentence.replace(surroundingGroup, "");
			readjustProteinPositions(oldBound, newBound, interactors);
			m = p.matcher(sentence);
		}		
		return sentence;		
	}
}
