package test;

import machinelearning.kernel.KimKernel;
import machinelearning.kernel.SyntacticKernel;
import deeper.Interaction;
import edu.uci.ics.jung.graph.DirectedEdge;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.impl.DirectedSparseEdge;
import edu.uci.ics.jung.utils.UserData;
import graph.Keys;
import jung.entities.DependencyVertex;
import jung.entities.Path;
import junit.framework.TestCase;

public class DependencyKernelTest extends TestCase {

	public void testDependencyKernel(){
		//building tree:
		//			control
		//			/	\
		//		subj	comp_by
		//		/			\
		//	expression		sigE
		//		/
		//	mod_att
		//		/
		//	ywhE
		Vertex v1 = new DependencyVertex();
		v1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		v1.addUserDatum(Keys.LABEL, "control", UserData.SHARED);
		Vertex v2 = new DependencyVertex();
		v2.addUserDatum(Keys.INDEX, 1, UserData.SHARED);
		v2.addUserDatum(Keys.LABEL, "expression", UserData.SHARED);
		Vertex v3 = new DependencyVertex();
		v3.addUserDatum(Keys.INDEX, 2, UserData.SHARED);
		v3.addUserDatum(Keys.LABEL, "ywhE", UserData.SHARED);
		Vertex v4 = new DependencyVertex();
		v4.addUserDatum(Keys.INDEX, 3, UserData.SHARED);
		v4.addUserDatum(Keys.LABEL, "sigF", UserData.SHARED);
		Path p1 = new Path(v1);
		p1.addVertex(v2);
		p1.addVertex(v3);
		p1.addVertex(v4);
		DirectedEdge e1 = new DirectedSparseEdge(v1,v2);
		e1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e1.addUserDatum(Keys.LABEL, "subj", UserData.SHARED);
		p1.addEdge(e1);		
		DirectedEdge e2 = new DirectedSparseEdge(v2,v3);
		e2.addUserDatum(Keys.INDEX, 1, UserData.SHARED);
		e2.addUserDatum(Keys.LABEL, "mod_att", UserData.SHARED);
		p1.addEdge(e2);
		DirectedEdge e3 = new DirectedSparseEdge(v1,v4);
		e3.addUserDatum(Keys.INDEX, 2, UserData.SHARED);
		e3.addUserDatum(Keys.LABEL, "comp_by", UserData.SHARED);
		p1.addEdge(e3);
		
		
		//building tree:
		//			abra
		//			/	\
		//		subj	comp_by
		//		/			\
		//	kadabra			shvabra
		//		/
		//	foo
		//	/
		// zanzibabra
		v1 = new DependencyVertex();
		v1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		v1.addUserDatum(Keys.LABEL, "abra", UserData.SHARED);
		v2 = new DependencyVertex();
		v2.addUserDatum(Keys.INDEX, 1, UserData.SHARED);
		v2.addUserDatum(Keys.LABEL, "kadabra", UserData.SHARED);
		v3 = new DependencyVertex();
		v3.addUserDatum(Keys.INDEX, 2, UserData.SHARED);
		v3.addUserDatum(Keys.LABEL, "shvabra", UserData.SHARED);
		v4 = new DependencyVertex();
		v4.addUserDatum(Keys.INDEX, 3, UserData.SHARED);
		v4.addUserDatum(Keys.LABEL, "zanzibabra", UserData.SHARED);
		Path p2 = new Path(v1);
		p2.addVertex(v2);
		p2.addVertex(v3);
		p2.addVertex(v4);
		e1 = new DirectedSparseEdge(v1,v2);
		e1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e1.addUserDatum(Keys.LABEL, "subj", UserData.SHARED);
		p2.addEdge(e1);
		e2 = new DirectedSparseEdge(v1,v3);
		e2.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e2.addUserDatum(Keys.LABEL, "comp_by", UserData.SHARED);
		p2.addEdge(e2);
		e3 = new DirectedSparseEdge(v2,v4);
		e3.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e3.addUserDatum(Keys.LABEL, "foo", UserData.SHARED);
		p2.addEdge(e3);
		//building tree:
		//		abra
		//		/
		//	subj
		//	/
		// kadabra
		v1 = new DependencyVertex();
		v1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		v1.addUserDatum(Keys.LABEL, "abra", UserData.SHARED);
		v2 = new DependencyVertex();
		v2.addUserDatum(Keys.INDEX, 1, UserData.SHARED);
		v2.addUserDatum(Keys.LABEL, "kadabra", UserData.SHARED);
		Path p3 = new Path(v1);
		p3.addVertex(v2);
		e1 = new DirectedSparseEdge(v1,v2);
		e1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e1.addUserDatum(Keys.LABEL, "subj", UserData.SHARED);
		p3.addEdge(e1);
		
		//	building tree:
		//			abra
		//			/	\
		//		foo		comp_by
		//		/			\
		//	kadabra			shvabra
		//		/
		//	subj
		//	/
		// zanzibabra
		v1 = new DependencyVertex();
		v1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		v1.addUserDatum(Keys.LABEL, "abra", UserData.SHARED);
		v2 = new DependencyVertex();
		v2.addUserDatum(Keys.INDEX, 1, UserData.SHARED);
		v2.addUserDatum(Keys.LABEL, "kadabra", UserData.SHARED);
		v3 = new DependencyVertex();
		v3.addUserDatum(Keys.INDEX, 2, UserData.SHARED);
		v3.addUserDatum(Keys.LABEL, "shvabra", UserData.SHARED);
		v4 = new DependencyVertex();
		v4.addUserDatum(Keys.INDEX, 3, UserData.SHARED);
		v4.addUserDatum(Keys.LABEL, "zanzibabra", UserData.SHARED);
		Path p4 = new Path(v1);
		p4.addVertex(v2);
		p4.addVertex(v3);
		p4.addVertex(v4);
		e1 = new DirectedSparseEdge(v1,v2);
		e1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e1.addUserDatum(Keys.LABEL, "foo", UserData.SHARED);
		p4.addEdge(e1);
		e2 = new DirectedSparseEdge(v1,v3);
		e2.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e2.addUserDatum(Keys.LABEL, "comp_by", UserData.SHARED);
		p4.addEdge(e2);
		e3 = new DirectedSparseEdge(v2,v4);
		e3.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e3.addUserDatum(Keys.LABEL, "subj", UserData.SHARED);
		p4.addEdge(e3);
		
		/*Interaction i1 = new Interaction();
		Interaction i2 = new Interaction();
		Interaction i3 = new Interaction();
		Interaction i4 = new Interaction();
         */
		/*i1.depPath = p1;
		i2.depPath = p2;
		i3.depPath = p3;
		i4.depPath = p4;		
		KimKernel k = new KimKernel();
		double sim1 = k.compute(i1, i1);
		double sim2 = k.compute(i1, i2);
		double sim3 = k.compute(i1, i3);
		double sim4 = k.compute(i1, i4);
		double sim5 = k.compute(i2, i3);
		assertEquals("Equality with itself isn't passed",6.0, sim1);
		assertEquals("Equality with 'fork' isn't passed",3.0, sim2);
		assertEquals("Equality with 'chain' isn't passed",1.0, sim3);
		assertEquals("Equality with interrupted dependencies isn't passed",2.0, sim4);
		*/
	}
}
