package test;

import weka.core.Instance;
import deeper.Interaction;
import deeper.InteractionType;
import edu.uci.ics.jung.graph.DirectedEdge;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.impl.DirectedSparseEdge;
import edu.uci.ics.jung.graph.impl.SparseVertex;
import edu.uci.ics.jung.utils.UserData;
import graph.Keys;
import machinelearning.features.DEEPERFeatureBuilder;
import jung.entities.DependencyVertex;
import jung.entities.Path;
import junit.framework.TestCase;

public class FeatureBuilderTest extends TestCase {
	
	public void testDEEPERFeatureBuilder(){
		//		building tree:
		//			control
		//			/	\
		//		subj	prep_by
		//		/			\
		//	expression		sigE
		//		/
		//	amod
		//		/
		//	ywhE
		Vertex v1 = new DependencyVertex();
		v1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		v1.addUserDatum(Keys.LABEL, "control", UserData.SHARED);
		Vertex v2 = new DependencyVertex();
		v2.addUserDatum(Keys.INDEX, 1, UserData.SHARED);
		v2.addUserDatum(Keys.LABEL, "expression", UserData.SHARED);
		Vertex v3 = new DependencyVertex();
		v3.addUserDatum(Keys.INDEX, 2, UserData.SHARED);
		v3.addUserDatum(Keys.LABEL, "ywhE", UserData.SHARED);
		Vertex v4 = new DependencyVertex();
		v4.addUserDatum(Keys.INDEX, 3, UserData.SHARED);
		v4.addUserDatum(Keys.LABEL, "sigF", UserData.SHARED);
		Path p1 = new Path(v1);
		p1.addVertex(v2);
		p1.addVertex(v3);
		p1.addVertex(v4);
		DirectedEdge e1 = new DirectedSparseEdge(v1,v2);
		e1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e1.addUserDatum(Keys.LABEL, "subj", UserData.SHARED);
		p1.addEdge(e1);		
		DirectedEdge e2 = new DirectedSparseEdge(v2,v3);
		e2.addUserDatum(Keys.INDEX, 1, UserData.SHARED);
		e2.addUserDatum(Keys.LABEL, "amod", UserData.SHARED);
		p1.addEdge(e2);
		DirectedEdge e3 = new DirectedSparseEdge(v1,v4);
		e3.addUserDatum(Keys.INDEX, 2, UserData.SHARED);
		e3.addUserDatum(Keys.LABEL, "prep_by", UserData.SHARED);
		p1.addEdge(e3);
		p1.setLeaf1(v3);
		p1.setLeaf2(v4);
		
		//		building tree:
		//			ROOT
		//			 |
		//			 S
		//		  /		\
		//		VP		NP
		//		|		|
		//		NP		sigF
		//		|
		//	   ywhE
		v1 = new SparseVertex();
		v1.addUserDatum(Keys.INDEX, -1, UserData.SHARED);
		v1.addUserDatum(Keys.LABEL, "ROOT", UserData.SHARED);
		v2 = new SparseVertex();
		v2.addUserDatum(Keys.INDEX, -2, UserData.SHARED);
		v2.addUserDatum(Keys.LABEL, "S", UserData.SHARED);
		v3 = new SparseVertex();
		v3.addUserDatum(Keys.INDEX, -3, UserData.SHARED);
		v3.addUserDatum(Keys.LABEL, "VP", UserData.SHARED);
		v4 = new SparseVertex();
		v4.addUserDatum(Keys.INDEX, -4, UserData.SHARED);
		v4.addUserDatum(Keys.LABEL, "NP", UserData.SHARED);
		Vertex v5 = new SparseVertex();
		v5.addUserDatum(Keys.INDEX, -5, UserData.SHARED);
		v5.addUserDatum(Keys.LABEL, "NP", UserData.SHARED);
		Vertex v6 = new SparseVertex();
		v6.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		v6.addUserDatum(Keys.LABEL, "ywhE", UserData.SHARED);
		Vertex v7 = new SparseVertex();
		v7.addUserDatum(Keys.INDEX, 1, UserData.SHARED);
		v7.addUserDatum(Keys.LABEL, "sigF", UserData.SHARED);
		Path p2 = new Path(v1);
		p2.addVertex(v2);
		p2.addVertex(v3);
		p2.addVertex(v4);
		p2.addVertex(v5);
		p2.addVertex(v6);
		p2.addVertex(v7);
		e1 = new DirectedSparseEdge(v1,v2);
		e1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		p2.addEdge(e1);		
		e2 = new DirectedSparseEdge(v2,v3);
		e2.addUserDatum(Keys.INDEX, 1, UserData.SHARED);
		p2.addEdge(e2);
		e3 = new DirectedSparseEdge(v2,v4);
		e3.addUserDatum(Keys.INDEX, 2, UserData.SHARED);
		p2.addEdge(e3);
		DirectedEdge e4 = new DirectedSparseEdge(v3,v5);
		e4.addUserDatum(Keys.INDEX, 3, UserData.SHARED);
		p2.addEdge(e4);
		DirectedEdge e5 = new DirectedSparseEdge(v5,v6);
		e5.addUserDatum(Keys.INDEX, 4, UserData.SHARED);
		p2.addEdge(e5);
		DirectedEdge e6 = new DirectedSparseEdge(v4,v7);
		e6.addUserDatum(Keys.INDEX, 5, UserData.SHARED);
		p2.addEdge(e6);		
		p2.setLeaf1(v6);
		p2.setLeaf2(v7);
		
		//Interaction i = new Interaction();
		/*i.depPath = p1;
		i.parsePath = p2;
		i.status = InteractionType.REAL;
		
		DEEPERFeatureBuilder b = new DEEPERFeatureBuilder();
		b.defineDatasetStructure();
		Instance inst = b.getFeatureVector(i);
		assertEquals(inst.toString(), "14,40,?,?,?,?,82,?,?,?,?,?,1,1,4,3,1");
		*/
	}
}
