package test;

import deeper.Interaction;
import machinelearning.kernel.SyntacticKernel;
import edu.uci.ics.jung.graph.DirectedEdge;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.impl.DirectedSparseEdge;
import edu.uci.ics.jung.utils.UserData;
import graph.Keys;
import jung.entities.DependencyVertex;
import jung.entities.NewPath;
import jung.entities.Path;
import junit.framework.TestCase;

public class SyntacticKernelTest extends TestCase {

	public void testSyntacticKernel(){
		//building tree:
		//			control
		//			/	\
		//		subj	comp_by
		//		/			\
		//	expression		sigE
		//		/
		//	mod_att
		//		/
		//	ywhE
		Vertex v1 = new DependencyVertex();
		v1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		v1.addUserDatum(Keys.LABEL, "control", UserData.SHARED);
		Vertex v2 = new DependencyVertex();
		v2.addUserDatum(Keys.INDEX, 1, UserData.SHARED);
		v2.addUserDatum(Keys.LABEL, "expression", UserData.SHARED);
		Vertex v3 = new DependencyVertex();
		v3.addUserDatum(Keys.INDEX, 2, UserData.SHARED);
		v3.addUserDatum(Keys.LABEL, "ywhE", UserData.SHARED);
		Vertex v4 = new DependencyVertex();
		v4.addUserDatum(Keys.INDEX, 3, UserData.SHARED);
		v4.addUserDatum(Keys.LABEL, "sigF", UserData.SHARED);
		NewPath p1 = new NewPath(4);
		p1.vertices[0] = v1;
		p1.vertices[1] = v2;
		p1.vertices[2] = v3;
		p1.vertices[3] = v4;
		DirectedEdge e1 = new DirectedSparseEdge(v1,v2);
		e1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e1.addUserDatum(Keys.LABEL, "subj", UserData.SHARED);
		DirectedEdge e2 = new DirectedSparseEdge(v2,v3);
		e2.addUserDatum(Keys.INDEX, 1, UserData.SHARED);
		e2.addUserDatum(Keys.LABEL, "mod_att", UserData.SHARED);
		DirectedEdge e3 = new DirectedSparseEdge(v1,v4);
		e3.addUserDatum(Keys.INDEX, 2, UserData.SHARED);
		e3.addUserDatum(Keys.LABEL, "comp_by", UserData.SHARED);
		p1.edges[0] = e1;		
		p1.edges[1] = e2;
		p1.edges[2] = e3;
		
		//building tree:
		//			abra
		//			/	\
		//		subj	comp_by
		//		/			\
		//	kadabra			shvabra
		//		/
		//	foo
		//	/
		// zanzibabra
		v1 = new DependencyVertex();
		v1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		v1.addUserDatum(Keys.LABEL, "abra", UserData.SHARED);
		v2 = new DependencyVertex();
		v2.addUserDatum(Keys.INDEX, 1, UserData.SHARED);
		v2.addUserDatum(Keys.LABEL, "kadabra", UserData.SHARED);
		v3 = new DependencyVertex();
		v3.addUserDatum(Keys.INDEX, 2, UserData.SHARED);
		v3.addUserDatum(Keys.LABEL, "shvabra", UserData.SHARED);
		v4 = new DependencyVertex();
		v4.addUserDatum(Keys.INDEX, 3, UserData.SHARED);
		v4.addUserDatum(Keys.LABEL, "zanzibabra", UserData.SHARED);
		NewPath p2 = new NewPath(4);
		p2.vertices[0] = v1;
		p2.vertices[1] = v2;
		p2.vertices[2] = v3;
		p2.vertices[3] = v4;
		e1 = new DirectedSparseEdge(v1,v2);
		e1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e1.addUserDatum(Keys.LABEL, "subj", UserData.SHARED);
		e2 = new DirectedSparseEdge(v1,v3);
		e2.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e2.addUserDatum(Keys.LABEL, "comp_by", UserData.SHARED);
		e3 = new DirectedSparseEdge(v2,v4);
		e3.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e3.addUserDatum(Keys.LABEL, "foo", UserData.SHARED);
		p2.edges[0] = e1;		
		p2.edges[1] = e2;
		p2.edges[2] = e3;
		//building tree:
		//		abra
		//		/
		//	subj
		//	/
		// kadabra
		v1 = new DependencyVertex();
		v1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		v1.addUserDatum(Keys.LABEL, "abra", UserData.SHARED);
		v2 = new DependencyVertex();
		v2.addUserDatum(Keys.INDEX, 1, UserData.SHARED);
		v2.addUserDatum(Keys.LABEL, "kadabra", UserData.SHARED);
		NewPath p3 = new NewPath(2);
		p3.vertices[0] = v1;
		p3.vertices[1] = v2;
		e1 = new DirectedSparseEdge(v1,v2);
		e1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e1.addUserDatum(Keys.LABEL, "subj", UserData.SHARED);
		p3.edges[0] = e1;		
		
		
		//	building tree:
		//			abra
		//			/	\
		//		foo		comp_by
		//		/			\
		//	kadabra			shvabra
		//		/
		//	subj
		//	/
		// zanzibabra
		v1 = new DependencyVertex();
		v1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		v1.addUserDatum(Keys.LABEL, "abra", UserData.SHARED);
		v2 = new DependencyVertex();
		v2.addUserDatum(Keys.INDEX, 1, UserData.SHARED);
		v2.addUserDatum(Keys.LABEL, "kadabra", UserData.SHARED);
		v3 = new DependencyVertex();
		v3.addUserDatum(Keys.INDEX, 2, UserData.SHARED);
		v3.addUserDatum(Keys.LABEL, "shvabra", UserData.SHARED);
		v4 = new DependencyVertex();
		v4.addUserDatum(Keys.INDEX, 3, UserData.SHARED);
		v4.addUserDatum(Keys.LABEL, "zanzibabra", UserData.SHARED);
		NewPath p4 = new NewPath(4);
		p4.vertices[0] = v1;
		p4.vertices[1] = v2;
		p4.vertices[2] = v3;
		p4.vertices[3] = v4;
		e1 = new DirectedSparseEdge(v1,v2);
		e1.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e1.addUserDatum(Keys.LABEL, "foo", UserData.SHARED);
		e2 = new DirectedSparseEdge(v1,v3);
		e2.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e2.addUserDatum(Keys.LABEL, "comp_by", UserData.SHARED);
		e3 = new DirectedSparseEdge(v2,v4);
		e3.addUserDatum(Keys.INDEX, 0, UserData.SHARED);
		e3.addUserDatum(Keys.LABEL, "subj", UserData.SHARED);
		p4.edges[0] = e1;		
		p4.edges[1] = e2;
		p4.edges[2] = e3;

		
//		Interaction i1 = new Interaction();
//		Interaction i2 = new Interaction();
//		Interaction i3 = new Interaction();
//		Interaction i4 = new Interaction();
//		i1.depPath = p1;
//		i2.depPath = p2;
//		i3.depPath = p3;
//		i4.depPath = p4;
//		SyntacticKernel k = new SyntacticKernel();
//		double sim1 = k.compute(i1, i1);
//		double sim2 = k.compute(i1, i2);
//		double sim3 = k.compute(i1, i3);
//		double sim4 = k.compute(i1, i4);
//		assertEquals("Equality with itself isn't passed",6.0, sim1);
//		assertEquals("Equality with 'fork' isn't passed",3.0, sim2);
//		assertEquals("Equality with 'chain' isn't passed",1.0, sim3);
//		assertEquals("Equality with interrupted dependencies isn't passed",2.0, sim4);
	}
}
