/* string_index.c
 *
 * Written by Peter Palaga, 2009-04-13
 */

#include "string_index.h"
#include "svm_common.h"
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

STRING_INDEX* create_string_index(string_index_key_t initial_capacity, string_index_key_t expand_by) {
  STRING_INDEX* result;
  result = (STRING_INDEX*)my_malloc(sizeof(STRING_INDEX));
  result->size = 0;
  result->table=(char**)my_malloc(initial_capacity * sizeof(char*));
  result->capacity=initial_capacity;
  result->expand_by=expand_by;

  /* this is to make ZERO_INDEX valid */
  index_for_value("", 0, 0, result);
  return result;
}

char* value_for_index(string_index_key_t index, STRING_INDEX* string_index) {
  if (string_index->size <= index) {
    printf("Index %d is out of string-index size (%d).", index, string_index->size);
    exit(1);
  }
  return string_index->table[index];
}

string_index_key_t index_for_value(char* buffer, size_t len, unsigned char is_terminal, STRING_INDEX* string_index) {
  string_index_key_t i;
  size_t j;
  char* new_value;
  char** new_table;

  /* copy the value */
  new_value = (char*)my_malloc((len + 1) * sizeof(char));
  for (j = 0; j < len; j++) {
    new_value[j] = is_terminal ? tolower(buffer[j]) : toupper(buffer[j]);
  }
  new_value[j] = '\0';

  for (i = 0; i < string_index->size; i++) {
    char* val = string_index->table[i];
    if (strcmp(val, new_value) == 0) {
      return i;
    }
  }

  /* value not found - add it */

  /* ensure capacity */
  if (string_index->size == string_index->capacity) {
    unsigned long new_capacity = string_index->capacity + string_index->expand_by;
    if (new_capacity > string_index_max_capacity) {
      perror ("new_capacity > string_index_max_capacity");
      exit (1);
    }

    new_table = (char**)realloc(string_index->table, new_capacity*sizeof(char*));
    if (new_table == NULL) {
      perror ("Out of memory!\n");
      exit (1);
    }
    string_index->table = new_table;
    string_index->capacity = new_capacity;
  }

  /* add */

  string_index->table[string_index->size] = new_value;

  string_index->size++;

  return string_index->size - 1;

}
