#!/usr/bin/perl -w

use strict;

my $parseFile = shift; #input 1 from user
my $tagFile = shift; #input 2 from user

open (IN1, $parseFile) || die "cannot open $parseFile for reading: $!";
open (IN2, $tagFile) || die "cannot open $tagFile for reading: $!";


my $prot1name;
my $prot2name;

my $MAX_DIST = 1000000;

sub increment_depths {
	my $node_ref = shift;
	my $parent_depth = shift;
	$node_ref->{depth} = $parent_depth+1;
	if (not exists $node_ref->{children}) {
		return;
	}
	for my $ch (@{$node_ref->{children}}) {
		increment_depths($ch,$node_ref->{depth});
	}
}


############
my $output_string = "";
my @relation_array;
my $i = 0;

my $line;
my $line2;

my $tag1;
my $tag2;

while (defined($line = <IN1>)) {
	$_ = $line;
	chomp;
	if (/^\s*$/) {
		if ($output_string ne "") {	
		    $line2 = <IN2>;
			my @tags = split(/\|#\|/,$line2);
			$tag1 = lc($tags[0]);
			$tag2 = lc($tags[1]);		
			paths(@relation_array);
		}
		$output_string = "";
		@relation_array = (); 
		$i = 0;
		next;
	}
	s/\s+$//;
	/^([^\(]*)\((.*)\)$/;
	my $relation = $1;
	my ($part1,$part2) = split(/\,\ /,$2,2);

#print STDERR "$part1 $part2\n";

	$part1 =~ /^(.*)\-(\d+)$/;
	my($t1, $ind1) = split(/-/,$part1,2);
#	my $t1 = $1;
#	my $ind1 = $2;
#	print STDERR $part1,"\n";
	
	$part2 =~ /^(.*)\-(\d+)$/;
	my($t2, $ind2) = split(/-/,$part2,2);
#	my $t2 = $1;
#	my $ind2 = $2;
#	print STDERR $part2,"\n";
	
#	print "\n",$part1, " ", $part2, "\n";
#	print $t1, " ", $ind1, " ",  $t2, " ", $ind2, "\n";
	$output_string .= lc($t1)." ".lc($ind1)."\t".lc($relation)."\t".lc($t2)." ".lc($ind2)."\n";
	$relation_array[$i]=lc($t1)."\t".lc($ind1)."\t".lc($relation)."\t".lc($t2)."\t".lc($ind2);
	$i++;
}
############

sub paths {

	my @rel_array = @_;
	my %nodes;
	my %p_nodes;
	
	$tag1 =~ s/\{|\}|\[|\]|\(|\)|\-|\+|\/|\\|\.|\?|\:|\;|\,|\"|\'|\<|\>|\!|\~|\`|\%|\$|\@|\&|\|/ /gi;
	$tag2 =~ s/\{|\}|\[|\]|\(|\)|\-|\+|\/|\\|\.|\?|\:|\;|\,|\"|\'|\<|\>|\!|\~|\`|\%|\$|\@|\&|\|/ /gi;
	
	my @prot1 = split(/\s+/,$tag1);
	my @prot2 = split(/\s+/,$tag2);
	
	foreach my $r (@rel_array) {
	# term 1 ($t1) at sentence position $ind1 is the parent
    # of term 2 ($t2) at position $ind2 with the relation $rel
		my ($t1, $ind1, $rel, $t2, $ind2) = split(/\t/,$r);

		if (not exists $nodes{$ind1}) {
			$nodes{$ind1}{depth} = 0;
		}
		if (not exists $nodes{$ind2}) {
			$nodes{$ind2}{depth} = 0;
		}
		$nodes{$ind1}{term} = $t1;
		$nodes{$ind2}{term} = $t2;
		$nodes{$ind1}{ind} = $ind1;
		$nodes{$ind2}{ind} = $ind2;
		$nodes{$ind2}{parent} = \%{$nodes{$ind1}};
		$nodes{$ind2}{rel} = $rel;
		push (@{$nodes{$ind1}{children}},\%{$nodes{$ind2}});

		increment_depths(\%{$nodes{$ind2}},$nodes{$ind1}{depth});
		
	}
	
	###determine prot1name and prot2name#####
	my %hash1;
	my %hash2;
	
	$prot1name = "protx1";
	$prot2name = "protx2";
	
	if ($#prot1 == 0){ #prot name is one token
		$prot1name = $prot1[0];
	}
	else{
			foreach my $prot (@prot1){
				$hash1{$prot} = $prot;
			}
			
			foreach my $k (keys %hash1){
				foreach my $n (keys %nodes){
					if ($nodes{$n}{term} eq $k){
						#print "term: ", $k, "\n";
						if (exists $nodes{$n}{parent}) {
							if (exists $hash1{$nodes{$n}{parent}->{term}}){
									if (!exists $nodes{$n}{parent}->{parent}){
										$prot1name=$nodes{$n}{parent}->{term};
									}
									else {
										my $t =  $nodes{$n}{parent}->{parent};
										if (!exists $hash1{$t->{term}}){
											$prot1name = $nodes{$n}{parent}->{term};
										}
									}
									
									#print "parent: ", $k,"\t", $prot2name, "\n";
							}
						}#if exists parent
					}#if
				}#foreach n
			}#foreach k
	}
	######prot2
	if ($#prot2 == 0){ #prot name is one token
		$prot2name = $prot2[0];
	}
	else{
			foreach my $prot (@prot2){
				$hash2{$prot} = $prot;
			}
			
			foreach my $k (keys %hash2){
				foreach my $n (keys %nodes){
					if ($nodes{$n}{term} eq $k){
						#print "term: ", $k, "\n";
						if (exists $nodes{$n}{parent}) {
							if (exists $hash2{$nodes{$n}{parent}->{term}}){
									if (!exists $nodes{$n}{parent}->{parent}){
										$prot2name=$nodes{$n}{parent}->{term};
									}
									else {
										my $t =  $nodes{$n}{parent}->{parent};
										if (!exists $hash2{$t->{term}}){
											$prot2name = $nodes{$n}{parent}->{term};
										}
									}
									
									#print "parent: ", $k,"\t", $prot2name, "\n";
							}
						}#if exists parent
					}#if
				}#foreach n
			}#foreach k
			
			
	}
	if ($prot1name eq "protx1"){
		$prot1name = $prot1[0];
	}
	
	if ($prot2name eq "protx2"){
		$prot2name = $prot2[0];
	}
	
	#print "prot1 prot2: ", $prot1name, "\t", $prot2name, "\n";
	
	foreach my $r (@rel_array) {
	# term 1 ($t1) at sentence position $ind1 is the parent
    # of term 2 ($t2) at position $ind2 with the relation $rel
		my ($t1, $ind1, $rel, $t2, $ind2) = split(/\t/,$r);
		
		if ($t1 eq $prot1name) {
			$p_nodes{1}{$ind1} = 1;
		}
		elsif ($t1 eq $prot2name) {
			$p_nodes{2}{$ind1} = 1;
		}
		if ($t2 eq $prot1name) {
			$p_nodes{1}{$ind2} = 1;
		}
		elsif ($t2 eq $prot2name) {
			$p_nodes{2}{$ind2} = 1;
		}
	}

#
#

	my ($min_dist,$p1_move,$p2_move,$p1_min_dist_to_action,$p2_min_dist_to_action,$depth_diff) =
		($MAX_DIST,0,0,$MAX_DIST,$MAX_DIST,-1);
	my $min_parent;
	my @minpath;

	if (exists $p_nodes{1} && exists $p_nodes{2}) {
		foreach my $ind1 (keys %{$p_nodes{1}}) {
		foreach my $ind2 (keys %{$p_nodes{2}}) {
			my $dist = 0;
			my $p1 = \%{$nodes{$ind1}};
			
			my $p2 = \%{$nodes{$ind2}};
			my ($p1m,$p2m) = (0,0);
			my @p1path;
			my @p2path;
			my $newpath = 0;
			while (1) {
				if ($p1 == $p2) {
					if ($dist < $min_dist) {
						$newpath = 1;
						$min_dist = $dist;
						$p1_move = $p1m;
						$p2_move = $p2m;
						$depth_diff = abs($nodes{$ind1}{depth} - $nodes{$ind2}{depth});
						shift (@p2path);
						push(@p1path,@p2path);
					}
					last;
				}
				elsif ($p1->{depth} > $p2->{depth}) {
					if (exists $p1->{parent}) {
						push(@p1path,$p1->{rel});
						$p1 = $p1->{parent};
						push(@p1path,$p1->{term});
						$p1m++;
					}
					else { last; }
				}
				else {
					if (exists $p2->{parent}) {
						unshift(@p2path,$p2->{rel});
						$p2 = $p2->{parent};
						unshift(@p2path,$p2->{term});
						$p2m++;
					}
					else { last; }
				}
			}
			if ($newpath) {
				@minpath = @p1path;
			}
		}
		}
	}


	if (@minpath) {
		print $minpath[0];
		for my $i (1..$#minpath) {
			print " $minpath[$i]";
		}
		print "\t";
		print $minpath[$#minpath];
		for (my $i=$#minpath-1; $i>=0;--$i) {
			print " $minpath[$i]";
		}
	}

	print "\n";
}
