/************************************************************************/
/*                                                                      */
/*   kernel.h                                                           */
/*                                                                      */
/*   User defined kernel function. Feel free to plug in your own.       */
/*                                                                      */
/*   Copyright: Thorsten Joachims                                       */
/*   Date: 16.12.97                                                     */
/*                                                                      */
/************************************************************************/

/* KERNEL_PARM is defined in svm_common.h The field 'custom' is reserved for */
/* parameters of the user defined kernel. You can also access and use */
/* the parameters of the other kernels. Just replace the line 
             return((double)(1.0)); 
   with your own kernel. */

  /* Example: The following computes the polynomial kernel. sprod_ss
              computes the inner product between two sparse vectors. 

      return((CFLOAT)pow(kernel_parm->coef_lin*sprod_ss(a->words,b->words)
             +kernel_parm->coef_const,(double)kernel_parm->poly_degree)); 
  */

/* If you are implementing a kernel that is not based on a
   feature/value representation, you might want to make use of the
   field "userdefined" in SVECTOR. By default, this field will contain
   whatever string you put behind a # sign in the example file. So, if
   a line in your training file looks like

   -1 1:3 5:6 #abcdefg

   then the SVECTOR field "words" will contain the vector 1:3 5:6, and
   "userdefined" will contain the string "abcdefg". */

double custom_kernel(KERNEL_PARM *kernel_parm, SVECTOR *a, SVECTOR *b) 
     /* plug in you favorite kernel */                          
{
  //edit distance kernel

	//kernel value. it is max of pow(e,-edit(x,y)) of paths between two examples 
	double k=0.0;  
	double sim=0.0;
	float min = 2.0; //initial value for minimum edit distance
	float norm_dist = 0.0; //normalized edit distance

	double e = 2.71828;
	//double e = 5.0;

	//parsing and tokenizing the paths
	char *tokenPtr;
	char *tokenPtr2;
	
	int i=0,j=0;

	int len1=0,len2=0;//lengths of st1 and st2
	
	//MAX_PATHS is the maximum number of paths for an example
	//MAX_PATH_LEN is the maximum length of a path for an example
	
	char paths1[MAX_PATHS][MAX_PATH_LEN]; 
	char paths2[MAX_PATHS][MAX_PATH_LEN];

	//MAX_ALL_PATHS_LEN is the max total length of all the paths for an example
	char s1[MAX_ALL_PATHS_LEN]; 
	char s2[MAX_ALL_PATHS_LEN];

	strcpy(s1,a->userdefined);
	strcpy(s2,b->userdefined);

	tokenPtr = strtok(s1, "\t");

		

	while (tokenPtr != NULL){
		
		strcpy(paths1[len1],tokenPtr);
		len1++;
		tokenPtr = strtok(NULL,"\t");
		
		
	}

	
	tokenPtr2 = strtok(s2,"\t");
	

	while (tokenPtr2 != NULL){
		
		strcpy(paths2[len2],tokenPtr2);
		len2++;
		tokenPtr2 = strtok(NULL,"\t");
		
		
	}

	

	/*for (i=0;i<len1;i++){
		printf("%d %s\t",len1,paths1[i]);
	}
	
	printf("DONE!\n");*/
	
	//calculating minimum edit
	for (i=0;i<len1;i++){
		for (j=0;j<len2;j++){
			//sim = cosine_similarity(paths1[i],paths2[j]);
			norm_dist = edit_distance(paths1[i],paths2[j]);
			//if (dist!=0)
			//	printf("dist = %d\n",dist);
			if (norm_dist<min){
				min=norm_dist;
			}
		}
	}

	//calculating maximum cosine similarity
	/*for (i=0;i<len1;i++){
		for (j=0;j<len2;j++){
			sim = cosine_similarity(paths1[i],paths2[j]);
			//printf("%s %s sim = %f\n",paths1[i],paths2[j],sim);
			if (sim>k){
				k=sim;
			}
		}
	}*/

	//printf("DONE1!\n");
	//printf("k = %f\n",k);
	//k = pow(e,-min*3);
	//similarity 1 - min
	
	//bu satiri kullandik testlerde
	//k = 1 - min;

	k = pow(e,-4.5*min);

	//printf("k = %f\n",k);
	//k=min;
	//printf("DONE8!\n");
	//k = ((double)1)/((double)min/5);
	return((double)(k)); 
}

double cosine_similarity(char *st1, char *st2){

	double sim=0.0;
	char *tokenPtr;
	
	int i=0,j=0,found=0;

	int len1=0,len2=0;//lengths of st1 and st2
	
	int norm_len1=0, norm_len2=0;
	
	char words1[MAX_WORDS][MAX_WORD_LEN]; 
	char words2[MAX_WORDS][MAX_WORD_LEN];

	char s1[MAX_PATH_LEN];
	char s2[MAX_PATH_LEN];

	int freq1[MAX_WORDS];
	int freq2[MAX_WORDS];

	strcpy(s1,st1);
	strcpy(s2,st2);
	
	for (i=0;i<MAX_WORDS;i++)
		freq1[i]=0;
	
	for (i=0;i<MAX_WORDS;i++)
		freq2[i]=0;

	tokenPtr = strtok(s1, " ");
	
	if (tokenPtr!=NULL){
		strcpy(words1[len1],tokenPtr);
		freq1[len1]++;
		len1++;
	}

	tokenPtr = strtok(NULL," ");

	while (tokenPtr != NULL){
		for (i=0;i<len1;i++){
			if (strcmp(words1[i],tokenPtr) ==0){
				freq1[i]++;
				found=1;
				break;
			
			}
		
		}
		
		if (!found){

			strcpy(words1[len1],tokenPtr);
			freq1[len1]++;
			len1++;
			
		}
		
		tokenPtr = strtok(NULL," ");
		found=0;
		
	}


	tokenPtr = strtok(s2," ");
	

	if (tokenPtr!=NULL){
		strcpy(words2[len2],tokenPtr);
		freq2[len2]++;
		len2++;
	}

	tokenPtr = strtok(NULL," ");

	found = 0;

	while (tokenPtr != NULL){

		for (i=0;i<len2;i++){
			if (strcmp(words2[i],tokenPtr) ==0){
				freq2[i]++;
				found=1;
				break;
			
			}
		
		}
		
		if (!found){

			strcpy(words2[len2],tokenPtr);
			freq2[len2]++;
			len2++;
		}
		
		tokenPtr = strtok(NULL," ");
		found=0;
		
	}
	
	//calculating cosine similarity
	for (i=0;i<len1;i++){
		for (j=0;j<len2;j++){
			if (strcmp(words1[i],words2[j]) == 0)
				sim=sim + freq1[i]*freq2[j];
		}
	}
	

	for (i=0;i<len1;i++)
		norm_len1=norm_len1 + freq1[i]*freq1[i];
	
	for (i=0;i<len2;i++)
		norm_len2=norm_len2 + freq2[i]*freq2[i];

	sim = sim/sqrt(norm_len1*norm_len2);
	

	//printf("sim = %f\n",sim);
	return sim;
}


/**********************************************************/
/*Implementation of Word-based Levenshtein (Edit) distance*/
/**********************************************************/

float edit_distance(char *st1,char *st2)
/*Compute word-based levenshtein (edit) distance between st1 and st2*/
{
  
	char *tokenPtr;
	int k,i,j,n,m,cost,distance;
	int d[MAX_ALL_PATHS_LEN];
	
	int len1=0,len2=0;//lengths of st1 and st2
	
	char words1[MAX_WORDS][MAX_WORD_LEN]; 
	char words2[MAX_WORDS][MAX_WORD_LEN];

	char s1[MAX_PATH_LEN];
	char s2[MAX_PATH_LEN];

	float norm_dist;

	strcpy(s1,st1);
	strcpy(s2,st2);
	
	tokenPtr = strtok(s1, " ");
	
	//printf("DONE1!\n");
	
	while (tokenPtr != NULL){
		strcpy(words1[len1],tokenPtr);
		len1++;
		
		tokenPtr = strtok(NULL," ");	
	}


	tokenPtr = strtok(s2," ");
	
	while (tokenPtr != NULL){
		strcpy(words2[len2],tokenPtr);
		len2++;
		
		tokenPtr = strtok(NULL," ");
		
	}	
	
 //Step 1
  
  n=len1;
  m=len2;
  
  //printf("DONE2!\n");

  if(n!=0 && m!=0)
  {
	//printf("DONE3!\n");
    //d=malloc((sizeof(int))*(m+1)*(n+1));
	//printf("DONE4!\n");
    m++;
    n++;
    //Step 2	
    for(k=0;k<n;k++)
	d[k]=k;
    for(k=0;k<m;k++)
      d[k*n]=k;
    //Step 3 and 4	
    for(i=1;i<n;i++)
      for(j=1;j<m;j++)
	  {
        //Step 5
		if (strcmp(words1[i-1],words2[j-1])==0)	
		  //if(st1[i-1]==st2[j-1])
          cost=0;
        else
          cost=1;
        //Step 6			 
        d[j*n+i]=minimum(d[(j-1)*n+i]+1,d[j*n+i-1]+1,d[(j-1)*n+i-1]+cost);
      }
    distance=d[n*m-1];
	//printf("DONE5!\n");
    //free(d);
	//printf("DONE6!\n");
    if (len1>=len2)
		norm_dist = ((float)distance)/(float)(len1);
	else
		norm_dist = ((float)distance)/(float)(len2);
	
	return norm_dist;
  }
  else if (n==0 && m==0){
	//printf("DONE7!\n");
    return 0; //a negative return value means that one or both strings are empty.

  }
  else if (m==0){
	//printf("DONE7!\n");
    return 1; //a negative return value means that one or both strings are empty.

  }
  else if (n==0){
	//printf("DONE7!\n");
    return 1; //a negative return value means that one or both strings are empty.

  }
}

int minimum(int a,int b,int c)
/*Gets the minimum of three values*/
{
  int min=a;
  if(b<min)
    min=b;
  if(c<min)
    min=c;
  return min;
}

